/* 
 * sxDemo.c --
 *
 *	This is a sample window application that demonstrates many
 *	of the features of the Sx package.  Unfortunately, it's so
 *	simple that it doesn't make very good use of the "clientData"
 *	arguments.
 *
 * Copyright (C) 1986, 1988 Regents of the University of California.
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or
 * implied warranty.
 */

#ifndef lint
static char rcsid[] = "$Header: /sprite/src/lib/sx/sxDemo/RCS/sxDemo.c,v 1.6 89/05/11 16:56:46 ouster Exp $ SPRITE (Berkeley)";
#endif not lint

#include <stdio.h>
#include <strings.h>
#include <X11/Xlib.h>
#define Time SpriteTime
#include "sx.h"

/*
 * Forward declarations of callback procedures.
 */

void MakeButtonProc(), ChangeTextProc(), DeleteButtonProc();
void NewButtonMenuProc(), DeleteButtonMenuProc(), EnableMaskProc();
void Notifier1Proc(), Notifier2Proc();
void QuitProc();
void ScrollProc();
void RedisplayProc();
void ButtonProc();
void MakeIconProc(), ChangeIconProc(), DeleteIconProc();

/*
 * Pull-down menu declarations:
 */

Sx_MenuEntry buttons[] = {
    {NULL, "Make Button", NULL, -1, MakeButtonProc, (ClientData) NULL},
    {NULL, "Change Text", NULL, -1, ChangeTextProc, (ClientData) NULL},
    {NULL, "Delete Button", NULL, -1, DeleteButtonProc, (ClientData) NULL}
};

Sx_MenuEntry icons[] = {
    {NULL, "Make Icon", NULL, -1, MakeIconProc, (ClientData) NULL},
    {NULL, "Change Picture", NULL, -1, ChangeIconProc, (ClientData) NULL},
    {NULL, "Delete Icon", NULL, -1, DeleteIconProc, (ClientData) NULL}
};

Sx_MenuEntry menus[] = {
    {"Make new button menu", NULL, NULL, -1, NewButtonMenuProc,
	(ClientData) NULL},
    {"Delete newest button menu", NULL, NULL, -1, DeleteButtonMenuProc,
	(ClientData) NULL},
    {"Disable this menu entry", NULL, NULL, -1, EnableMaskProc,
	(ClientData) ~0x4},
    {"Enable previous menu entry", NULL, NULL, -1, EnableMaskProc,
	(ClientData) ~0}
};

Sx_MenuEntry notifiers[] = {
    {"Test Notifier 1", NULL, NULL, -1, Notifier1Proc,
	(ClientData) NULL},
    {"Test Notifier 2", NULL, NULL, -1, Notifier2Proc,
	(ClientData) NULL}
};

Sx_MenuEntry quit[] = {
    {NULL, "Quit", NULL, -1, QuitProc, (ClientData) NULL}
};

/*
 * Windows that make up SxDemo's application.
 */

static Display *display;

Window w;			/* Top-level window. */
Window title, vertScrollbar, horizScrollbar;
Window menuBar, buttonsMenu, iconsMenu, menusMenu, notifiersMenu, quitMenu;
Window entry;
Window buttonsMenu2 = NULL;
Window button = NULL;
Window icon = NULL;

/*
 * Font and color information:
 */

static XFontStruct *fontPtr;
static unsigned long foreground, background;
static GC gc;

#ifdef notdef
/*
 * Icon information (generated with "bitmap" program):
 */

#define smile_width 32
#define smile_height 32
static short smile_bits[] = {
   0x0000, 0x0000, 0x0000, 0x0000,
   0x0000, 0x0000, 0x0000, 0x0000,
   0x0000, 0x0000, 0x0000, 0x0000,
   0x0180, 0x0180, 0x03c0, 0x03c0,
   0x03c0, 0x03c0, 0x03c0, 0x03c0,
   0x0180, 0x0180, 0x8000, 0x0001,
   0x8000, 0x0001, 0x8004, 0x2001,
   0x8004, 0x2001, 0x800c, 0x3001,
   0x8008, 0x1001, 0x0008, 0x1000,
   0x0018, 0x1800, 0x0010, 0x0800,
   0x0030, 0x0c00, 0x0020, 0x0400,
   0x0060, 0x0600, 0x0040, 0x0200,
   0x01c0, 0x0380, 0x0100, 0x0080,
   0x0300, 0x00c0, 0x0600, 0x0060,
   0x0c00, 0x0030, 0xf800, 0x001f,
   0x0000, 0x0000, 0x0000, 0x0000};

#define frown_width 32
#define frown_height 32
static short frown_bits[] = {
   0x0000, 0x0000, 0x0000, 0x0000,
   0x0000, 0x0000, 0x0000, 0x0000,
   0x0000, 0x0000, 0x0000, 0x0000,
   0x0180, 0x0180, 0x03c0, 0x03c0,
   0x03c0, 0x03c0, 0x03c0, 0x03c0,
   0x0180, 0x0180, 0x0000, 0x0000,
   0x0000, 0x0000, 0x0000, 0x0000,
   0xf000, 0x000f, 0x3800, 0x0038,
   0x0e00, 0x00e0, 0x0300, 0x0180,
   0x0180, 0x0300, 0x00c0, 0x0600,
   0x0040, 0x0400, 0x0060, 0x0c00,
   0x0020, 0x0800, 0x0030, 0x1800,
   0x0010, 0x1000, 0x0010, 0x1000,
   0x0010, 0x1000, 0x0010, 0x1000,
   0x0010, 0x1000, 0x0000, 0x0000,
   0x0000, 0x0000, 0x0000, 0x0000};
#endif
	/* ARGSUSED */

main(argc, argv)
    int argc;
    char **argv;
{
    int height;
    char entryText[100], *defString;
    XColor hard, exact;
    XGCValues gcValues;

    /*
     * Open a connection to the display.
     */

    display = XOpenDisplay((char *) NULL);
/*     XSynchronize(display, TRUE); */
    Sx_SetErrorHandler();
    if (display == NULL) {
	printf("Couldn't open display.  Are you sure X is running?\n");
	return;
    }

    /*
     * Read defaults, and set up color and font information.
     */
    
    foreground = BlackPixel(display, DefaultScreen(display));
    background = WhitePixel(display, DefaultScreen(display));
    if (DisplayCells(display, DefaultScreen(display)) > 2) {
	defString = XGetDefault(display, argv[0], "Foreground");
	if ((defString != NULL) && (XAllocNamedColor(display,
		DefaultColormap(display, DefaultScreen(display)),
		defString, &hard, &exact) == 0)) {
	    foreground = hard.pixel;
	}
    }
    if (DisplayCells(display, DefaultScreen(display)) > 2) {
	defString = XGetDefault(display, argv[0], "Background");
	if ((defString != NULL) && (XAllocNamedColor(display,
		DefaultColormap(display, DefaultScreen(display)),
		defString, &hard, &exact) == 0)) {
	    background = hard.pixel;
	}
    }
    defString = XGetDefault(display, argv[0], "BodyFont");
    if (defString != NULL) {
	fontPtr = XLoadQueryFont(display, defString);
    }
    if (fontPtr == NULL) {
	fontPtr = Sx_GetDefaultFont(display);
    }
    gcValues.foreground = foreground;
    gcValues.background = background;
    gcValues.font = fontPtr->fid;
    gc = XCreateGC(display, RootWindow(display, DefaultScreen(display)),
    	    GCForeground|GCBackground|GCFont, &gcValues);

    /*
     * Create the window and its subwindows.
     */

    w = XCreateSimpleWindow(display,
	    RootWindow(display, DefaultScreen(display)), 100, 100,
	    450, 300, 2, foreground, background);
    (void) Sx_HandlerCreate(display, w, ExposureMask, RedisplayProc,
	    (ClientData) NULL);

    height = Sx_DefaultHeight(display, fontPtr);
    title = Sx_TitleCreate(display, w, SX_TOP, height, 1, fontPtr,
	    foreground, background, foreground, (char *) NULL,
	    "Sx Demonstrator Program", (char *) NULL);

    menuBar = Sx_CreatePacked(display, w, SX_TOP, height, 0, 1,
	    foreground, (Window) 0, background);
    menusMenu = Sx_MenuCreate(display, menuBar, "Menus", 4, menus, fontPtr,
	    foreground, background);
    buttonsMenu = Sx_MenuCreate(display, menuBar, "Buttons", 3, buttons,
	    fontPtr, foreground, background);
    iconsMenu = Sx_MenuCreate(display, menuBar, "Icons", 3, icons, fontPtr,
	    foreground, background);
    notifiersMenu = Sx_MenuCreate(display, menuBar, "Notifiers", 2,
	    notifiers, fontPtr, foreground, background);
    quitMenu = Sx_MenuCreate(display, menuBar, "Quit", 1, quit, fontPtr,
	    foreground, background);
    
    horizScrollbar = Sx_ScrollbarCreate(display, w, SX_BOTTOM, 1,
	    foreground, background, background, background, ScrollProc,
	    (ClientData) NULL);
    Sx_ScrollbarSetRange(display, horizScrollbar, 0.0, .2);
    vertScrollbar = Sx_ScrollbarCreate(display, w, SX_RIGHT, 1,
	    foreground, background, background, background, ScrollProc,
	    (ClientData) NULL);
    Sx_ScrollbarSetRange(display, vertScrollbar, 0.0, .2);

    entry = Sx_CreatePacked(display, w, SX_BOTTOM, height, 0, 1,
	    foreground, (Window) NULL, background);
    Sx_EntryMake(display, entry, w, "Left-click and type here: ", fontPtr,
	    foreground, background, entryText, 100);

    XMapWindow(display, w);
    while (1) {
	XEvent event;

	XNextEvent(display, &event);
	Sx_HandleEvent(&event);
    }
}

/*
 * Callback procedure invoked whenever a mouse button is pressed over
 * the button subwindow:
 */

	/* ARGSUSED */
void
ButtonProc(clientData, eventPtr)
    ClientData clientData;	/* Not used. */
    XEvent *eventPtr;		/* What happened. */
{
    static char *name[] = {"button #1", "button #2", "button #3",
	   "button #4", "button #5"};

    printf("You touched me with the %s mouse button.\n",
	name[eventPtr->xbutton.button - 1]);
}

/*
 * Callback procedure invoked whenever the "Make Button" menu entry is invoked.
 */

	/* ARGSUSED */
void
MakeButtonProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Index of menu entry;  not used. */
    Window window;		/* Not used. */
{
    if (button != NULL) {
	printf("Can't create the button:  it exists already.\n");
	return;
    }
    button = Sx_ButtonCreate(display, w, 100, 200, 0, 0, 1, "Click on me",
	    fontPtr, foreground, background, background, foreground,
	    ButtonProc, (ClientData) NULL);
    XMapWindow(display, button);
}

/*
 * Callback procedure invoked whenever the "Change Text" menu entry is invoked.
 */

	/* ARGSUSED */
void
ChangeTextProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    if (button == NULL) {
	printf("Create the button before changing its text.\n");
	return;
    }
    Sx_ButtonMake(display, button, "New text", fontPtr, foreground, background,
	    background, foreground, ButtonProc, (ClientData) NULL);
}

/*
 * Callback procedure invoked whenever the "Delete Button" menu entry
 * is invoked.
 */

	/* ARGSUSED */
void
DeleteButtonProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    if (button == NULL) {
	printf("Can't delete the button:  it doesn't exist.\n");
	return;
    }
    XDestroyWindow(display, button);
    button = NULL;
}

/*
 * Callback procedure invoked whenever the "Make Icon" menu entry
 * is invoked.
 */

	/* ARGSUSED */
void
MakeIconProc(clientData, index, window)
    ClientData clientData;	/* Not used */
    int index;			/* Not used */
    Window window;		/* Not used */
{
#ifdef undef
    if (icon != NULL) {
	printf("Can't create the icon:  it exists already.\n");
	return;
    }
    icon = Sx_IconCreate(w, 24, 192,
	    		 XMakePixmap ( XStoreBitmap (smile_width, smile_height,
						     	    smile_bits),
			  	       BlackPixel, WhitePixel  ),
 			 smile_width, smile_height, 2,
			 ButtonProc, (ClientData) NULL );
#endif
}

/*
 * Callback procedure invoked whenever the "Change Picture" menu entry
 * is invoked.
 */

	/* ARGSUSED */
void
ChangeIconProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
#ifdef notdef
    if (icon == NULL) {
	printf("Create the icon before changing its picture.\n");
	return;
    }
    Sx_IconMake (icon, XMakePixmap ( XStoreBitmap (frown_width, frown_height,
							 frown_bits),
				    BlackPixel, WhitePixel  ),
		 frown_width, frown_height, ButtonProc, (ClientData) NULL );
#endif
}

/*
 * Callback procedure invoked whenever the "Delete Icon" menu entry
 * is invoked.
 */

	/* ARGSUSED */
void
DeleteIconProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    if (icon == NULL) {
	printf("Can't delete the icon:  it doesn't exist.\n");
	return;
    }
    XDestroyWindow(display, icon);
    icon = NULL;
}

/*
 * Callback procedure invoked whenever the "Make new button menu" menu entry
 * is invoked.
 */

	/* ARGSUSED */
void
NewButtonMenuProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    if (buttonsMenu2 != NULL) {
	printf("You've already got a second buttons menu.  Delete it\n");
	printf("before making a new one.\n");
	return;
    }
    buttonsMenu2 = Sx_MenuCreate(display, menuBar, "2nd Buttons", 3, buttons,
	   fontPtr, foreground, background);
}

/*
 * Callback procedure invoked whenever the "Delete newest button menu"
 * menu entry is invoked.
 */

	/* ARGSUSED */
void
DeleteButtonMenuProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    if (buttonsMenu2 == NULL) {
	printf("Create a new buttons menu before trying to delete.\n");
	return;
    }
    XDestroyWindow(display, buttonsMenu2);
    buttonsMenu2 = NULL;
}

/*
 * Callback procedure invoked by several different menu entries to
 * enable or disable menu entries.
 */

	/* ARGSUSED */
void
EnableMaskProc(clientData, index, window)
    ClientData clientData;	/* New mask. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    int mask;

    /*
     * The clientData value here is actually the new mask for the
     * "Menus" menu.  By passing different clientData values in,
     * the same procedure can handle several different menu entries.
     */
    
    mask = (int) clientData;
    (void) Sx_MenuSetMask(display, menusMenu, mask);
}

/*
 * Callback procedure invoked whenever the "Test Notifier 1"
 * menu entry is invoked.
 */

	/* ARGSUSED */
void
Notifier1Proc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    static char *message = "This notifier is centered in the root window.  \
Notice that it grabs the server, so you can't do anything else until \
you select an option.  Notice also how the Sx package automatically \
chopped one long string up into left-justified lines for display in the \
window.";

    (void) Sx_Notify(display, RootWindow(display, DefaultScreen(display)),
	    -1, -1, 0, message, fontPtr, 0, "OK", NULL);
}

/*
 * Callback procedure invoked whenever the "Test Notifier 2"
 * menu entry is invoked.
 */

	/* ARGSUSED */
void
Notifier2Proc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    static char *message = "This notifier was centered in the sxDemo window, \
but still grabs the server.\n\nIt uses center justification \
for the text.";
    int result;

    result = Sx_Notify(display, w, -1, -1, 0, message, fontPtr,
	    1, "She loves me", "She loves me not", NULL);
    if (result == 0) {
	printf("You selected the \"she loves me\" option.\n");
    } else {
	printf("You selected the \"she loves me not\" option.\n");
    }
}

/*
 * Callback procedure invoked whenever the "Quit"
 * menu entry is invoked.
 */

	/* ARGSUSED */
void
QuitProc(clientData, index, window)
    ClientData clientData;	/* Not used. */
    int index;			/* Not used. */
    Window window;		/* Not used. */
{
    exit(0);
}

/*
 * Callback procedure invoked whenever the user requests scrolling by
 * clicking over one of the scrollbars.
 */

	/* ARGSUSED */
void
ScrollProc(clientData, distance, units, window)
    ClientData clientData;	/* Not used. */
    float distance;		/* Amount of motion. */
    int units;			/* SX_SCROLL_ABSOLUTE or SX_SCROLL_PAGES. */
    Window window;		/* Scrollbar window. */
{
    char *direction;

    if (window == vertScrollbar) {
	direction = "vertical";
    } else {
	direction = "horizontal";
    }

    printf("You just requested %s scrolling", direction);
    if (units == SX_SCROLL_ABSOLUTE) {
	printf(" to center position %.2f in the view.\n", distance);
	distance -= .1;
    } else {
	float top, bottom;
	printf(" by %.2f pages.\n", distance);
	Sx_ScrollbarGetRange(display, window, &top, &bottom);
	distance = (distance * .2) + top;
    }
    printf("I can't actually scroll the window, but I will ");
    printf("update the scrollbar display.\n");
    if (distance < 0) {
	distance = 0.0;
    }
    if (distance > .8) {
	distance = .8;
    }
    Sx_ScrollbarSetRange(display, window, distance, distance + .2);
}

/*
 * Callback procedure that is invoked whenever the window needs to
 * be redrawn.  It just displays some help text.
 */

	/* ARGSUSED */
void
RedisplayProc(clientData, eventPtr)
    ClientData clientData;	/* Not used. */
    XEvent *eventPtr;		/* ExposeWindow event;  not used. */
{
    static char *(msgs[]) = {
	"To try out the Sx features:",
	"(a) press a mouse button over a menu name (above)",
	"    and select a menu entry, or",
	"(b) click a button over a scrollbar, or",
	"(c) resize the window (the packer will re-size",
	"    subwindows).",
	"Information messages will appear on standard",
	"output (not in this window).",
	NULL
    };
    char **stringPtr;
    int y;

    for (y = 50, stringPtr = msgs; *stringPtr != NULL;
	    stringPtr++, y += fontPtr->ascent + fontPtr->descent + 1) {
	XDrawString(display, w, gc, 5, y, *stringPtr,
		strlen(*stringPtr));
    }
}
