/*
 * @(#)display.c	1.2	1/3/85
 *
 * This file contains routines to implement the higher level display
 * driver routines for the SUN Gremlin picture editor.
 *
 * Mark Opperman (opcode@monet.BERKELEY)
 *
 * Modifications by Jim Thompson (james@ernie.berkeley.edu)
 */

#include "gremlin.h"

/* imports from graphics.c */

extern GRArc();
extern GRClear();
extern GRCurve();
extern GRBSpline();
extern GRBezier();
extern GRCurrentSetOn();		/* force display of current set */
extern GRDisplayJustify();
extern GRNewElement(); 
extern GRPutText(); 
extern GRSetCurve(); 
extern GRSetBSpline();
extern GRSetBezier();
extern GRSetLineStyle();
extern GRSetStippleStyle();
extern GRSetTextPos(); 
extern GRStippleFill(); 
extern GRVector();

extern curve_set;			/* true if spline pre-computed */

/* imports from main.c */

extern Window		pix_sw;
extern struct rect	pix_size;
extern PMRec		cset_pm;
extern PMRec		scratch_pm;
extern ELT *cset;
extern CsetOn;
extern SHOWPOINTS;
extern SUN_XORIGIN;
extern SUN_YORIGIN;

/* imports from long*.c */

extern LGShowPoints();

/* locals */

struct rect cset_rect;
struct pm_pos cset_orig;
int minsunx, maxsunx, minsuny, maxsuny;
static struct rect rect_bounding();

/*
 * This routine displays an arbitrary element type 
 * using the parameters stored with the element.  
 * Elements are drawn by Exclusive Oring the screen.
 */
DISScreenAdd(element, mask)
register ELT *element;
int mask;
{
    register POINT *p0, *p1, *p2;
    POINT point;
    register x, y, width, height;
    struct rect elmt_rect;
    register struct rect *r = &elmt_rect;

    if (DBNullelt(element))
	return;

    /* determine bounds for this element */
    minsunx = maxsunx = dbx_to_win(element->ptlist->x);
    minsuny = maxsuny = dby_to_win(element->ptlist->y);

    if (TEXT(element->type)) {
	GRSetTextPos(element->textpt, element->type, element->brushf,
			element->size, element->ptlist, &point);
	GRPutText(element->textpt, element->brushf, element->size, 
			&point);
	if (mask & csetmask)	/* display justification marker */
	    GRDisplayJustify(element);
    }
    else {
	switch (element->type) {
	     case ARC:  
		p1 = element->ptlist;
		p2 = PTNextPoint(p1);
		/* angle is stored in size */
		GRArc(p1, p2, (float) element->size, element->brushf);
		break;
	    case CURVE:  
		if (!curve_set)
		    (void)GRSetCurve(element->ptlist);
		GRCurve(element->brushf);
		curve_set = 0;
		break;
	    case B_SPLINE:  
		if (!curve_set)
		    (void)GRSetBSpline(element->ptlist);
		(void)GRBSpline(element->brushf);
		curve_set = 0;
		break;
	    case BEZIER:  
		if (!curve_set)
		    (void)GRSetBezier(element->ptlist);
		(void)GRBezier(element->brushf);
		curve_set = 0;
		break;
	    case POLYGON:
		if (element->brushf != 0) {	/* bordered polygon */
		    p0 = p1 = element->ptlist;
		    p2 = PTNextPoint(p1);
		    GRSetLineStyle(element->brushf);

		    while (!Nullpoint(p2)) {
			GRVector(p1->x, p1->y, p2->x, p2->y);
			p1 = p2;
			p2 = PTNextPoint(p2);
		    }

		    /* if last point not specified, join end points */
		    if ((p0->x != p1->x) || (p0->y != p1->y))
			    GRVector(p1->x, p1->y, p0->x, p0->y);
		}
		else {		/* unbordered: find min/max */
		    p0 = element->ptlist;

		    while (!Nullpoint(p0)) {
			MINMAX(minsunx, maxsunx, dbx_to_win(p0->x));
			MINMAX(minsuny, maxsuny, dby_to_win(p0->y));
			p0 = PTNextPoint(p0);
		    }
		}

		GRSetStippleStyle(element->size);
		GRStippleFill(element->ptlist);
		break;
	    case VECTOR:  
		p1 = element->ptlist;
		p2 = PTNextPoint(p1);
		GRSetLineStyle(element->brushf);

		while (!Nullpoint(p2)) {
		    GRVector(p1->x, p1->y, p2->x, p2->y);
		    p1 = p2;
		    p2 = PTNextPoint(p2);
		}
		break;
	}
    }

    x = max(minsunx - 8,0);
    y = max(minsuny - 8,0);
    width = maxsunx + 8 - x;
    height = maxsuny + 8 - y;

    if (mask & pixmask) {
	XCopyArea(display, scratch_pm.pm, pix_sw, pix_xor_gc,
		   x, y, (unsigned)width, (unsigned)height, x, y);
    }

    if (mask & csetmask) {
	XCopyArea(display, scratch_pm.pm, cset_pm.pm, pix_xor_gc,
		  x, y, (unsigned)width, (unsigned)height, x, y);
	r->r_left = x;
	r->r_top = y;
	r->r_width = width;
	r->r_height = height;
	cset_rect = rect_bounding(r,&cset_rect);
	cset_orig.x = cset_rect.r_left;
	cset_orig.y = cset_rect.r_top;
    }

    /* clear scratch_pr */
    XSetFunction(display, pix_gc, GXcopy);
    XSetForeground(display, pix_gc, background);
    XFillRectangle(display, scratch_pm.pm, pix_gc, x, y, 
		   (unsigned)width, (unsigned)height);
    XSetForeground(display, pix_gc, foreground);

}  /* end DISScreenAdd */


/*
 * This routine erases an arbitrary element type by redrawing the 
 * element with XOR.  This is the same as drawing the element.
 */
DISScreenErase(element, mask)
register ELT *element;
register mask;
{
    DISScreenAdd(element, mask);
}  /* end ScreenErase */ 

/*
 * Turn off any "showpoints" and erase the current set by XORing
 * the cset pixrect with the picture.  ONLY to be used when the
 * current set will be redrawn immediately afterwards.
 */
fasterase()
{
    CSP();			/* clear show points */

    GRCurrentSetOff();		/* erase current set */
    CsetOn = 1;			/* ON, the current must be redrawn */
    GRClear(csetmask);
}


/*
 * This routine clears the current set pixrect.
 */
DISClearSetDisplay()
{
    register ELT *elist;

    GRCurrentSetOn();

    if (SHOWPOINTS)
	LGShowPoints();

    elist = cset;
    while (!DBNullelt(elist)) {
	if (TEXT(elist->type))		/* turn off text handle */
	    GRDisplayJustify(elist);
	elist = DBNextofSet(elist);
    }

    GRClear(csetmask);
}  /* end DISClearSetDisplay */


DISIcon()
{
}


DISOnTop()
{
}

DISHide()
{
}

static struct rect
rect_bounding(r1, r2)
    struct rect *r1, *r2;
{
    struct rect newRect;
    int		i, j;

    if (r1->r_height == 0) {
	newRect.r_left = r2->r_left;
	newRect.r_top = r2->r_top;
	newRect.r_width = r2->r_width;
	newRect.r_height = r2->r_height;
	return(newRect);
    } else if (r2->r_height == 0) {
	newRect.r_left = r1->r_left;
	newRect.r_top = r1->r_top;
	newRect.r_width = r1->r_width;
	newRect.r_height = r1->r_height;
	return(newRect);
    }
    if (r1->r_left < r2->r_left) {
	newRect.r_left = r1->r_left;
    } else {
	newRect.r_left = r2->r_left;
    }
    if (r1->r_top < r2->r_top) {
	newRect.r_top = r1->r_top;
    } else {
	newRect.r_top = r2->r_top;
    }
    i = r1->r_left + r1->r_width;
    j = r2->r_left + r2->r_width;
    if (i > j) {
	newRect.r_width = i - newRect.r_left;
    } else {
	newRect.r_width = j - newRect.r_left;
    }
    i = r1->r_top + r1->r_height;
    j = r2->r_top + r2->r_height;
    if (i > j) {
	newRect.r_height = i - newRect.r_top;
    } else {
	newRect.r_height = j - newRect.r_top;
    }
    return(newRect);
}
