/*
 * @(#)graphics2.c	1.2	1/3/85
 *
 * Line drawing and polygon fill routines for the SUN Gremlin
 * picture editor.  Line drawing courtesy of dsun.c and polygon
 * fill courtesy of dvar.c.
 *
 * Original by Mark Opperman (opcode@monet.BERKELEY)
 *
 * Modifications by Yih-Farn Chen (yfchen@monet.berkeley.edu)
 *
 */

#include "gremlin.h"

/* imports from main.c */

extern PMRec	scratch_pm;
extern linestyle;		/* type of line (1 - 6) */
extern linemod;			/* line drawing mask (SOLID, DOTTED, ...) */
extern linethickness;		/* 1, 2, 3 */
extern SymbolicLines;		/* TRUE if OK to use XDrawVector for all
				 * lines */
extern SUN_XORIGIN;		/* database x-coord of upper left screen */
extern SUN_YORIGIN;		/* database y-coord of upper left screen */

/* imports from display.c */

extern minsunx, maxsunx, minsuny, maxsuny;

/* imports from menu.c */

extern struct _menu menu[];
extern int HiStipple[];

/* imports from graphics.c */

extern char stipple_patterns[NDISPSTIPPLES][128];
extern rasterlength;
extern bytesperline;
extern nlines;			/* scratch_pm x size */
extern char *fill;		/* zero origin in filling buffer */

/* imports from C */

extern char *calloc();

/* locals */

typedef struct poly {
    struct poly *next;		/* doubly-linked lists of vectors */
    struct poly *prev;
    int param;			/* bressenham line algorithm parameter */
    short dy;			/* delta-y for calculating line */
    short dx;			/* delta-x for calculating line */
    short curry;		/* current y in this vector */
    short endx;			/* where vector ends */
} polyvector;

int stippleType = 0;

/*
 * Vector from (x1, y1) to (x2, y2) using global linemod and linethickness.
 * Parameters in database coordinates system.
 * Always write to scratch_pm.
 * Borrowed from dsun.
 */
GRVector(dbx1, dby1, dbx2, dby2)
float dbx1, dby1, dbx2, dby2;
{
    register x0;	/* starting point and line-walking registers */
    register y0;
    register res;
    register i;		/* line-bleeding carrier */
    int dx;		/* parameters in the line calculations */
    int dy;
    int xinc;
    int yinc;
    int x1;		/* end-point of the line */
    int y1;
    int radius;
    int top;		/* how much to bleed line in "up" (left) direction */
    int bottom;		/* how much to bleed line in "down" (right) direction */
    int stop1;		/* place to stop making circles at start of line */
    int stop2;		/* place to start making circles at end of line */
    int halfstop;	/* distance tween stop1 and stop3 */
    int stop3;		/* midpoint `tween making circles and lines */

    x0 = dbx_to_win(dbx1);	/* convert endpoints to SUN coordinates */
    y0 = dby_to_win(dby1);
    x1 = dbx_to_win(dbx2);
    y1 = dby_to_win(dby2);

    MINMAX(minsunx, maxsunx, x0); 
    MINMAX(minsuny, maxsuny, y0); 
    MINMAX(minsunx, maxsunx, x1);
    MINMAX(minsuny, maxsuny, y1);

    if ((SymbolicLines) || (linestyle == 5 /* NARROW */)) {
	XSetFunction(display, pix_gc, drawOP);
	XDrawLine(display, scratch_pm.pm, pix_gc, x0, y0, x1, y1);
	return;
    }

    xinc = 1;
    yinc = 1;
    if ((dx = x1 - x0) < 0) {
        xinc = -1;
        dx = -dx;
    }
    if ((dy = y1 - y0) < 0) {
        yinc = -1;
        dy = -dy;
    }

    radius = (linethickness - 1) / 2;
    RoundEnd(x0, y0, radius, TRUE);	/* add ends of line */
    RoundEnd(x1, y1, radius, TRUE);	/* (nice and curvy) */

    top = linethickness;	/* increase line thickness if at an angle */
    stop1 = halfstop = 0;
    if ((i = (int) (sqrt ((double) (dx * dx + dy * dy)) + 0.01)) < 2)
	return;			/* small lines are done with endpoints */

    if (dx >= dy) {
	top = (linethickness * i) / dx;
	stop1 = (linethickness * dy) / (i + 1);
	halfstop = (radius * dy) / i;
    } else {
	top = (linethickness * i) / dy;
	stop1 = (linethickness * dx) / (i + 1);
	halfstop = (radius * dx) / i;
    }

    bottom = (top - 1) / 2;
    top = top / 2;

    if (dx >= dy) {
	res = (dy >> 1) - (dx >> 1);
	if (linethickness >= i) {
	    stop1 = stop2 = x0;
	    halfstop = i + 1;
	} else if (xinc == 1) {
	    stop2 = x1 - stop1;
	    stop1 = x0 + stop1;
	    stop3 = x0 + halfstop;
	} else {
	    stop2 = x1 + stop1;
	    stop1 = x0 - stop1;
	    stop3 = x0 - halfstop;
	}

	while (x0 != stop1) {
	    RoundEnd(x0, y0, radius, FALSE);
            if ((x0 & linemod) && (xinc == 1 ? x0 > stop3 : x0 < stop3)) {
		for (i=y0-top; i<=y0+bottom; i++)
		    point(x0, i);
	    }
            if (res >= 0) {
                res -= dx;
                y0 += yinc;
            }
            res += dy;
            x0 += xinc;
	}

        while (x0 != stop2) {
            if (x0 & linemod) {
		for (i=y0-top; i<=y0+bottom; i++)
		    point(x0, i);
	    }
            if (res >= 0) {
                res -= dx;
                y0 += yinc;
            }
            res += dy;
            x0 += xinc;
        } 

	stop3 = x1 + (xinc == 1 ? -halfstop : halfstop);

        while (x0 != x1) {
	    RoundEnd(x0, y0, radius, FALSE);
            if ((x0 &linemod) && (xinc == 1 ? x0 < stop3 : x0 > stop3)) {
		for (i = y0 - top; i <= y0 + bottom; i++)
		    point(x0, i);
	    }
            if (res >= 0) {
                res -= dx;
                y0 += yinc;
            }
            res += dy;
            x0 += xinc;
        } 
    } else {
	res = (dx >> 1) - (dy >> 1);

	if (linethickness >= i) {
	    stop1 = stop2 = y0;
	    halfstop = i + 1;
	} else if (yinc == 1) {
	    stop2 = y1 - stop1;
	    stop1 = y0 + stop1;
	    stop3 = y0 + halfstop;
	} else {
	    stop2 = y1 + stop1;
	    stop1 = y0 - stop1;
	    stop3 = y0 - halfstop;
	}

        while (y0 != stop1) {
	    RoundEnd(x0, y0, radius, FALSE);
            if ((y0 & linemod) && (yinc == 1 ? y0 > stop3 : y0 < stop3)) {
		for (i = x0 - top; i <= x0 + bottom; i++)
		    point(i, y0);
	    }
            if (res >= 0) {
                res -= dy;
                x0 += xinc;
            }
	    res += dx;
            y0 += yinc;
        }

        while (y0 != stop2) {
            if (y0&linemod) {
		for (i=x0-top; i<=x0+bottom; i++)
		    point(i, y0);
	    }
            if (res >= 0) {
                res -= dy;
                x0 += xinc;
            }
	    res += dx;
            y0 += yinc;
        }

	stop3 = y1 + (yinc == 1 ? -halfstop : halfstop);

        while (y0 != y1) {
	    RoundEnd(x0, y0, radius, FALSE);
            if ((y0 & linemod) && (yinc == 1 ? y0 < stop3 : y0 > stop3)) {
		for (i=x0-top; i<=x0+bottom; i++)
		    point(i, y0);
	    }
            if (res >= 0) {
                res -= dy;
                x0 += xinc;
            }
	    res += dx;
            y0 += yinc;
        }
    }
}


/*
 * Plots a filled (if requested) circle of the specified radius
 * centered about (x, y).
 * Coordinates are window relative.
 * Modified from dsun source.
 */
RoundEnd(x, y, radius, filled)
register x;
register y;
int radius, filled;
{
    float xs, ys, epsilon;
    register j, k;


    point(x, y+radius);		/* do the starting point of the circle */

    if (radius < 1)		/* if circle is tiny, quit now */
	return;

    point(x, y-radius);
    if (y-radius < minsuny)
	minsuny = y-radius;

    /* Calculate trajectory of the circle for 1/4 the circumference 
       (while ys is positive) and mirror to get the other three quadrants. */

    xs = 0.0;
    ys = (float) radius;
    epsilon = 1.0 / ys;

    while (ys >= 0) {
	j = (int) (xs + 0.5);
	k = (int) (ys + 0.5);
        if (filled) {		/* fill from center */
	    do {
		point(j+x, k+y);
		point(j+x, y-k);
		point(x-j, k+y);
		point(x-j, y-k);
	    } while (--k >= 0);
        } else {		/* do the perimeter only */
	    point(j+x, k+y);
	    point(j+x, y-k);
	    point(x-j, k+y);
	    point(x-j, y-k);
        }
        xs += epsilon * ys;	/* generate circumference */
        ys -= epsilon * xs;
    }
}  /* end RoundEnd */;


/*
 * Set drawing parameters for filling polygons.
 * Must be called before GRStippleFill().
 */
/*
 * Modifications: polygon style larger than 8, but less than 257
 * is allowed.
 * Date: 12/17/85
 */
GRSetStippleStyle(style)
int style;
{
    if ((style <= 0) || (style > NDISPSTIPPLES)) {
	fprintf(stderr, "GRSetStippleStyle: bad stipple #%d\n", style);
	return;
    }
    stippleType = style - 1;
}

PMRec fill_pms[NDISPSTIPPLES];

/*
 * Fill polygon defined by points in plist using parameters set by
 * previous call to GRSetStippleStyle().
 */
GRStippleFill(plist)
POINT *plist;
{
    POINT	*p1;
    XPoint	xPoints[MAXPOINTS];
    int		npts;
    XGCValues	gcValues;

    for (npts = 0, p1 = plist; !Nullpoint(p1); npts++, p1 = PTNextPoint(p1)) {
	xPoints[npts].x = dbx_to_win(p1->x);
	xPoints[npts].y = dby_to_win(p1->y);
    }

    if (fill_pms[stippleType].pm != 0) {
	gcValues.stipple = fill_pms[stippleType].pm;
	gcValues.function = drawOP;
	gcValues.fill_style = FillStippled;
	XChangeGC(display, pix_gc, GCFunction | GCStipple | GCFillStyle, 
		  &gcValues);
	XFillPolygon(display, scratch_pm.pm, pix_gc, xPoints, npts, Complex,
		     CoordModeOrigin);
	XSetFillStyle(display, pix_gc, FillSolid);
    }
}  /* end GRStippleFill */

point(x, y)
    int	x, y;
{
    XDrawPoint(display, scratch_pm.pm, pix_xor_gc, x, y);
}
