/*
 * @(#)gremlin.h	1.1	10/21/84
 *
 * Standard definitions used by the SUN Gremlin picture editor.
 *
 * Original by  Mark Opperman 
 *
 * Modifications by Yih-Farn Chen (yfchen@monet.berkeley.edu)
 * Modifications by Jim Thompson (james@ernie.berkeley.edu)
 *
 */

#ifndef _GREMLIN
#define _GREMLIN

#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>

#define nullpt  -1

#define MAXPOINTS 200

#define pixmask     1
#define csetmask    2
#define scratchmask 4

#define BOTLEFT 0
#define BOTRIGHT 1
#define CENTCENT 2
#define VECTOR 3
#define ARC 4			/* arc or circle */
#define CURVE 5			/* interpolated spline */
#define POLYGON 6
#define B_SPLINE 7		/* B-spline */
#define BEZIER 8		/* Third-degree Bezier curve */
#define TOPLEFT 10
#define TOPCENT 11
#define TOPRIGHT 12
#define CENTLEFT 13
#define CENTRIGHT 14
#define BOTCENT 15
#define TEXT(t) ( ((t) <= CENTCENT) || ((t) >= TOPLEFT) )
/* WARNING * WARNING * WARNING * WARNING * WARNING * WARNING * WARNING 
 *    The above (TEXT) test is dependent on the relative values of the 
 *    constants and will have to change if these values change or if new
 *    commands are added with value greater than BOTCENT
 */


#define RFONT 0
#define IFONT 1
#define BFONT 2
#define SFONT 3

#define NOADJ 0
#define HORZ  1
#define VERT  2
#define MAN   3

#define NUSER 4
#define NFONTS 4
#define NBRUSHES 6
#define NSIZES 4
#define NMENUSTIPPLES 8      /* stipple menus */
#define NDISPSTIPPLES 256    /* displayable stipples */
#define NJUSTS 9
#define JUSTMODES 17

#define ADD 1
#define DELETE 2
#define MOD 3

#define Delimiter(c) ((c == '\0') || (c == ' ') || (c == '\t'))

#define MINMAX(min, max, val)   if (val < min) \
				    min = val; \
				else if (val > max) \
				    max = val;

/* The following represents the maximum distance a point may be from another
 * and still be affected by gravity.  The distance is represented as the
 * square of the number of pixels (32) of the actual distance.  
 */

#define MAXDIST  32
#define MAXGDIST 1024	/* 32 * 32 */

typedef struct pm_pos {
    int x, y;
} pm_pos;

typedef struct rect {
    int        r_left, r_top;
    unsigned   r_width, r_height;
} Rect;

typedef struct point {
	    float x, y;
	    struct point *nextpt;
        } POINT;

typedef struct elmt {
	    int type, brushf, size, textlength;
	    char *textpt;
	    POINT *ptlist;
	    struct rect outline;
	    struct elmt *nextelt, *setnext;
        } ELT;

typedef struct unlt {
	    int action;
	    ELT *(*dbase), *oldelt, *newelt;
	    struct unlt *nextun;
        } UNELT;


/* 
 * database macros
 */

#define DBInit()	  ((ELT *) NULL)
#define DBNextElt(elt)    ((elt)->nextelt)
#define DBNextofSet(elt)  ((elt)->setnext)
#define DBNullelt(elt)    ((elt) == NULL)

#define PTInit()	  ((POINT *) NULL)
#define PTNextPoint(pt)   ((pt)->nextpt)
#define Nullpoint(pt)     ((pt) == (POINT *) NULL)
 

/* sun version parameters */

/* icon display parameters */

#define ICON_SIZE  16
#define ICON_BORDER  8

#define MENU_TOP  0	/* 17 */
#define MENU_LEFT  0
#define MENU_RIGHT  104
#define MENU_BOTTOM  1000
#define MENU_COLUMNS  4

#define MENU_X  (ICON_SIZE + ICON_BORDER)
#define MENU_Y  (ICON_SIZE + ICON_BORDER)

/* subwindow stuff */

#define	BORDER_WIDTH	2
#define TEXTSW_HEIGHT 33
#define MENUSW_WIDTH 104
#define PIXSW_WIDTH 512
#define PIXSW_HEIGHT 512

#define stipple1_pm  white_pm
#define stipple2_pm  gray_pm
#define stipple3_pm  _50_pm
#define stipple4_pm  black_pm

#define is_stipple(pm)  (((pm) == &stipple1_pm) || ((pm) == &stipple2_pm) || \
			 ((pm) == &stipple3_pm) || ((pm) == &stipple4_pm) || \
			 ((pm) == &stipple5_pm) || ((pm) == &stipple6_pm) || \
			 ((pm) == &stipple7_pm) || ((pm) == &stipple8_pm))

#define winx_to_db(x) ((float) ((x) + SUN_XORIGIN))
#define winy_to_db(y) ((float) (SUN_YORIGIN - (y)))
#define dbx_to_win(x) (((int) (x)) - SUN_XORIGIN)
#define dby_to_win(y) (SUN_YORIGIN  - ((int) (y)))

typedef struct inter_functions {	/* functions when interactive */
	    int (*ms_left)();			/* mouse left button */
	    int (*ms_center)();			/* mouse center button */
	    int (*ms_right)();			/* mouse right button */
	    int (*ms_move)();			/* mouse movement */
	    int (*win_exit)();			/* window exit */
	} INTER_FUNC;


typedef struct PMRec {
    unsigned	width, height;
    Pixmap	pm;
} PMRec;

struct _menu {
	    int menu_x;				/* icon x coordinate */
	    int menu_y;				/* icon y coordinate */
	    PMRec *pm_ptr;			/* icon pixmap pointer */
	    int (*menu_function)();		/* left button function */
	    int (*menu_modify)();		/* middle button function */
	    int (*menu_help)();			/* right button function */
	    INTER_FUNC *inter_act;		/* actions for interactive */
	};

#define TEXT_BASELINE  10
#define TEXT_BUFMAX  128

#ifndef TRUE
#define TRUE (1)
#define FALSE (0)
#endif

/*
 * New stuff for X11 port.
 */
extern	Display		*display;
extern	Window		root_window;
extern	Window		main_window;
extern	unsigned long	foreground;
extern	unsigned long	background;
extern	unsigned int	root_height;
extern	unsigned int	root_width;
extern	unsigned int	root_depth;
extern	GC		root_gc;
extern	GC		title_gc;
extern	GC		text_gc;
extern	GC		text_xor_gc;
extern	GC		menu_gc;
extern	GC		menu_xor_gc;
extern	GC		pix_gc;
extern	GC		pix_xor_gc;
extern	GC		pixmap_gc;
extern	int		buttonUp;
extern  PMRec		grid32_pm;
extern	unsigned int	drawOP;
extern	unsigned int	menuEraseOP;

#endif
