/*
 * @(#)help1.c	1.2	1/3/85
 *
 * Routines to provide help screens for the SUN Gremlin picture editor.
 *
 * Mark Opperman (opcode@monet.BERKELEY)
 *
 * Modifications by Jim Thompson (james@ernie.berkeley.edu)
 */

#include "gremlin.h"
#include "icondata.h"


/* imports from help.c */
extern help_screen();


static char curve_HELP[] = "\
A curve is determined by a number of points distributed along\n\
its trajectory.  There are three types of curves with different\n\
characteristics:\n\
   B-Splines (default) lie within the convex hull formed by the points,\n\
and is smoother than the Interpolated curve.  The curve passes through the\n\
first and last points, unless they are the same. In that case, a\n\
smooth closed curve is drawn which may not pass through the end points.\n\
   Interpolated Curves will always pass through all supplied\n\
points.  If the first and last points are the same, a smooth closed\n\
figure will be drawn.\n\
   Bezier curves also lie within the convex hull, but always pass\n\
through the end points.\n\
\n\
Curves are drawn using the current brush style and current curve type,\n\
which is shown on the curve icon.  Change curve types by holding down\n\
the shift key while selecting the icon with the middle mouse button.\n\
\n\
With the middle mouse button, vectors, curves,  and polygons in the\n\
current set are modified to become curves of the current type.\n\
\n\
When interactive, the left button specifies the points of the\n\
curve. Double-click on the last point to mark the end of the\n\
curve.  The points are connected with straight lines, which\n\
become curved when the double-click occurs.  The center button\n\
removes the most recent point specified. If there is no drawing\n\
in progress, the center button \"undoes\" the most\n\
recent action which changed the picture.";

curve_help()
{
    help_screen(&curve_pm, "Draw Curve ('b')", curve_HELP);
}

static char justify_HELP[] = "\
Select text justification by moving marker to one of nine\n\
positioning points within the JUSTIFY icon and pressing the\n\
left mouse button.\n\
\n\
Modify justification of text in the current set by moving the\n\
marker as above and then pressing the middle mouse button.\n\
\n\
When text is displayed in the current set, its justification\n\
mode is indicated by a small dot.";

justify_help()
{
    help_screen(&justify_pm, "Text Justification", justify_HELP);
}


static char size1_HELP[] = "\n\
Set the default font size to one with the left mouse button.\n\
\n\
Modify text in the current set to size one with the middle\n\
mouse button.";

size1_help()
{
    help_screen(&size1_pm, "Set Text Size One", size1_HELP);
}


static char roman_HELP[] = "\
Set the default font to Roman with the left mouse button.\n\
\n\
Modify text in the current set to Roman font with the middle\n\
mouse button.";

roman_help()
{
    help_screen(&roman_pm, "Set Roman Text Font", roman_HELP);
}


static char scale_HELP[] = "\
This icon invokes a pop-up menu of three commands:\n\
Scale XY, Scale X, and Scale Y.\n\
Three points are used to define a transformation.\n\
The scaling ratio is defined by the ratio of the distances\n\
between the first and second and the first and third points.\n\
The scaling can be done on X coordinates only, on Y coordinates only,\n\
or it can be done on both coordinates of the points which define\n\
the current set.\n\
The short command 's' is defined as \"Scale XY\".";

scale_help()
{
    help_screen(&scale_pm, "Scale Current Set ('s')", scale_HELP);
}


static char move_HELP[] = "\
Translation uses two points to define a relative movement.\n\
The current set is translated through the relative distance\n\
between the two points.\n\
\n\
When interactive, press the left button at any point, and drag\n\
the current set to the new location with the button down.  The\n\
center button \"undoes\" the last action which changed the picture.";

move_help()
{
    help_screen(&move_pm, "Translate Current Set ('t')", move_HELP);
}


static char hmirror_HELP[] = "\
Mirroring uses one point to define a transformation.\n\
The current set is reflected about the horizontal line\n\
containing the point.";

hmirror_help()
{
    help_screen(&hmirror_pm, "Horizontal Mirror", hmirror_HELP);
}


static char vmirror_HELP[] = "\
Mirroring uses one point to define a transformation.\n\
The current set is reflected about the vertical line\n\
containing the point.";

vmirror_help()
{
    help_screen(&vmirror_pm, "Vertical Mirror", vmirror_HELP);
}


static char include_HELP[] = "\
The current set is selected by points.  Using the left mouse\n\
button, the current set will include ONLY those elements near\n\
the points.  With the middle mouse button, those elements near\n\
the points will be ADDED to the current set.\n\
\n\
If no points are selected, the entire picture becomes the\n\
current set.";

include_help()
{
    help_screen(&include_pm, "Define Current Set ('d')", include_HELP);
}


static char put_HELP[] = "\
The current set is copied into the specified set buffer for\n\
possible later retrieval.  An optional positioning point may\n\
be specified for use in positioning the set when it is later\n\
copied into a picture.  With no positioning point specified,\n\
a point is selected from among the reference points of the\n\
current set.";

put1_help()
{
    help_screen(&put1_pm, "Save Current Set in Buffer One ('1')", put_HELP);
}


put2_help()
{
    help_screen(&put2_pm, "Save Current Set in Buffer Two ('2')", put_HELP);
}


put3_help()
{
    help_screen(&put3_pm, "Save Current Set in Buffer Three ('3')", put_HELP);
}


put4_help()
{
    help_screen(&put4_pm, "Save Current Set in Buffer Four ('4')", put_HELP);
}


static char horizontal_HELP[] = "\
Horizontal adjustment forces each point laid down to lie on\n\
a horizontal line from the previous point.  The left mouse\n\
button toggles this drawing mode.";

horizontal_help()
{
    help_screen(&horizontal_pm, "Horizontal Adjustment", horizontal_HELP);
}


static char vertical_HELP[] = "\
Vertical adjustment forces each point laid down to lie on a\n\
vertical line from the previous point.  The left mouse button\n\
toggles this drawing mode.";

vertical_help()
{
    help_screen(&vertical_pm, "Vertical Adjustment", vertical_HELP);
}


static char stipple_HELP[] = "\
Select the stipple pattern used for drawing polygons with\n\
the left mouse button.\n\
\n\
Modify polygons in the current set to the specified stipple\n\
pattern with the middle mouse button.";

stipple1_help()
{
    help_screen(&white_pm, "Set Stipple Pattern One", stipple_HELP);
}


stipple2_help()
{
    help_screen(&gray_pm, "Set Stipple Pattern Two", stipple_HELP);
}


stipple3_help()
{
    help_screen(&_50_pm, "Set Stipple Pattern Three", stipple_HELP);
}


stipple4_help()
{
    help_screen(&black_pm, "Set Stipple Pattern Four", stipple_HELP);
}


stipple5_help()
{
    help_screen(&stipple5_pm, "Set Stipple Pattern Five", stipple_HELP);
}


stipple6_help()
{
    help_screen(&stipple6_pm, "Set Stipple Pattern Six", stipple_HELP);
}


stipple7_help()
{
    help_screen(&stipple7_pm, "Set Stipple Pattern Seven", stipple_HELP);
}


stipple8_help()
{
    help_screen(&stipple8_pm, "Set Stipple Pattern Eight", stipple_HELP);
}


static char size2_HELP[] = "\n\
Set the default font size to two with the left mouse button.\n\
\n\
Modify text in the current set to size two with the middle\n\
mouse button.";

size2_help()
{
    help_screen(&size2_pm, "Set Text Size Two", size2_HELP);
}


static char italics_HELP[] = "\
Set the default font to Italics with the left mouse button.\n\
\n\
Modify text in the current set to Italics font with the middle\n\
mouse button.";

italics_help()
{
    help_screen(&italics_pm, "Set Italics Text Font", italics_HELP);
}


static char copy_HELP[] = "\
Copying uses two or more points.  A copy of the current set\n\
is made and translated by a relative distance between the\n\
first and each additional point.  The last copy becomes\n\
the new current set.\n\
\n\
When interactive, press the left button at any point, and drag\n\
a copy of the current set to the new location with the button\n\
down.  The center button \"undoes\" the previous action which\n\
modified the picture.";

copy_help()
{
    help_screen(&copy_pm, "Copy Current Set ('c')", copy_HELP);
}


static char erase_HELP[] = "\
The current set is erased.  See also the undo command.";

erase_help()
{
    help_screen(&erase_pm, "Erase Current Set ('e')", erase_HELP);
}


static char movepoint_HELP[] = "\
This command uses one or more points.  The element of the\n\
current set which contains the point closest to the first of\n\
these points is redrawn with that point replaced by the\n\
remaining points, or deleted if there is only one point.\n\
\n\
When interactive, press the left button near the point in the\n\
current set which you want to move, then move it with the button\n\
down.  To delete a point, 'click' the left button near the point\n\
without moving the mouse.  The center button 'undoes' the most \n\
recent change to the picture.";

movepoint_help()
{
    help_screen(&movepoint_pm, "Move Point", movepoint_HELP);
}


static char rotate_HELP[] = "\
Three points are used to define a rotation.  The rotation is\n\
performed relative to the first point, through an angle formed\n\
by the lines between points one and two and points one and\n\
three, respectively.";

rotate_help()
{
    help_screen(&rotate_pm, "Rotate Current Set ('r')", rotate_HELP);
}




static char boxinc_HELP[] = "\
Two points define a rectangular area (the endpoints of the diagonal\n\
of the rectangle).\n\
\n\
With the left mouse button, all elements contained within the\n\
the rectangle become the current set.\n\
\n\
With the middle mouse button, those same elements are ADDED\n\
to the current set.\n\
\n\
When interactive, the left button selects the current set, and\n\
the center button ADDS to the current set.  In both cases,\n\
\"clicking\" the button selects the element near the cursor\n\
(like Define Current Set).  Press the button and move the mouse\n\
with the button down to select all elements within the resulting\n\
rectangle.\n\
\n\
The SHIFT key can be used to select the interactive mode of defining the\n\
current set from any other mode.  When the SHIFT key is released, the\n\
previous mode resumes.";

boxinc_help()
{
    help_screen(&boxinc_pm, "Select Area for Current Set ('f')", boxinc_HELP);
}


static char manhattan_HELP[] = "\
Manhattan adjustment forces each point laid down to be either\n\
directly horizontal or vertical with respect to the previous\n\
point, whichever it is closer to.  The left mouse button\n\
toggles this drawing mode.";

manhattan_help()
{
    help_screen(&horvert_pm, "Manhattan Adjustment ('z')", manhattan_HELP);
}


static char gravity_HELP[] = "\
This command toggles each time it is selected.  When on,\n\
gravity forces a point to coincide with the nearest existing\n\
point or reference point.  It will only take affect, however,\n\
if the point is near enough to an element to be gravitiated\n\
to it.\n\
Gravity is in affect while interactive, but its influence is only\n\
visible when the mouse is still.  Therefore, pausing while doing\n\
interactive drawing will cause the feedback to \"gravitate\" to\n\
its final position.";

gravity_help()
{
    help_screen(&gravity_pm, "Set Gravity ('g')", gravity_HELP);
}


static char size3_HELP[] = "\n\
Set the default font size to three with the left mouse button.\n\
\n\
Modify text in the current set to size three with the middle\n\
mouse button.";

size3_help()
{
    help_screen(&size3_pm, "Set Text Size Three", size3_HELP);
}


static char bold_HELP[] = "\
Set the default font to Bold with the left mouse button.\n\
\n\
Modify text in the current set to Bold font with the middle\n\
mouse button.";

bold_help()
{
    help_screen(&bold_pm, "Set Bold Text Font", bold_HELP);
}


static char brush_HELP[] = "\
The left mouse button sets the current brush style.\n\
\n\
The middle mouse button modifies all elements in the current\n\
set (except text) to the selected brush style.";

brush1_help()
{
    help_screen(&dotted_pm, "Set Dotted Line Style", brush_HELP);
}


brush2_help()
{
    help_screen(&broken_pm, "Set Broken Line Style", brush_HELP);
}


brush3_help()
{
    help_screen(&thick_pm, "Set Thick Line Style", brush_HELP);
}


brush4_help()
{
    help_screen(&dashed_pm, "Set Dashed Line Style", brush_HELP);
}


brush5_help()
{
    help_screen(&narrow_pm, "Set Narrow Line Style", brush_HELP);
}


brush6_help()
{
    help_screen(&medium_pm, "Set Medium Line Style", brush_HELP);
}


static char arrow_HELP[] = "\
There are three styles of arrows; an arrowhead (the default),\n\
an arrow (a line with arrowhead attached), and a two-headed\n\
arrow. The style of arrow is toggled with the center button\n\
on the menu icon.\n\
\n\
This command requires two points. The arrow is drawn from the\n\
first point to the second.\n\
\n\
When interactive, press the left button at the \"tail\" of the\n\
arrow and drag the head to its position with the button down.\n\
The center button \"undoes\" the last command which modified\n\
the picture.";

arrow_help()
{
    help_screen(&arrow_pm, "Draw Arrowhead ('w')", arrow_HELP);
}


static char text_HELP[] = "\
Text is positioned using one or two points.  If two points\n\
are used, the text is positioned relative to their locus.\n\
The text specified in the text subwindow is displayed using\n\
the current font, size and justification.\n\
\n\
See the text subwindow help display for an explanation of\n\
quick text entry.";

text_help()
{
    help_screen(&text_pm, "Display Text", text_HELP);
}


static char misc_HELP[] = "\
The left mouse button invokes a pop-up menu of infrequently used\n\
commands: Clear Points, Show Points, Gripe (rarely used)\n\
and Point.\n\
\n\
Clear Points will clear all positioning points and reference\n\
points from the display.\n\
\n\
Show Points will display the reference points of those\n\
elements in the current set.\n\
\n\
Flash On/Off will turn on/off the flashing of the current set.\n\
\n\
Gripe displays a message indicating the mail address of\n\
the current Gremlin maintainer.\n\
\n\
Point can be used to lay down a point at a specific location.\n\
The coordinates of the point must first be entered in the\n\
text subwindow.\n\
\n\
The middle button invokes a pop-up menu to select a stipple pattern\n\
to be replaced.  The font index of the new stipple pattern should\n\
be specified in the text sunbwindow before invocation." ;

misc_help()
{
    help_screen(&misc_pm, "Miscellaneous Commands", misc_HELP);
}


static char get_HELP[] = "\
This command retrieves a set from the specified buffer and\n\
copies it into the picture.  At least one point must be\n\
specified, indicating the position(s) in the picture where\n\
the set is to be copied.";

get1_help()
{
    help_screen(&get1_pm, "Add Buffer One To Picture", get_HELP);
}


get3_help()
{
    help_screen(&get3_pm, "Add Buffer Three To Picture", get_HELP);
}


get2_help()
{
    help_screen(&get2_pm, "Add Buffer Two To Picture", get_HELP);
}


get4_help()
{
    help_screen(&get4_pm, "Add Buffer Four To Picture", get_HELP);
}


static char linestyle_HELP[] = "\
This command toggles symbolic line display.  When highlighted,\n\
this icon indicates that elements will be drawn using their\n\
true line style (or brush).  When not highlighted, all\n\
elements are drawn using the narrow brush, decreasing display\n\
times.";

linestyle_help()
{
    help_screen(&linestyle_pm, "Toggle Line Style Mode", linestyle_HELP);
}


static char align_HELP[] = "\
This command forces points to be aligned on pixel boundaries\n\
as specified in the icon.  Alignment occurs in powers of two.\n\
To select the next alignment value use the left mouse button,\n\
and to select the previous value use the middle mouse button.\n\
\n\
The precedence of point positioning modifiers is as follows:\n\
gravity will override alignment, and adjustment (vertical,\n\
horizontal or manhattan) will be applied to the point to which\n\
the point has been gravitated.";

align_help()
{
    help_screen(&align_pm, "Set Point Alignment", align_HELP);
}

static char size4_HELP[] = "\n\
Set the default font size to four with the left mouse button.\n\
\n\
Modify text in the current set to size four with the middle\n\
mouse button.";

size4_help()
{
    help_screen(&size4_pm, "Set Text Size Four", size4_HELP);
}

static char filecabinet_HELP[] = "\
This icon produces a pop-up menu to select commands for manipulating\n\
files: Edit, Path, Read, Write and Save Set. With each command,\n\
a file or path name should be specified in the text subwindow before\n\
invocation. \n\
\n\
The Edit command causes a new file to be opened for editing.  Picture\n\
sets saved in the buffers are preserved across edits.  The file name\n\
will be displayed in the Gremlin tool border.  \n\
\n\
The Read command is used to add elements from the specified file into\n\
the current picture.  The new elements become the current set.  A\n\
point may be specified to position the file in the picture.\n\
\n\
The Write command saves the entire picture in a file.  If no file name\n\
is specified in the text subwindow, the current Edit file name is\n\
used.  An optional point may be specified to aid in Reading the\n\
picture later.  \n\
\n\
The Save Set command is similar to the Write command with two exceptions:\n\
only the current set is written, and a file name must be specified.\n\
\n\
The Path command is used to set the directory search path for the Edit\n\
and Read commands.  Directory names should be separated by colons and\n\
may include the ~ notation.  If a file cannot be found using any of\n\
the paths, a final check will be made in the Gremlin library,\n\
/usr/local/lib/gremlin.  \n\
\n\
The Directory command is used to set the current working directory,\n\
which may include the ~ notation.  With no argument, Directory displays\n\
the name of the current directory";

filecabinet_help()
{
    help_screen(&filecabinet_pm, "File Commands", filecabinet_HELP);
}


static char special_HELP[] = "\
Set the default font to Special with the left mouse button.\n\
\n\
Modify text in the current set to Special font with the middle\n\
mouse button.";

special_help()
{
    help_screen(&special_pm, "Set Special Text Font", special_HELP);
}


static char arc_HELP[] = "\
This command requires two points to draw a full circle or\n\
three points to draw an arc.  The first point determines the\n\
center of a circle.  The second is a point on the circle,\n\
thus defining the radius.  An optional third point determines\n\
a counter-clockwise angle from the second point which is the\n\
extent of the arc.\n\
\n\
When interactive, press the left button at the center on the\n\
circle, and drag the perimeter with the button down.  There\n\
is currently way to interactively draw an arc.  The center\n\
button \"undoes\" the most recent action which modified the\n\
picture.";

arc_help()
{
    help_screen(&arc_pm, "Draw Circle or Arc ('a')", arc_HELP);
}


static char vector_HELP[] = "\
With the left mouse button, a line is drawn connecting each\n\
of the points layed down in order.  The current brush (narrow,\n\
dotted, etc) is used.\n\
\n\
With the middle mouse button, curves and polygons in the\n\
current set are modified to become vectors.\n\
\n\
When interactive, the left button specifies points of the\n\
vectors, with a double-click on the last point to signal the\n\
end of the sequence of points.  The points are connected with\n\
lines, which assume the proper brush when the double-click\n\
occurs.  The lines may be drawn with the left button up or down;\n\
all except the first point are specified when the button goes UP.\n\
The center button removes the most recent point plotted.  If there\n\
is no drawing in progress, the center button \"undoes\" the most\n\
recent action which changed the picture.";

vector_help()
{
    help_screen(&vector_pm, "Draw Vector ('v')", vector_HELP);
}


static char box_HELP[] = "\
Two points are used to define the endpoints of the diagonal\n\
of a rectangle.  A box is drawn in the current brush which\n\
forms the rectangle.\n\
\n\
When interactive, press the left button at any corner of the\n\
rectangle, and drag the opposite corner with the button down.\n\
The center button \"undoes\" the most recent action which changed\n\
the picture.";

box_help()
{
    help_screen(&box_pm, "Draw Box ('x')", box_HELP);
}


static char grid_HELP[] = "\
This command toggles the display of a grid used to aid in\n\
laying down points.  The grid is displayed on 32 pixel\n\
boundaries.";

grid_help()
{
    help_screen(&grid_pm, "Toggle Grid Display ('q')", grid_HELP);
}


static char littlepoint_HELP[] = "\
This command toggles the style of points displayed.  When the\n\
icon is highlighted, points are indicated by a small circle\n\
and are numbered from zero.  When the icon is unhighlighted,\n\
points are displayed as a small diamond.";

littlepoint_help()
{
    help_screen(&littlepoint_pm, "Toggle Point Style", littlepoint_HELP);
}


static char undo_HELP[] = "\
This command undoes the last command which modified the\n\
picture contents.";

undo_help()
{
    help_screen(&undo_pm, "Undo Last Command", undo_HELP);
}


static char pan_HELP[] = "\
With the left mouse button this command requires one point.\n\
The entire picture and current set are translated such that\n\
the specified point is located at the center of the display.\n\
\n\
With the middle mouse button no point is required.  The\n\
picture is translated such that its absolute center is\n\
brought to the center of the display.";

pan_help()
{
    help_screen(&pan_pm, "Panning", pan_HELP);
}

static char polygon_HELP[] = "\
Polygons can be drawn either bordered or unbordered.  At\n\
least three points are required to draw a polygon.  If the\n\
first and the last points are not the same, that line\n\
segment will be added automatically.  With the left mouse\n\
button, a filled polygon will be added to the display in the\n\
current stipple style.  If a border is to be drawn, this\n\
will be added in the current line style.\n\
\n\
The middle mouse button is used to modify curves, vectors and\n\
other polygons in the current set to be polygons of the\n\
selected type (bordered or unbordered).\n\
\n\
When interactive, the left button specifies the vertices of the\n\
polygon, with a double-click on the last point to signal the end\n\
of the drawing.  The vertices are connected by solid lines, which\n\
are redrawn with the proper brush, and the polygon filled, when the\n\
double-click occurs.  The center button removes the most recent\n\
point plotted.  When no drawing is in progress, the center button\n\
\"undoes\" the most recent action which changed the picture.";

bpolygon_help()
{
    help_screen(&bpolygon_pm, "Fill Bordered Polygon", polygon_HELP);
}


polygon_help()
{
    help_screen(&polygon_pm, "Fill Polygon", polygon_HELP);
}



