/*
 * @(#)inter.c	3.0	3/27/86
 *
 * Routines for interactive commands for SUN Gremlin
 *
 * Jim Thompson (james@ernie.berkeley.edu)
 *
 */

#include "icondata.h"
#include "gremlin.h"

/* imports from database files */

extern ELT *DBCopy();
extern ELT *DBCreateElt(); 

extern POINT *PTMakePoint();
extern POINT *PTMakeTextPoints();

extern DBDelete();
extern DBClearElt();
extern DBXform();
extern DBBounded();
extern DBAddSet();
extern DBClearSet();
extern DBGravitate();

/* imports from graphics.c */

extern GRCurrentSetOn();
extern GRCurrentSetOff();

/* imports from display.c */

extern struct rect cset_rect;
extern struct pm_pos cset_orig;

/* imports from long.c */

extern LGUndo();
extern int LGPoint(), LGDrift();
extern LGIncludeSet();
extern CP();			/* clear points */
extern CSP();			/* clear show points */

/* imports from text.c */

extern TxMsgOK();
extern TxPutMsg();

/* imports from menu.c */

extern struct _menu menu[];
extern menu_inter();
extern int HiMode[];

/* imports from short.c */

extern int adj[];			/* mapping from Adjustment to HiMode */

/* imports from feedback.c */

extern FBDefine();
extern FBBox();
extern FBErase();
#define	ALL	1		/* option to erase */
#define LAST	2		/* option to erase */

/* imports from C */

extern char *malloc();

/* imports from main.c */

extern error();			    /* print error message */
extern ELT *PICTURE;                /* current PICTURE database      */
extern ELT *cset;                   /* current set database          */
extern ELT *MEN[];                  /* pointers for user symbols     */
extern ELT arhead;                  /* arrow head template           */

extern POINT *POINTLIST;            /* accumulated point list        */
extern float PX, PY;
extern float Lastx, Lasty;
extern SUN_XORIGIN;
extern SUN_YORIGIN;
extern SEQ;                         /* point sequence number         */
extern Adjustment;		    /* adjustment type */
extern CsetOn;
extern Defining;
extern FLASH_READY;		    /* alright to flash */

extern (*lastcommand)();	    /* previous command */
extern lasttext;		    /* TRUE if previous command wants text */
extern inter_index;

extern SHUpdate(), SHAgain(), SHDrawArc(), SHDrawCurve(), SHCopy(),
       SHDefineSet(), SHErase(), SHSetArea(), SHGravity(), SHHide(),
       SHGrid(), SHIcon(), SHOnTop(), SHRotate(), SHScale(),
       SHScaleX(), SHScaleY(), SHScaleXY(), SHTranslate(),
       SHDrawVector(), SHMAdjust(), SHBox(), SHArrow(),
       SHSave1(), SHSave2(), SHSave3(), SHSave4();


/*
 * Forward declarations.
 */

int	INBoxLeft(), INUndo(), INHelp(), INBoxMove(), INExit(),
	INArcLeft(), INArcMove(), 
	INPolyLeft(), INPolyCenter(), INPolyMove(), INBPolyCenter(),
	INVectLeft(), INVectCenter(), INVectMove(), INCurveCenter(),
	INArrowLeft(), 
	INTransLeft(), INTransMove(), INTransCenter(),
	INCopyLeft(), INCopyMove(),
	INMovePtLeft(), INMovePtMove(),
	INDefineLeft();

/*
 * Define interactive routines
 */
INTER_FUNC INBox = {INBoxLeft, INUndo, menu_inter, INBoxMove, INExit},
	INArc = {INArcLeft, INUndo, menu_inter, INArcMove, INExit},
	INVect = {INVectLeft, INVectCenter, menu_inter, INVectMove, INExit},
	INCurve = {INVectLeft, INVectCenter, menu_inter, INVectMove, INExit},
	INArrow = {INArrowLeft, INUndo, menu_inter, INVectMove, INExit},
	INPoly = {INPolyLeft, INPolyCenter, menu_inter, INPolyMove, INExit},
	INBPoly = {INPolyLeft, INPolyCenter, menu_inter, INPolyMove, INExit},
	INTrans = {INTransLeft,INTransCenter, menu_inter, INTransMove, INExit},
	INCopy = {INCopyLeft, INTransCenter, menu_inter, INCopyMove, INExit},
	INMovePt = {INMovePtLeft, INUndo, menu_inter, INMovePtMove, INExit},
	INDefine = {INDefineLeft, INDefineLeft, menu_inter, INBoxMove, INExit};
/*
 * locals
 */

#define	ButtonUp	buttonUp
#define	ButtonDown	(!buttonUp)

static int SaveAdjust;		/* value of overridden adjustment */
static int CopyPhase;		/* Phase of copying */
				/* 0 if no copy in progress */
				/* 1 if copy in progress, but no movement yet*/
				/* 2 if copy has moved */
static int MovePt;		/* point being moved in the current set */
static ELT *MovePtElt;		/* element having a point moved */
static int FirstPt;		/* placing first point */

static int last_button;

/*
 * Generic routines
 */
/*ARGSUSED*/
INUndo(eventPtr)
    XButtonEvent	*eventPtr;
{
	if (ButtonUp) return;
	if (SEQ) {
	    FBErase(ALL);
	    LGDeletePoint();
	} else {
	    LGUndo();
	    CP();
	}
}
/*ARGSUSED*/
INHelp(eventPtr)
XButtonEvent *eventPtr;
{
	if (ButtonUp) return;
	(*(menu[inter_index].menu_help))();
}
/*ARGSUSED*/
INExit(eventPtr)
	XCrossingEvent	*eventPtr;
{
	FBErase(ALL);
	CP();
	if (SaveAdjust) {
	    Adjustment = SaveAdjust;
	    MNHighLt(HiMode[adj[Adjustment]]);
	    SaveAdjust = 0;
	}
	if (CopyPhase) {		/* copy in progress */
	    if (CopyPhase == 2) {	/* remove copy */
		GRCurrentSetOff();		/* fix up the screen */
		cset_rect.r_left = cset_orig.x;
		cset_rect.r_top = cset_orig.y;
		CsetOn = 1;			/* real current set is on */
	    }
	    CopyPhase = 0;
	}

	if ((cset_rect.r_left != cset_orig.x) || /* must be moving */
	    (cset_rect.r_top != cset_orig.y)) {
		FLASH_READY = 0;
		GRCurrentSetOff();
		cset_rect.r_left = cset_orig.x;
		cset_rect.r_top = cset_orig.y;
		FLASH_READY = 1;
	}
	GRCurrentSetOn();
}


/*
 * Draw Box routines
 */

INBoxLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	TxMsgOK();
	/* save any adjustment, and tell user (once) it is overridden */
	if (Adjustment) {
	    TxPutMsg("Overriding Adjustment");
	    MNUnHighLt(HiMode[adj[Adjustment]]);
	    SaveAdjust = Adjustment;
	    Adjustment = NOADJ;
	}
	PX = winx_to_db(eventPtr->x);
	PY = winy_to_db(eventPtr->y);
	(void)LGPoint();
	if (ButtonUp) { /* button up - draw box */
	    if (SEQ == 2) 		/* if there are two points */
	    	SHBox();		/* draw box */
	    else CP();			/* just clear points */
	    FBErase(ALL);		/* erase feedback */
	}
}

INBoxMove(eventPtr)
register XMotionEvent *eventPtr;
{
	register int MinX,MinY,MaxX,MaxY;

	if (SEQ) {

	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0);
	    MaxX = dbx_to_win(PX);
	    MaxY = dby_to_win(PY);
	    MinX = dbx_to_win(POINTLIST->x);
	    MinY = dby_to_win(POINTLIST->y);
	    FBBox(MaxX,MaxY,MinX,MinY);		/* Draw feedback */
	}
	FLASH_READY = 1;	/* ok to flash */
}


INArcLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	TxMsgOK();
	PX = winx_to_db(eventPtr->x);
	PY = winy_to_db(eventPtr->y);
	(void)LGPoint();
	if (ButtonUp) { /* button up - draw box */
	    if (SEQ == 2) 		/* if there are two points */
		SHDrawArc();
	    else CP();			/* just clear points */
	    FBErase(ALL);		/* erase feedback */
	}
}

INArcMove(eventPtr)
register XMotionEvent *eventPtr;
{
	register int RadX,RadY,CntrX,CntrY;

	if (SEQ) {
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0);
	    RadX = dbx_to_win(PX);
	    RadY = dby_to_win(PY);
	    CntrX = dbx_to_win(POINTLIST->x);
	    CntrY = dby_to_win(POINTLIST->y);
	    FBCircle(CntrX,CntrY,RadX,RadY);		/* Draw feedback */
	}
	FLASH_READY = 1;	/* ok to flash */
}

/*
 * Arrow stuff
 */
INArrowLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	POINT points[2];

	PX = winx_to_db(eventPtr->x);		/* grab the current location */
	PY = winy_to_db(eventPtr->y);
	if (ButtonUp) {		/* if button up */
	    if (SEQ) {
		(void)LGPoint();		/* add point to list */
		SHArrow();
		CP();				/* in case of error */
	    }
	    FBErase(ALL);
	    return;
	}
	/* button down */
	TxMsgOK();
	(void)LGPoint();			/* add first point to list */
	points[0].x = points[1].x = PX;
	points[0].y = points[1].y = PY;
	points[0].nextpt = &points[1];
	points[1].nextpt = (POINT *) NULL;
	FBDefine(points[0],1);		/* define feedback */
}


/*
 * Polygon stuff
 */

INPolyLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	POINT point;
	long t;

	t = Button_Interval(eventPtr);

	PX = winx_to_db(eventPtr->x);		/* grab the current location */
	PY = winy_to_db(eventPtr->y);
	if (ButtonUp) {		/* if button up */
	    if (SEQ) {
		if ((FirstPt) && t < 20) {	/* still on the first point */
		    FirstPt = 0;
		    return;
		}
		if (LGPoint()) {	/* if a new point */
		    point.x = PX;
		    point.y = PY;
		    FBChange(point,SEQ-1);	/* correct feedback */
		    (void)FBAdd(point);		/* add point to feedback */
		} else if (SEQ != 1 && t < 20) { /* a click; draw the figure */
		    if (inter_index == 36)
			LGPolygon();
		    else LGBPolygon();
		    if (!SEQ) FBErase(ALL);	/* erase feedback if OK */
		}
	    }
	    FirstPt = 0;
	    return;
	}
	/* button down */
	if (SEQ == 0) {				/* the first point */
	    TxMsgOK();
	    if ((Adjustment == HORZ) || (Adjustment == VERT)) {
		TxPutMsg("Overriding Adjustment");
		MNUnHighLt(HiMode[adj[Adjustment]]);
		SaveAdjust = Adjustment;
		Adjustment = MAN;
	    }
	    (void)LGPoint();			/* add first point to list */
	    point.x = PX;
	    point.y = PY;
	    point.nextpt = (POINT *) NULL;
	    FBDefine(point,1);		/* define feedback */
	    (void)FBAdd(point);		/* add second point to feedback */
	    FirstPt = 1;
	}
}

INPolyMove(eventPtr)
register XMotionEvent *eventPtr;
{
	POINT point;

	if (SEQ) {
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0);
	    point.x = PX;
	    point.y = PY;
	    FBChange(point,SEQ);		/* Draw feedback */
	}
	FLASH_READY = 1;	/* ok to flash */
}

/*ARGSUSED*/
INPolyCenter(eventPtr)
XButtonEvent *eventPtr;
{

	if (ButtonUp) return;
	if (SEQ) {
	    FBErase(LAST);
	    LGDeletePoint();
	} else {
	    LGUndo();
	    CP();
	}
}

/*
 * Vector stuff
 */

INVectLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	POINT point;
	long t;

	t = Button_Interval(eventPtr);

	PX = winx_to_db(eventPtr->x);		/* grab the current location */
	PY = winy_to_db(eventPtr->y);
	if (ButtonUp) {		/* if button up */
	    if (SEQ) {
                if ((FirstPt) && t < 20) {      /* still on the first point */
                    FirstPt = 0;
                    return;
                }
		if(LGPoint()) {			/* add point to list */
		    point.x = PX;
		    point.y = PY;
		    FBChange(point,SEQ-1);	/* correct feedback */
		    (void)FBAdd(point);		/* add point to feedback */
		} else if (SEQ != 1 && t < 20) { /* a click; draw the figure */
		    if (inter_index == 55)
			SHDrawVector();
		    else SHDrawCurve();
		    if (!SEQ) FBErase(ALL);	/* erase feedback if OK */
		}
	    }
	    FirstPt = 0;
	    return;
	}
	/* button down */
	if (SEQ == 0) {				/* the first point */
	    TxMsgOK();
	    (void)LGPoint();			/* add first point to list */
	    point.x = PX;
	    point.y = PY;
	    point.nextpt = (POINT *) NULL;
	    FBDefine(point,0);		/* define feedback */
	    (void)FBAdd(point);		/* add point to feedback */
	    FirstPt = 1;
	}
}

INVectMove(eventPtr)
register XMotionEvent *eventPtr;
{
	POINT point;

	if (SEQ) {
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0); 
	    point.x = PX;
	    point.y = PY;
	    FBChange(point,SEQ);		/* Draw feedback */
	}
	FLASH_READY = 1;	/* ok to flash */
}

/*ARGSUSED*/
INVectCenter(eventPtr)
XButtonEvent *eventPtr;
{

	if (ButtonUp) return;
	if (SEQ) {
	    FBErase(LAST);
	    LGDeletePoint();
	} else {
	    LGUndo();
	    CP();
	}
}


/*
 * Define current set interactively
 */

INDefineLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	long t;

	t = Button_Interval(eventPtr);
	TxMsgOK();
	if (DBNullelt(PICTURE)) {	/* empty picture */
	    error("Picture is empty");
	    return;
	}
	/* save any adjustment, and tell user (once) it is overridden */
	if (Adjustment) {
	    MNUnHighLt(HiMode[adj[Adjustment]]);
	    SaveAdjust = Adjustment;
	    Adjustment = NOADJ;
	}
	PX = winx_to_db(eventPtr->x);
	PY = winy_to_db(eventPtr->y);
	if (t > 20) (void)LGPoint();	/* no second point needed if a click */
	if (ButtonUp) {    /* button up */
	    GRCurrentSetOn();
	    FBErase(ALL);		/* Erase all feedback */
	    switch (eventPtr->button) {
	    case 1:
		if (SEQ == 1) SHDefineSet(); /* point defines set */
		else SHSetArea();		/* otherwise use area */
		break;
	    case 2:
		if (SEQ == 1) LGIncludeSet(); /* add point to set */
		else SHMSetArea();		/* add area to set */
		break;
	    }
	    if (SaveAdjust) {
		Adjustment = SaveAdjust;
		MNHighLt(HiMode[adj[Adjustment]]);
		SaveAdjust = 0;
	    }
	}
}

/*
 * Move current set
 */

INTransLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	TxMsgOK();
	if (DBNullelt(cset)) return;
	CSP();				/* turn off show points */
	GRCurrentSetOn();		/* make sure set is on */
	PX = winx_to_db(eventPtr->x);
	PY = winy_to_db(eventPtr->y);
	(void)LGPoint();
	if (ButtonUp) {    /* button up */
	    if (SEQ < 2) {	/* changed his mind */
		CP();			/* clear any residue */
	    } else {
		SHTranslate();
	    }
	    FLASH_READY = 1;		/* ok to flash */
	}
}

INTransMove(eventPtr)
register XMotionEvent *eventPtr;
{
	if (SEQ) {
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0); 
	    GRCurrentSetOff();
	    cset_rect.r_left = cset_orig.x + (short) (PX-Lastx);
	    cset_rect.r_top = cset_orig.y + (short) (Lasty-PY);
	    GRCurrentSetOn();
	} else {
	    FLASH_READY = 1;	/* ok to flash if not moving */
	}
}
/*ARGSUSED*/
INTransCenter(eventPtr)
XButtonEvent *eventPtr;
{
	if (ButtonUp) return;
	if (SEQ) {
	    if (CopyPhase) {		/* copy in progress */
		if (CopyPhase == 2) {	/* remove copy */
		    GRCurrentSetOff();		/* fix up the screen */
		    cset_rect.r_left = cset_orig.x;
		    cset_rect.r_top = cset_orig.y;
		    CsetOn = 1;		/* real current set is on */
		}
		CopyPhase = 0;
	    }

	    if ((cset_rect.r_left != cset_orig.x) || /* must be moving */
	    (cset_rect.r_top != cset_orig.y)) {
		FLASH_READY = 0;
		GRCurrentSetOff();
		cset_rect.r_left = cset_orig.x;
		cset_rect.r_top = cset_orig.y;
		FLASH_READY = 1;
	    }
	    GRCurrentSetOn();
	    LGDeletePoint();
	} else {
	    LGUndo();
	    CP();
	}
}

/*
 * Copy current set
 */

INCopyLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	TxMsgOK();
	if (DBNullelt(cset)) return;
	FLASH_READY = 0;
	PX = winx_to_db(eventPtr->x);
	PY = winy_to_db(eventPtr->y);
	(void)LGPoint();
	if (ButtonUp) {    /* button up */
	    if (CopyPhase == 2) {		/* copy has moved */
		GRCurrentSetOff();		/* fix up the screen */
		cset_rect.r_left = cset_orig.x;
		cset_rect.r_top = cset_orig.y;
		CsetOn = 1;			/* real current set is on */
	    }
	    CopyPhase = 0;
	    if (SEQ < 2) {	/* changed his mind */
		CP();			/* clear any residue */
		FBErase(ALL);
	    } else 
		SHCopy();
	} else {
	    GRCurrentSetOn();
	    CopyPhase = 1;		/* next movement will duplicate it */
	}
}

INCopyMove(eventPtr)
register XMotionEvent *eventPtr;
{
	if (SEQ) {
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0); 
	    if (CopyPhase == 1) {	/* first movement */
		GRCurrentSetOn();	/* turn on original */
		CsetOn = 0;		/* copy is off */
		CopyPhase = 2;		/* CsetOn is out of phase */
	    } else
		GRCurrentSetOff();	/* turn off copy */
	    cset_rect.r_left = cset_orig.x + (short) (PX-Lastx);
	    cset_rect.r_top = cset_orig.y + (short) (Lasty-PY);
	    GRCurrentSetOn();		/* turn copy back on */
	} else {
	    FLASH_READY = 1;	/* ok to flash if not moving */
	}
}

/*
 * Move a point in the current set
 */
INMovePtLeft(eventPtr)
register XButtonEvent *eventPtr;
{
	POINT *p1, *p2;
	int closed = 0;			/* is the figure closed */
	register int RadX, RadY, CntrX, CntrY;

	if (ButtonDown) {		/* find the closest point */
	    TxMsgOK();
	    FLASH_READY = 0;
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    DBGravitate (PX, PY, &PX, &PY, &p1, &MovePtElt, cset, TRUE);
	    if (DBNullelt(MovePtElt) || TEXT(MovePtElt->type)) {
		error("can't find a good element");
	        return;
	    }

	    GRCurrentSetOff();
	    p2 = MovePtElt->ptlist;
	    MovePt = 0;
	    while (p2 != p1) {
		MovePt++;
		p2 = PTNextPoint(p2);
	    }

	    switch (MovePtElt->type) {
		case POLYGON:
		    closed = 1;
		case VECTOR:
		case CURVE:
		case B_SPLINE:
		case BEZIER:
		    FBDefine(*(MovePtElt->ptlist),closed);
		    break;
		case ARC:
		    p2 = MovePtElt->ptlist;
		    RadX = dbx_to_win(PTNextPoint(p2)->x);
		    RadY = dby_to_win(PTNextPoint(p2)->y);
		    CntrX = dbx_to_win(p2->x);
		    CntrY = dby_to_win(p2->y);
		    FBCircle(CntrX,CntrY,RadX,RadY);	/* Draw feedback */
		    break;
	    }

	    (void)LGPoint();			/* install point */
	} else { /* button up */
	    FBErase(ALL);
	    if (SEQ) {				/* still wants to do it */
		GRCurrentSetOn();
		PX = winx_to_db(eventPtr->x);
		PY = winy_to_db(eventPtr->y);
		(void)LGPoint();
		LGMPoint();
	    }
	    CP();			/* clear all points */
	}
}

INMovePtMove(eventPtr)
register XMotionEvent *eventPtr;
{
	POINT point;
	POINT *p1;
	register int CntrX, CntrY, RadX, RadY;

	if (SEQ) {
	    GRCurrentSetOff();
	    PX = winx_to_db(eventPtr->x);
	    PY = winy_to_db(eventPtr->y);
	    LGDrift(0); 
	    switch (MovePtElt->type) {
		case ARC:
		    p1 = MovePtElt->ptlist;
		    if (MovePt) {		/* moving the edge */
			RadX = dbx_to_win(PX);
			RadY = dby_to_win(PY);
			CntrX = dbx_to_win(p1->x);
			CntrY = dby_to_win(p1->y);
		    } else {			/* moving the center */
			RadX = dbx_to_win(PTNextPoint(p1)->x);
			RadY = dby_to_win(PTNextPoint(p1)->y);
			CntrX = dbx_to_win(PX);
			CntrY = dby_to_win(PY);
		    }
		    FBCircle(CntrX,CntrY,RadX,RadY);	/* Draw feedback */
		    break;
		case VECTOR:
		case CURVE:
		case B_SPLINE:
		case BEZIER:
		case POLYGON:
		    point.x = PX;
		    point.y = PY;
		    FBChange(point,MovePt);	/* Draw feedback */
		    break;
	    }
	} else {
	    FLASH_READY = 1;		/* ok to flash if not moving */
	}

}

int
Button_Interval(eventPtr)
register XButtonEvent *eventPtr;
{
	long	itime;

	itime = (eventPtr->time - last_button) / 10;

	last_button = eventPtr->time;

	return itime;
}
