/*
 * @(#)main.c	1.2	1/3/85
 *
 * Main program for the SUN Gremlin picture editor.
 *
 * Many of the routines in SUN Gremlin are direct descendants of
 * their counterparts in the original Gremlin written for the AED
 * by Barry Roitblat.
 *
 * Original SUN Gremlin by Mark Opperman 
 *
 * Modifications by: Yih-Farn Chen (yfchen@monet.berkeley.edu)
 *                   Jim Thompson (james@arpa.Berkeley.EDU) 
 *		     Mike Nelson (nelson@ginger.Berkeley.EDU)
 *
 */

#include <ctype.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/file.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "gremlin.h"
#include "icondata.h"
#include "sx.h"
#include "icons/icon.kbdcursor"
#include "icons/icon.diamond"
#include "icons/icon.uparrow"
#include "icons/gremlin.icon"

#include <X11/keysym.h>
#include <X11/keysymdef.h>

int		debugLevel = 0;
extern int      errno;

/* database imports */

extern ELT *DBRead();
extern POINT *PTMakePoint();
extern FILE *POpen();

/* imports from startup.c */

extern STERROR;
extern STgremlinrc();

/* imports from menu.c */

extern MNDisplayMenu();
extern MNInitMenu();
extern MNSHCommand();
extern MNShift();
extern MNUnShift();

/* graphics imports */

extern GRBlankGrid();
extern GRCurrentSet();
extern GRCurrentSetOn();
extern GRDisplayGrid();
extern GRFontInit();

/* imports from undodb.c */

extern UNForget();
extern UNELT *unlist;
extern UNELT *unback;
 
/* imports from long.c */

extern SHOWPOINTS;
extern char *Editfile;
extern CP();
extern LGQuit();
extern nop();

/* imports from short.c */

extern SHUpdate();

/* imports from C */
/*
extern char *malloc();
extern char *sprintf();
extern char *strcat();
extern char *strcpy();
*/
/* imports from strings.c */

extern char version[];
extern char GLibrary[];
extern char text_font[];

/* imports from menu.c */

extern menu_left();
extern menu_middle();
extern menu_right();
extern menu_winexit();
extern mouse_move();
extern menu_inter();

/* imports from pix.c */

extern pix_left();
extern pix_middle();
extern pix_right();
extern pix_move();
extern pix_winexit();


/* imports from suntext.c */

extern text_left();
extern text_middle();
extern text_right();
extern text_winexit();
extern text_putvalue();
extern void text_output();
extern TxInit();
extern TxPutMsg();

/* Declaration of Globals */

ELT arhead;
ELT *cset;
ELT *MEN[NUSER];
ELT *PICTURE;

POINT *BACKPOINT;
POINT MENPOINT[NUSER];
POINT *POINTLIST;

int Artmode = FALSE;
int Adjustment = NOADJ;
int Alignment = 4;
int Arrowtype = 0;
int Curvetype = B_SPLINE;	/* (Interpolated, B-spline, or Bezier) */
int CBRUSH = 5;
int CFONT = 1;
int CJUST = 0;
int CSIZE = 1;
int CSTIPPLE = 1;
int Defining = FALSE;		/* SHIFT key is down (define current set) */
int GravityOn = FALSE;
int Gridon = FALSE;
int Gridsize = 32;
int Orientation;
int SEQ = 0;
int CHANGED = FALSE;
int SEARCH = TRUE;
int SymbolicLines = 0;
int newfileformat = 1;			/* 1=sungremlinfile, 0=gremlinfile */
int TOOLINSTALLED = 0;
int (*lastcommand)();			/* last command's routine pointer */
int lasttext = FALSE;			/* TRUE if last command uses text */

int inter_index = 0;		        /* index of current interactive fnc */
int shifted_index = 0;		/* saved interactive function while shifted */
INTER_FUNC Normal_act =		        /* normal pix actions */
{pix_left, pix_middle, menu_inter, pix_move, pix_winexit};
INTER_FUNC *pix_act = &Normal_act;	/* current pix_actions */

float PX, PY;				/* user point coordinate */
float Lastx, Lasty;			/* last user point coordinate */
long timeon_sec = 0;
long timeon_usec = 800000;		/* current set flash on period */
long timeoff_sec = 0;
long timeoff_usec = 400000;		/* current set flash off period */

int FLASH_READY = 0;			/* TRUE if time to flash current set */
int CsetOn = 1;				/* TRUE if current set displayed */

int SUN_XORIGIN = 0;			/* top left corner in gremlin coords */
int SUN_YORIGIN = 511;

/*
 * Cursors.
 */
Cursor	kbd_cursor;
Cursor	main_cursor;
Cursor	menu_cursor;

/*
 * Root window stuff.
 */
Display	*display;
Window	root_window;
Window	main_window;
GC	root_gc;
unsigned long	foreground;
unsigned long	background;
unsigned int	root_height;
unsigned int	root_width;
unsigned int	root_depth;
unsigned int	drawOP;
unsigned int	menuEraseOP;
unsigned int	xorOP;
Colormap	colorMap;
unsigned int	planeMask;
unsigned long	pixel;

/* Title subwindow stuff */

char		namestripe[256];
Window		title_sw;
GC		title_gc;

/* text subwindow */

Window		text_sw;
GC		text_gc;
GC		text_xor_gc;
struct rect	text_size;
XFontStruct	*text_pf;
static		init_text();
static		text_selected();
static		text_sighandler();
extern unsigned	text_width;
extern unsigned	text_height;

/* menu subwindow */

Window		menu_sw;
GC		menu_gc;
GC		menu_xor_gc;
struct rect	menu_size;
static		init_menu();
static		menu_selected();
static		menu_sighandler();

/* pix subwindow */

Window		pix_sw;
GC		pix_gc;
GC		pix_xor_gc;
struct rect	pix_size;
static		init_pix();
static		pix_selected();
static		pix_sighandler();
PMRec		cset_pm;
PMRec		scratch_pm;
GC		pixmap_gc;

PMRec		grid32_pm;

static XColor black = {0, 0, 0, 0, 0};
static XColor white = {0, ~0, ~0, ~0, 0};

/*
 * 1 if the last pix window event caused was a mouse button going up.
 */
int buttonUp;

static init_text();
static init_menu();
static init_pix();
static void SetErrorHandler();

/*
 * Gremlin icon.
 */
Pixmap gremlin_pm;

/*
 * This routine prints an error message in the text subwindow.
 */
error(s)
char *s;
{
    TxPutMsg(s);
}

static	void	TimeSubtract();
struct timeval	lastTime = {0, 0};

main(argc, argv)
int argc;
char *argv[];
{
    char		*arg, *file, *gremlinrc, *usage;
    char		*geometry = NULL;
    XSizeHints		sizeHints;
    XGCValues		gcValues;
    XEvent		event;
    int			dum;
    struct timeval	curTime;

    file = "";
    gremlinrc = "";
    usage = "usage: gremlin [-o] [-s <.gremlinrc>] [file] [generic tool arguments]\n";

    /*
     * Parse command line arguments.
     */
    while (--argc > 0) {
	arg = *++argv;
	if (*arg == '=') {
	    geometry = arg;
	} else if (*arg != '-') {
	    file = arg;
	} else {
	    switch (*++arg) {
		case 's':
		    if (*++arg == '\0')
			if (--argc > 0)
			    arg = *++argv;
		    if (argc < 0) {
			printf("%s", usage);
			exit(1);
		    }
		    gremlinrc = arg;
		    break;
		case 'o':
		    newfileformat = 0;
		    break;
		default:
		    printf("%s", usage);
		    exit(1);
	    }
	}
    }

    /*
     * Open a connection to the display.
     */
    display = XOpenDisplay((char *) NULL);
    SetErrorHandler();
    if (display == (Display *)NULL) {
	printf("Couldn't open display.  Are you sure X is running?\n");
	return;
    }

    if (debugLevel > 1) {
	XSynchronize(display, 1);
    }

    root_window = RootWindow(display, DefaultScreen(display));
    (void)XGetGeometry(display, root_window, (Window *)&dum, &dum, &dum, 
		 &root_width, &root_height, (unsigned *)&dum,
		 &root_depth);

    /*
     * Determine the size and location of the main window.
     */
    sizeHints.x = 0;
    sizeHints.y = 0;
    sizeHints.width = PIXSW_WIDTH + MENUSW_WIDTH + 4 * BORDER_WIDTH;
    sizeHints.height = PIXSW_HEIGHT + Sx_DefaultHeight(display, text_pf) + 
		       TEXTSW_HEIGHT + 6 * BORDER_WIDTH;
    sizeHints.max_width = root_width;
    sizeHints.max_height = root_height;
    sizeHints.flags = PMaxSize | USSize;
    if (geometry != NULL) {
	int	flags;

	flags = XParseGeometry(geometry, &sizeHints.x, &sizeHints.y,
			       (unsigned *)&sizeHints.width, 
			       (unsigned *)&sizeHints.height);
	if ((flags & (WidthValue|HeightValue)) == (WidthValue|HeightValue)) {
	    sizeHints.flags = USSize;
	}
	if ((flags & (XValue|YValue)) == (XValue|YValue)) {
	    sizeHints.flags |= USPosition;
	    if (flags & XNegative) {
		sizeHints.x = DisplayWidth(display, DefaultScreen(display))
			+ sizeHints.x - sizeHints.width - 4;
	    }
	    if (flags & YNegative) {
		sizeHints.y = DisplayHeight(display, DefaultScreen(display))
			+ sizeHints.y - sizeHints.height - 4;
	    }
	}
    }

    strcpy(namestripe, version);

    /*
     * Read defaults and set up font information.
     */
    if (root_depth == 1) {
	foreground = BlackPixel(display, DefaultScreen(display));
	background = WhitePixel(display, DefaultScreen(display));
	if (foreground == 1) {
	    drawOP = GXor;
	    menuEraseOP = GXandInverted;
	    xorOP = GXxor;
	} else {
	    drawOP = GXand;
	    menuEraseOP = GXorInverted;
	    xorOP = GXequiv;
	}
	planeMask = 1;
    } else {
	XColor	color;
	/*
	 * Allocate contiguous color map entries for black and white.
	 */
	colorMap = XDefaultColormap(display, DefaultScreen(display));
	XAllocColorCells(display, colorMap, 1, &planeMask, 1, &pixel, 1);
	foreground = pixel | planeMask;
	background = pixel;
	drawOP = GXor;
	menuEraseOP = GXandInverted;
	xorOP = GXxor;
	XStoreNamedColor(display, colorMap, "black", foreground, 
			 DoRed | DoGreen | DoBlue);
	XStoreNamedColor(display, colorMap, "white", background, 
			 DoRed | DoGreen | DoBlue);
    }
    text_pf = XLoadQueryFont(display, text_font);
    if (text_pf == NULL) {
	fprintf(stderr, "Couldn't open font file\n");
	exit(1);
    }
    gcValues.foreground = foreground;
    gcValues.background = background;
    gcValues.font = text_pf->fid;
    gcValues.graphics_exposures = False;
    gcValues.plane_mask = planeMask;

    root_gc = XCreateGC(display, root_window,
	    GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask,
			&gcValues);
    /*
     * Initialize all pixmaps.
     */
    ICCreate();

    /*
     * Create the main window.
     */
    main_window = XCreateSimpleWindow(display, root_window,
				      sizeHints.x, sizeHints.y, 
		      (unsigned)sizeHints.width, (unsigned)sizeHints.height,
				      BORDER_WIDTH, 
				      foreground, background);

    /*
     * Create the sub-windows.
     */
    title_sw = Sx_CreatePacked(display, main_window, SX_TOP, 
			Sx_DefaultHeight(display, text_pf), 0, 
			BORDER_WIDTH, foreground, (Window) NULL, background);
    title_gc = XCreateGC(display, root_window,
	     GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask, 
		         &gcValues);
    text_sw = Sx_CreatePacked(display, main_window, SX_TOP, TEXTSW_HEIGHT, 0,
			BORDER_WIDTH, foreground, (Window) NULL, background);
    text_gc = XCreateGC(display, root_window,
	    GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask, 
		       &gcValues);

    XSetPlaneMask(display, text_gc, (unsigned long)1);
    menu_sw = Sx_CreatePacked(display, main_window, SX_LEFT, MENUSW_WIDTH, 0,
			BORDER_WIDTH, foreground, (Window) NULL, background);
    menu_gc = XCreateGC(display, root_window, 
	    GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask, 
		       &gcValues);
    pix_sw = Sx_CreatePacked(display, main_window, SX_TOP, 0, 1, 0,
			foreground, (Window) NULL, background);
    pix_gc = XCreateGC(display, root_window, 
	   GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask, 
		      &gcValues);

    /*
     * Create pixmaps for xor operations.
     */
    gcValues.function = xorOP;
    text_xor_gc = XCreateGC(display, root_window,
	    GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask|
			GCFunction, 
		       &gcValues);
    pix_xor_gc = XCreateGC(display, root_window,
	    GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask|
			GCFunction, 
		       &gcValues);
    menu_xor_gc = XCreateGC(display, root_window,
	    GCForeground|GCBackground|GCFont|GCGraphicsExposures|GCPlaneMask|
			GCFunction, 
		       &gcValues);
    /*
     * Set up the icon and default properties.
     */
    gremlin_pm = XCreateBitmapFromData(display, root_window, 
				   (char *)gremlin_icon_data,
				   GREMLIN_ICON_WIDTH, GREMLIN_ICON_HEIGHT);
    XSetStandardProperties(display, main_window, "gremlin", "gremlin",
			   gremlin_pm, (char **)NULL, 0, &sizeHints);
    /*
     * Create a GC for the pixmaps that are of depth 1.
     */
    pixmap_gc = XCreateGC(display, gremlin_pm,
			GCForeground|GCBackground|GCFont, &gcValues);

    /*
     * If necessary copy the grid from a pixmap of depth 1 to a pixmap
     * of depth 8 to speed up drawing the grid on color displays.
     */
    grid32_pm = replgrid32_pm;
#ifdef notdef
    if (root_depth > 1) {
	grid32_pm.pm = XCreatePixmap(display, root_window,
			           replgrid32_pm.width, replgrid32_pm.height,
				   root_depth);
	XSetFunction(display, pix_gc, GXcopy);
	XCopyPlane(display, replgrid32_pm.pm, grid32_pm.pm, pix_gc,
		   0, 0, grid32_pm.width, grid32_pm.height, 0, 0,
		   (unsigned long)1);
    }
#endif

    init_text();
    init_menu();
    init_pix();

    /* set up gremlin variables */
    main_init(file, gremlinrc);

    TOOLINSTALLED = 1;

    /*
     * Map the main_window and process events.
     */
    XMapWindow(display, main_window);

    while (1) {
	int		readmask;
	int		nfds;
	int		numEvents;
	struct	timeval waitTime;
	struct	timeval diff;
        Window	root, child;
        int	rootx, rooty, winx, winy;
        unsigned int buttons;

	if (FLASH_READY && timeoff_usec != 0) {
	    gettimeofday(&curTime, NULL);

	    if (lastTime.tv_sec == 0) {
		lastTime = curTime;
	    } else {
		TimeSubtract(curTime, lastTime, &diff);
		if (CsetOn) {
		    if (diff.tv_sec * 1000000 + diff.tv_usec >=
		        timeon_sec * 1000000 + timeon_usec) {
			GRCurrentSet();
			lastTime = curTime;
		    }
		} else {
		    if (diff.tv_sec * 1000000 + diff.tv_usec >=
		        timeoff_sec * 1000000 + timeoff_usec) {
			GRCurrentSet();
			lastTime = curTime;
		    }
		}
	    }

	    numEvents = XPending(display);
	    if (numEvents > 0) {
		do {
		    XNextEvent(display, &event);
		    if (debugLevel > 0) {
			printf("type=%d window=%x serial=%d\n", event.type, 
				event.xany.window, event.xany.serial);
			fflush(stdout);
		    }
		    Sx_HandleEvent(&event);

		    numEvents--;
		} while (numEvents > 0);
		/*
		 * Dummy call to allow another motion event to be
		 * captured 
		 */
		if(event.type == MotionNotify &&
		   event.xmotion.window == pix_sw) {
		   XQueryPointer(display,pix_sw,&root,&child,&rootx,&rooty,
				 &winx,&winy,&buttons);
		}
	    } else {
		if (CsetOn) {
		    waitTime.tv_sec = timeon_sec;
		    waitTime.tv_usec = timeon_usec;
		} else {
		    waitTime.tv_sec = timeoff_sec;
		    waitTime.tv_usec = timeoff_usec;
		}
		 readmask = ConnectionNumber(display);/* 1 << display->fd; */
		
		nfds = select(32, &readmask, (int *)0, (int *)0, &waitTime);
		if (nfds < 0 && errno != EINTR) {
		    perror("Select");
		    exit(1);
		}
	    }
	} else {
	    lastTime.tv_sec = 0;
	    XNextEvent(display, &event);
	    if (debugLevel > 0) {
		printf("type=%d window=%x serial=%d\n", event.type, 
			event.xany.window, event.xany.serial);
		fflush(stdout);
	    }
	    Sx_HandleEvent(&event);
	    /*
	     * Dummy call to allow another motion event to be
	     * captured 
	     */
	    if(event.type == MotionNotify &&
	       event.xmotion.window == pix_sw) {
	       XQueryPointer(display,pix_sw,&root,&child,&rootx,&rooty,
			     &winx,&winy,&buttons);
	    }
	}
    }
}

static
init_text()
{
    Pixmap	source, mask;
    unsigned	dum;

    /*
     * Create the handlers.
     */
    Sx_HandlerCreate(display, text_sw, ExposureMask,
		     text_sighandler, (ClientData)0);
    Sx_HandlerCreate(display, text_sw, ButtonPressMask | ButtonReleaseMask |
				       KeyPressMask | 
				       LeaveWindowMask | EnterWindowMask,
		     text_selected, (ClientData)0);
    /*
     * Find out how big the window is.
     */
    (void)XGetGeometry(display, text_sw, (Window *)&dum, &text_size.r_left,
		 &text_size.r_top, &text_size.r_width, &text_size.r_height,
		 &dum, &dum);
    /*
     * Set up the cursor.
     */
    source = XCreateBitmapFromData(display, root_window, 
				   (char *)icon_kbdcursor_data,
				   KBDCURSOR_WIDTH, KBDCURSOR_HEIGHT);
    mask = XCreateBitmapFromData(display, root_window, 
				 (char *)icon_kbdcursor_mask,
				 KBDCURSOR_WIDTH, KBDCURSOR_HEIGHT);
    kbd_cursor = XCreatePixmapCursor(display, source, mask, &black, &white,
				     KBDCURSOR_X_HOT, KBDCURSOR_Y_HOT);
    XFreePixmap(display, source);
    XFreePixmap(display, mask);
    XDefineCursor(display, text_sw, kbd_cursor);
    /*
     * Set up default font sizes.
     */
    text_width = XTextWidth(text_pf, "M", 1);
    text_height = Sx_DefaultHeight(display, text_pf);
}


static
init_menu()
{
    Pixmap	source, mask;
    unsigned	dum;

    /*
     * Create the handlers.
     */
    Sx_HandlerCreate(display, menu_sw, ExposureMask,
		     menu_sighandler, (ClientData)0);
    Sx_HandlerCreate(display, menu_sw, ButtonPressMask | ButtonReleaseMask |
				       KeyPressMask | 
				       LeaveWindowMask | EnterWindowMask |
				       PointerMotionMask,
		     menu_selected, (ClientData)0);
    /*
     * Find out how big the window is.
     */
    (void)XGetGeometry(display, text_sw, (Window *)&dum, &text_size.r_left,
		 &text_size.r_top, &text_size.r_width, &text_size.r_height,
		 &dum, &dum);
    /*
     * Set up the cursor.
     */
    source = XCreateBitmapFromData(display, root_window, 
				   (char *)icon_diamond_data,
				   DIAMOND_WIDTH, DIAMOND_HEIGHT);
    mask = XCreateBitmapFromData(display, root_window, 
				   (char *)icon_diamond_mask,
				 DIAMOND_WIDTH, DIAMOND_HEIGHT);
    menu_cursor = XCreatePixmapCursor(display, source, mask, &black, &white,
				      DIAMOND_X_HOT, DIAMOND_Y_HOT);
    XFreePixmap(display, source);
    XFreePixmap(display, mask);
    XDefineCursor(display, menu_sw, menu_cursor);
}


static
init_pix()
{
    XGCValues	gcValues;
    Pixmap	source, mask;

    /*
     * Create the event handlers.
     */
    Sx_HandlerCreate(display, pix_sw, ExposureMask,
		     pix_sighandler, (ClientData)0);
    Sx_HandlerCreate(display, pix_sw, ButtonPressMask | ButtonReleaseMask |
			PointerMotionMask | KeyPressMask | KeyReleaseMask |
			LeaveWindowMask | EnterWindowMask |
			PointerMotionHintMask,
		     pix_selected, (ClientData)0);
    /*
     * Set up the cursors.
     */
    source = XCreateBitmapFromData(display, root_window, 
				   (char *)icon_uparrow_data,
				   UPARROW_WIDTH, UPARROW_HEIGHT);
    mask = XCreateBitmapFromData(display, root_window, 
				   (char *)icon_uparrow_mask,
				   UPARROW_WIDTH, UPARROW_HEIGHT);
    main_cursor = XCreatePixmapCursor(display, source, mask, &black, &white,
				      UPARROW_X_HOT, UPARROW_Y_HOT);
    XFreePixmap(display, source);
    XFreePixmap(display, mask);
    XDefineCursor(display, pix_sw, main_cursor);

    /*
     * Create the pixmap for retaining the image of the current set.
     */
    cset_pm.pm = XCreatePixmap(display, root_window, root_width,
			       root_height, root_depth);
    if (cset_pm.pm == NULL) {
	printf("can't create cset_pm\n");
	exit(1);
    }
    cset_pm.width = root_width;
    cset_pm.height = root_height;

    /*
     * Create pixmap to do all element drawing in memory (faster)
     */
    scratch_pm.pm = XCreatePixmap(display, root_window,
			       root_width, root_height, root_depth);
    if (scratch_pm.pm == NULL) {
	printf("can't create scratch_pm\n");
	exit(1);
    }
    scratch_pm.width = root_width;
    scratch_pm.height = root_height;

    /*
     * Clear out the current set and scratch pixmaps.
     */
    gcValues.foreground = background;
    gcValues.function = GXcopy;
    XChangeGC(display, pix_gc, GCForeground | GCFunction, &gcValues);
    XFillRectangle(display, cset_pm.pm, pix_gc, 0, 0, 
		   root_width, root_height);
    XFillRectangle(display, scratch_pm.pm, pix_gc, 0, 0, 
		   root_width, root_height);
    XSetForeground(display, pix_gc, foreground);
}

/*ARGSUSED*/
static
text_selected(dum, eventPtr)
    int		dum;
    XEvent	*eventPtr;
{
    switch (eventPtr->type) {
	case ButtonPress:
	    if (eventPtr->xbutton.button == 1) {
		text_left(&eventPtr->xbutton);
	    } else if (eventPtr->xbutton.button == 2) {
		text_middle(&eventPtr->xbutton);
	    } else if (eventPtr->xbutton.button == 3) {
		text_right(&eventPtr->xbutton);
	    }
	    break;
	case EnterNotify:
	    if (Sx_FocusEnabled(display)) {
		XWindowAttributes	winAttr;
		XGetWindowAttributes(display, text_sw, &winAttr);
		if (winAttr.map_state == IsViewable) {
		    XSetInputFocus(display, text_sw, RevertToNone, CurrentTime);
		}
	    }
	    break;
        case LeaveNotify:
	    if (Sx_FocusEnabled(display)) {
		XSetInputFocus(display, None, RevertToNone, CurrentTime);
	    }
	    text_winexit(&eventPtr->xcrossing);
	    break;
        case KeyPress: {
#define CNTRL_V		22
#define MAX_CHARS	100
	    int	numBytes;
	    char	buf[MAX_CHARS + 1];

	    numBytes = XLookupString(&eventPtr->xkey, buf, MAX_CHARS,
				     (KeySym *)NULL, (XComposeStatus *)NULL);
	    if (numBytes == 1 && isascii(buf[0])) {
		if (buf[0] == CNTRL_V) {
		    char format[SX_FORMAT_SIZE];
		    char *selPtr;

		    numBytes = Sx_SelectionGet(display, "text", 0, MAX_CHARS,
					       buf, format);
		    for (selPtr = buf; numBytes > 0; selPtr++, numBytes--) {
			text_output(*selPtr);
		    }
		} else {
		    text_output(buf[0]);
		}
	    }
	    break;
	}
    }
}

/*ARGSUSED*/
static
menu_selected(dum, eventPtr)
    int		dum;
    XEvent	*eventPtr;
{
    switch (eventPtr->type) {
	case MotionNotify:
	    mouse_move(&eventPtr->xmotion);
	    break;
	case ButtonPress:
	    if (eventPtr->xbutton.button == 1) {
		menu_left(&eventPtr->xbutton);
	    } else if (eventPtr->xbutton.button == 2) {
		menu_middle(&eventPtr->xbutton);
	    } else if (eventPtr->xbutton.button == 3) {
		menu_right(&eventPtr->xbutton);
	    }
	    break;
	case EnterNotify:
	    if (Sx_FocusEnabled(display)) {
		XWindowAttributes	winAttr;
		XGetWindowAttributes(display, menu_sw, &winAttr);
		if (winAttr.map_state == IsViewable) {
		    XSetInputFocus(display, menu_sw, RevertToNone, CurrentTime);
		}
	    }
	    break;
	case LeaveNotify:
	    if (Sx_FocusEnabled(display)) {
		XSetInputFocus(display, None, RevertToNone, CurrentTime);
	    }
	    menu_winexit(&eventPtr->xcrossing);
	    break;
        case KeyPress: {
	    int	numBytes;
	    char	keyString[20];
	    char	shcmd[2];
    
	    numBytes = XLookupString(&eventPtr->xkey, keyString, 20,
				     (KeySym *)NULL, (XComposeStatus *)NULL);
	    if (numBytes == 1 && isascii(keyString[0])) {
		if ((shcmd[0] = keyString[0]) != '.')
		    lasttext = FALSE;
		shcmd[1] = '\0';
		MNSHCommand(shcmd);
	    }
	}
    }
}

/*ARGSUSED*/
static
pix_selected(dum, eventPtr)
    int		dum;
    XEvent	*eventPtr;
{
    char		shcmd[2];

    buttonUp = 0;
    switch (eventPtr->type) {
	case KeyRelease:
	    if (Defining && (eventPtr->xkey.state & ShiftMask)) {
		MNUnShift();
	    }
	    break;
	case ButtonRelease:
	case ButtonPress:
	    buttonUp = eventPtr->type == ButtonRelease;
	    if (eventPtr->xbutton.button == 1) {
		FLASH_READY = 0;
		(*(pix_act->ms_left))(&eventPtr->xbutton);
		FLASH_READY = 1;
	    } else if (eventPtr->xbutton.button == 2) {
		FLASH_READY = 0;
		(*(pix_act->ms_center)) (&eventPtr->xbutton);
		FLASH_READY = 1;
	    } else if (eventPtr->xbutton.button == 3) {
		FLASH_READY = 0;
		(*(pix_act->ms_right)) (&eventPtr->xbutton);
		FLASH_READY = 1;
	    }
	    break;
	case EnterNotify:

	    if (Sx_FocusEnabled(display)) {
		XWindowAttributes	winAttr;
		XGetWindowAttributes(display, pix_sw, &winAttr);
		if (winAttr.map_state == IsViewable) {
		    XSetInputFocus(display, pix_sw, RevertToNone, CurrentTime);
		}
	    }
	    break;
	case LeaveNotify:
	    if (Sx_FocusEnabled(display)) {
		XSetInputFocus(display, None, RevertToNone, CurrentTime);
	    }
	    if (Defining) {
		MNUnShift();
	    }
	    FLASH_READY = 0;
	    (*(pix_act->win_exit))(&eventPtr->xcrossing);
	    break;
	case MotionNotify:
	    FLASH_READY = 0;/* not ok to flash */
	    (*(pix_act->ms_move)) (&eventPtr->xmotion);
	    break;
	case KeyPress: {
	    int	numBytes;
	    char	keyString[20];

	    keyString[0] = '\0';
	    numBytes = XLookupString(&eventPtr->xkey, keyString, 20, 
				     (KeySym *)NULL, (XComposeStatus *)NULL);
	    if (numBytes == 0) {
		KeySym keySym;

		keyString[0] = '\0';
		keySym = XLookupKeysym(&eventPtr->xkey, 0);
		if (keySym == XK_Shift_L || keySym == XK_Shift_R) {
		    MNShift(&keyString[0]);
		    if (keyString[0] != '\0') {
			numBytes = 1;
		    } else {
			break;
		    }
		}
	    }
	    if (keyString[0] == '?') {
		MNUnShift();
	    }
	    if (isascii(keyString[0])) {
		if ((shcmd[0] = keyString[0]) != '.')
		    lasttext = FALSE;
		shcmd[1] = '\0';
	        FLASH_READY=0;
		MNSHCommand(shcmd);
		FLASH_READY=1;
	    }

	}
    }
}

/*ARGSUSED*/
static
text_sighandler(data, eventPtr)
    ClientData	data;
    XEvent	*eventPtr;
{
    unsigned		dum;

    if (eventPtr->type == Expose && eventPtr->xexpose.count > 0) {
	return;
    }
    (void)XGetGeometry(display, text_sw, (Window *)&dum, 
		 &text_size.r_left, &text_size.r_top,
		 &text_size.r_width, &text_size.r_height, &dum, &dum);
    XSetForeground(display, text_gc, background);
    XFillRectangle(display, text_sw, text_gc, 0, 0, text_size.r_width, 
		   text_size.r_height);
    XSetForeground(display, text_gc, foreground);

    text_putvalue();
}

/*ARGSUSED*/
static
menu_sighandler(data, eventPtr)
    ClientData	data;
    XEvent	*eventPtr;
{
    XGCValues	gcValues;
    unsigned	dum;

    if (eventPtr->type == Expose && eventPtr->xexpose.count > 0) {
	return;
    }
    (void)XGetGeometry(display, menu_sw, (Window *)&dum, &menu_size.r_left,
		 &menu_size.r_top, &menu_size.r_width, &menu_size.r_height,
		 &dum, &dum);
    gcValues.foreground = background;
    gcValues.function = GXcopy;
    XChangeGC(display, menu_gc, GCForeground | GCFunction, &gcValues);
    XFillRectangle(display, menu_sw, menu_gc, 0, 0, menu_size.r_width, 
		   menu_size.r_height);
    XSetForeground(display, menu_gc, foreground);

    MNDisplayMenu();
}

/*ARGSUSED*/
static
pix_sighandler(data, eventPtr)
    ClientData	data;
    XEvent	*eventPtr;
{
    unsigned	dum;

    if (eventPtr->type == Expose && eventPtr->xexpose.count > 0) {
	return;
    }

    GRCurrentSetOn();	/* make current set on */

    if (STERROR) {	/* check for startup error reading .gremlinrc */
	TxPutMsg("error in .gremlinrc");
	STERROR = 0;
    }

    (void)XGetGeometry(display, pix_sw, (Window *)&dum, &pix_size.r_left, 
		 &pix_size.r_top, &pix_size.r_width, &pix_size.r_height,
		 &dum, &dum);

    SHUpdate();
}


/*
 *  One time only start-up initialization.
 */
main_init(file, gremlinrc)
char *file;
char *gremlinrc;
{
    FILE *fp;
    POINT pos;
    int i, fd;
    char *prealname;

    signal(SIGINT, SIG_IGN);			/* ignore interrupts */
    TxInit();					/* text subwindow init */
    MNInitMenu();				/* menu subwindow init */
    (void)PSetPath(".");			/* file search path */
    Editfile = malloc(128);			/* current picture name */

    POINTLIST = PTInit();			/* drawing points */
    BACKPOINT = PTInit();			/* backup point list */

    PICTURE = DBInit();				/* picture database */
    cset = DBInit();				/* current set */
    for (i=0; i<4; ++i)				/* set buffers */
	MEN[i] = DBInit();

    unlist = unback = NULL;			/* undo pointers */
    make_arrowhead();				/* for > command */
    lastcommand = nop;				/* no last command yet */

    STgremlinrc(gremlinrc);			/* .gremlinrc processing */
    GRFontInit();	       /* must be called after CSIZE & CFONT set */

    strcpy(Editfile, file);			/* find edit file */
    if (*file != '\0') {
	fp = POpen(Editfile, &prealname, SEARCH);

	if (fp == NULL) {
	    strcat(namestripe, file);
	    error("creating new file");
	}
	else {
	    fclose(fp);
	    strcat(namestripe, prealname);
	    PICTURE = DBRead(Editfile, &Orientation, &pos);
	    if ((fd = open(prealname, O_WRONLY | O_APPEND)) < 0)
		strcat(namestripe, " (read only)");
	    else
		close(fd);
	}
    }
    else {
	strcat(namestripe, "new file");
    }

    /*
     * Set up the title window.
     */
    Sx_TitleMake(display, title_sw, text_pf, foreground, background, 
		 foreground, (char *)NULL, namestripe, (char *)NULL);
}  /* end main_init */


/*
 *  Make arrowhead element for later drawing.
 */
make_arrowhead()
{
    POINT *pos;

    pos = PTInit();			/* initialize arrowhead template */
    (void) PTMakePoint(0.0, 0.0, &pos);
    (void) PTMakePoint(-5.51, 3.51, &pos);
    (void) PTMakePoint(-3.51, 0.0, &pos);
    (void) PTMakePoint(-5.51, -3.51, &pos);
    (void) PTMakePoint(0.0, 0.0, &pos);
    arhead.type = VECTOR;
    arhead.ptlist = pos;
    arhead.brushf = 0;			/* brush filled in when used */
    arhead.size = 0;
    arhead.textpt = malloc(1);
    *(arhead.textpt) = '\0';
}


/*
 *----------------------------------------------------------------------
 *
 * SetErrorHandler --
 *
 *	Replace the standard X error handler with a new one that
 *	will ignore all "bad window" errors.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	A new error handler is installed.
 *
 *----------------------------------------------------------------------
 */

static int (*savedHandler)();
static void
SetErrorHandler()
{
    extern int ErrorHandler();		/* Forward reference. */
    extern int (*_XErrorFunction)();

    savedHandler = _XErrorFunction;
    _XErrorFunction = ErrorHandler;
}

int
ErrorHandler(display, errorPtr)
    Display *display;			/* Connection to server. */
    XErrorEvent *errorPtr;		/* Event that occurred. */
{
    extern void abort();

    if ((errorPtr->error_code == BadWindow)
	    || (errorPtr->error_code == BadDrawable)) {
	return 0;
    }
    abort();
    return (*savedHandler)(display, errorPtr);
}



/*
 *----------------------------------------------------------------------
 *
 * TimeSubtract --
 *
 *      Subtracts one time value from another.
 *
 * Results:
 *     The difference of the 2 times.
 *
 * Side effects:
 *     None.
 *
 *----------------------------------------------------------------------
 */

static void
TimeSubtract(time1, time2, resultPtr)
    struct timeval		time1;
    struct timeval		time2;
    register struct timeval	*resultPtr;
{
    resultPtr->tv_sec = time1.tv_sec - time2.tv_sec;
    resultPtr->tv_usec = time1.tv_usec - time2.tv_usec;

    if (resultPtr->tv_usec < 0) {
	resultPtr->tv_sec -= 1;
	resultPtr->tv_usec += 1000000;
    }
}

