/*
 * @(#)menu.c	1.2  1/3/85
 *
 * Menu subwindow routines for the SUN Gremlin picture editor.
 *
 * Original by Mark Opperman 
 *
 * Modifications by Yih-Farn Chen (yfchen@monet.berkeley.edu)
 * Modifications by Jim Thompson (james@ernie.berkeley.edu)
 *
 */

#include <errno.h>
#include "gremlin.h"
#include "icondata.h"
#include "menu.h"

/* imports from graphics.c */

extern GRCurrentSetOn();

/*  imports from display.c */
extern DISHide(), DISIcon(), DISOnTop();

/*  imports from short.c */

extern int adj[];
extern SHScaleX();
extern SHScaleY();
extern SHScaleXY();

/* imports from text.c */

extern TxMsgOK();

/* imports from sun.c */

extern pw_box();

/* imports from main.c */

extern Window menu_sw;
extern Window pix_sw;
extern SEQ;                         /* point sequence number         */
extern Artmode;			/* type of point layed down */
extern Adjustment;
extern Alignment;
extern GravityOn;
extern Arrowtype;
extern Curvetype;
extern Defining;
extern menu_fd;
extern pix_fd;
extern XFontStruct *text_pf;
extern struct rect menu_size;
extern ELT *MEN[];
extern SymbolicLines;
extern FLASH_READY;
extern long timeoff_sec;
extern long timeoff_usec;
extern check_cset();

extern (*lastcommand)();	/* previous command */
extern lasttext;		/* TRUE if previous cmd uses text input */

extern inter_index;		/* index of current interactive command */
extern shifted_index;		/* interactive function saved when shifted */
extern INTER_FUNC Normal_act;	/* Normal (non-interactive) pix actions */
extern INTER_FUNC *pix_act;	/* current pix actions */

/* forward references */

extern align_up();
extern align_down();


#define M_CLEAR_POINTS	1
#define M_SHOW_POINTS	2
#define M_GRIPE		3
#define M_POINT 	4
#define M_FLASH 	5

#define M_SCALEX 	1
#define M_SCALEY 	2
#define M_SCALEXY 	3

#define M_CHSTIPPLE1 	11
#define M_CHSTIPPLE2 	12
#define M_CHSTIPPLE3 	13
#define M_CHSTIPPLE4 	14
#define M_CHSTIPPLE5 	15
#define M_CHSTIPPLE6 	16
#define M_CHSTIPPLE7 	17
#define M_CHSTIPPLE8 	18

extern PMRec menu_stipple1_pm;
extern PMRec menu_stipple2_pm;
extern PMRec menu_stipple3_pm;
extern PMRec menu_stipple4_pm;
extern PMRec menu_stipple5_pm;
extern PMRec menu_stipple6_pm;
extern PMRec menu_stipple7_pm;
extern PMRec menu_stipple8_pm;

#ifdef notdef
struct menuitem stipple_items[] = {
    { MENU_GRAPHIC, (caddr_t) &menu_stipple1_pm, M_CHSTIPPLE1},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple2_pm, M_CHSTIPPLE2},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple3_pm, M_CHSTIPPLE3},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple4_pm, M_CHSTIPPLE4},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple5_pm, M_CHSTIPPLE5},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple6_pm, M_CHSTIPPLE6},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple7_pm, M_CHSTIPPLE7},
    { MENU_GRAPHIC, (caddr_t) &menu_stipple8_pm, M_CHSTIPPLE8}
};
				    
struct menu stipple_menu = {
    MENU_IMAGESTRING, "Stipple",
    sizeof(stipple_items) / sizeof(struct menuitem), stipple_items, 0, 0
}; 
					       
struct menu *stipple_menu_stack = &stipple_menu;
#endif

Sx_MenuEntry misc_menu[] = {
    {NULL, "Clear Points", NULL, -1, NULL, (ClientData) M_CLEAR_POINTS},
    {NULL, "Show Points", NULL, -1, NULL, (ClientData) M_SHOW_POINTS},
    {NULL, "Flash On/Off", NULL, -1, NULL, (ClientData) M_FLASH},
    {NULL, "Gripe", NULL, -1, NULL, (ClientData) M_GRIPE},
    {NULL, "Point", NULL, -1, NULL, (ClientData) M_POINT},
};
Menu *misc_menu_ptr;

#define M_EDIT		1
#define M_PATH		2
#define M_READ		3
#define M_SAVE_SET	4
#define M_WRITE		5
#define M_CD		6
#define M_QUIT		7

Sx_MenuEntry file_items[] = {
    { NULL, "Edit", NULL, -1, NULL, (ClientData) M_EDIT},
    { NULL, "Read", NULL, -1, NULL, (ClientData) M_READ},
    { NULL, "Save Set", NULL, -1, NULL, (ClientData) M_SAVE_SET},
    { NULL, "Write", NULL, -1, NULL, (ClientData) M_WRITE},
    { NULL, "Path", NULL, -1, NULL, (ClientData) M_PATH},
    { NULL, "Directory", NULL, -1, NULL, (ClientData) M_CD},
    { NULL, "Quit", NULL, -1, NULL, (ClientData) M_QUIT}
}; 
Menu *file_menu_ptr;

Sx_MenuEntry scale_items[] = {
    { NULL, "Scale XY", NULL, -1, NULL, (ClientData) M_SCALEXY},
    { NULL, "Scale X", NULL, -1, NULL, (ClientData) M_SCALEX},
    { NULL, "Scale Y", NULL, -1, NULL, (ClientData) M_SCALEY},
}; 
Menu *scale_menu_ptr;
								      
int icon_x, icon_y;	/* location of currently selected menu icon */
int icon_last = -1;	/* menu index of currently selected menu icon */
int just_x, just_y;	/* location of justify marker within justify icon */
int just_last = -1;	/* justify index of currently selected mode */
PMRec save_just_pm;

/* imports from main.c */

extern CFONT;
extern CSIZE;
extern CJUST;
extern CBRUSH;
extern CSTIPPLE;

/* imports from help.c */

extern justify_help(), help(), horizontal_help(), vertical_help(), 
       size1_help(), size2_help(), size3_help(), size4_help(), 
       roman_help(), italics_help(), bold_help(), special_help(),
       scale_help(), move_help(), hmirror_help(), vmirror_help(), 
       include_help(), linestyle_help(), align_help(), pan_help(),
       polygon_help(), bpolygon_help(),
       get1_help(), get2_help(), get3_help(), get4_help(), 
       put1_help(), put2_help(), put3_help(), put4_help(), 
       brush1_help(), brush2_help(), brush3_help(),
       brush4_help(), brush5_help(), brush6_help(), 
       stipple1_help(), stipple2_help(), stipple3_help(), stipple4_help(), 
       stipple5_help(), stipple6_help(), stipple7_help(), stipple8_help(), 
       copy_help(), arrow_help(), text_help(), misc_help(), undo_help(),
       erase_help(), movepoint_help(), rotate_help(), filecabinet_help(),
       boxinc_help(), manhattan_help(), gravity_help(), arc_help(),
       curve_help(), vector_help(), box_help(), grid_help(), 
       littlepoint_help(), menusw_help(), pixsw_help(), short_help();

/* imports from sunlong.c */

extern LGBrush1(), LGBrush2(), LGBrush3(), 
       LGBrush4(), LGBrush5(), LGBrush6(),
       LGChangeStippleMap1(), LGChangeStippleMap2(),
       LGChangeStippleMap3(), LGChangeStippleMap4(),
       LGChangeStippleMap5(), LGChangeStippleMap6(),
       LGChangeStippleMap7(), LGChangeStippleMap8(),
       LGClearPoints(), LGPath(), LGChDir(), LGEdit(),
       LGFont1(), LGFont2(), LGFont3(), LGFont4(), 
       LGGet1(), LGGet2(), LGGet3(), LGGet4(), 
       LGGripe(), LGHMirror(), 
       LGHAdjust(), LGIncludeSet(), LGLineStyle(), LGLittlePoint(), 
       LGMBrush1(), LGMBrush2(), LGMBrush3(), 
       LGMBrush4(), LGMBrush5(), LGMBrush6(), LGMCurve(),
       LGMFont1(), LGMFont2(), LGMFont3(), LGMFont4(),
       LGMJustify(), LGMPan(), LGQuit(),
       LGMSize1(), LGMSize2(), LGMSize3(), LGMSize4(), 
       LGMStipple1(), LGMStipple2(), LGMStipple3(), LGMStipple4(), 
       LGMStipple5(), LGMStipple6(), LGMStipple7(), LGMStipple8(), 
       LGMText(), LGMPoint(), LGMVector(), nop(), LGPan(),
       LGPolygon(), LGBPolygon(), LGMPolygon(), LGMBPolygon(),
       LGOPoint(), LGRead(), LGSave(), LGShowPoints(), 
       LGSize1(), LGSize2(), LGSize3(), LGSize4(), 
       LGStipple1(), LGStipple2(), LGStipple3(), LGStipple4(), 
       LGStipple5(), LGStipple6(), LGStipple7(), LGStipple8(),
       LGText(), LGUndo(), LGVAdjust(), LGVMirror(), LGWrite();


/* imports from short.c */

extern SHArrow(), SHBox(), SHCopy(), SHDefineSet(), SHDrawArc(), SHDrawCurve(), 
       SHDrawVector(), SHErase(), SHGravity(), SHGrid(), SHMAdjust(), 
       SHSave1(), SHSave2(), SHSave3(), SHSave4(), 
       SHRotate(), SHScale(), SHSetArea(), SHMSetArea(), SHTranslate();
extern SHUpdate(), SHAgain();

/* imports from inter.c */
extern INTER_FUNC INBox,INArc, INPoly, INBPoly, INVect, INCurve, INArrow,
	INDefine, INTrans, INCopy, INMovePt;

/* justify_mark marks the text handle positions for the JUSTIFY icon
   relative to the icon's upper left corner */

struct pm_pos justify_mark[9];

/* just_to_indx maps the justification values (TOPLEFT, ..., BOTRIGHT)
   to the corresponding index in the justify_mark array */

int just_to_indx[] = { 6, 8, 4, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 5, 7 };

/* indx_to_just maps the internal justification values (0, ..., 8)
   to the gremlin justification values (TOPLEFT, ..., BOTRIGHT) */

int indx_to_just[] = { TOPLEFT, TOPCENT, TOPRIGHT, 
		       CENTLEFT, CENTCENT, CENTRIGHT, 
		       BOTLEFT, BOTCENT, BOTRIGHT };

/* alternate icons for arrowhead */
PMRec *arrow_icons[] = {&arrowhead_pm, &arrow_pm, &dblarrow_pm};

/* alternate icons for curves */
PMRec *curve_icons[] = {&curve_pm, (PMRec *)0, &curve_B_pm, &curve_Bz_pm};
/* names of curve types */
char *curve_names[] = {"Interpolated Spline", /* CURVE type */
		       "   ",			/* dummy for POLYGON */
		       "B-Spline", 		/* B_SPLINE type */
		       "Bezier Curve" }; 	/* BEZIER type */

struct _menu menu[] = {
#define M_SIZE1 0
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP +  ICON_BORDER + MENU_Y * 1, &size1_pm, 
      LGSize1, LGMSize1, size1_help },

#define M_ROMAN (M_SIZE1+1)
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 2, &roman_pm, 
      LGFont1, LGMFont1, roman_help },

#define M_JUSTIFY (M_ROMAN+1)
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 1, &justify_pm, 
      nop, nop, nop },

#define M_SCALE (M_JUSTIFY+1)
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 9, &scale_pm, 
      SHScale, nop, scale_help },

#define M_TRANSLATE (M_SCALE+1)
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 8, &move_pm, 
      SHTranslate, nop, move_help, &INTrans },

#define M_HMIRROR (M_TRANSLATE+1)
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 11, &hmirror_pm, 
      LGHMirror, nop, hmirror_help },

#define M_VMIRROR (M_HMIRROR+1)
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 11, &vmirror_pm, 
      LGVMirror, nop, vmirror_help },

#define M_DEFINE_SET (M_VMIRROR+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 14, &include_pm, 
      SHDefineSet, LGIncludeSet, include_help },

#define M_PUT1 (M_DEFINE_SET+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 15, &put1_pm, 
      SHSave1, nop, put1_help },

#define M_PUT3 (M_PUT1+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 16, &put3_pm, 
      SHSave3, nop, put3_help },

#define M_HORZ_ADJUST (M_PUT3+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 17, &horizontal_pm, 
      LGHAdjust, nop, horizontal_help },

#define M_VERT_ADJUST (M_HORZ_ADJUST+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 18, &vertical_pm, 
      LGVAdjust, nop, vertical_help },

#define M_STIPPLE1 (M_VERT_ADJUST+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 3, &white_pm, 
      LGStipple1, LGMStipple1, stipple1_help },

#define M_STIPPLE5 (M_STIPPLE1+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 3, &stipple5_pm, 
      LGStipple5, LGMStipple5, stipple5_help },

#define M_SIZE2 (M_STIPPLE5+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 1, &size2_pm, 
      LGSize2, LGMSize2, size2_help },

#define M_ITALICS (M_SIZE2+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 2, &italics_pm, 
      LGFont2, LGMFont2, italics_help },

#define M_COPY (M_ITALICS+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 8, &copy_pm, 
      SHCopy, nop, copy_help, &INCopy },

#define M_ERASE (M_COPY+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 9, &erase_pm, 
      SHErase, nop, erase_help },

#define M_MOVE_POINT (M_ERASE+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 10, &movepoint_pm, 
      LGMPoint, nop, movepoint_help, &INMovePt },

#define M_ROTATE (M_MOVE_POINT+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 10, &rotate_pm, 
      SHRotate, nop, rotate_help },

#define M_FILE (M_ROTATE+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 13, &filecabinet_pm, 
      nop, nop, filecabinet_help },

#define M_PAN (M_FILE+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 14, &pan_pm, 
      LGPan, LGMPan, pan_help },

#define M_SET_AREA (M_PAN+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 14, &boxinc_pm, 
      SHSetArea, SHMSetArea, boxinc_help, &INDefine },

#define M_PUT2 (M_SET_AREA+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 15, &put2_pm, 
      SHSave2, nop, put2_help },

#define M_PUT4 (M_PUT2+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 16, &put4_pm, 
      SHSave4, nop, put4_help },

#define M_MANHATTAN_ADJUST (M_PUT4+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 17, &horvert_pm, 
      SHMAdjust, nop, manhattan_help },

#define M_GRAVITY (M_MANHATTAN_ADJUST+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 18, &gravity_pm, 
      SHGravity, nop, gravity_help },

#define M_STIPPLE6 (M_GRAVITY+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 4, &stipple6_pm, 
      LGStipple6, LGMStipple6, stipple6_help },

#define M_STIPPLE2 (M_STIPPLE6+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 4, &gray_pm, 
      LGStipple2, LGMStipple2, stipple2_help },

#define M_SIZE3 (M_STIPPLE2+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 2, &size3_pm, 
      LGSize3, LGMSize3, size3_help },

#define M_BOLD (M_SIZE3+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 3, &bold_pm, 
      LGFont3, LGMFont3, bold_help },

#define M_BRUSH1 (M_BOLD+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 4, &dotted_pm, 
      LGBrush1, LGMBrush1, brush1_help },

#define M_BRUSH2 (M_BRUSH1+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 6, &broken_pm, 
      LGBrush2, LGMBrush2, brush2_help },

#define M_BRUSH3 (M_BRUSH2+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 6, &thick_pm, 
      LGBrush3, LGMBrush3, brush3_help },

#define M_TEXT (M_BRUSH3+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 8, &text_pm, 
      LGText, LGMText, text_help },

#define M_BPOLYGON (M_TEXT+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 9, &bpolygon_pm, 
      LGBPolygon, LGMBPolygon, bpolygon_help, &INBPoly },

#define M_POLYGON (M_BPOLYGON+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 10, &polygon_pm, 
      LGPolygon, LGMPolygon, polygon_help, &INPoly },

#define M_ARROW (M_POLYGON+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 11, &arrowhead_pm, 
      SHArrow, nop, arrow_help, &INArrow },

#define M_HELP (M_ARROW+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 13, &question_pm, 
      help, short_help, help },

#define M_MISC (M_HELP+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 14, &misc_pm, 
      nop, nop, misc_help },

#define M_GET1 (M_MISC+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 15, &get1_pm, 
      LGGet1, nop, get1_help },

#define M_GET3 (M_GET1+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 16, &get3_pm, 
      LGGet3, nop, get3_help },

#define M_LINESTYLE (M_GET3+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 13, &linestyle_pm, 
      LGLineStyle, nop, linestyle_help },

#define M_ALIGN (M_LINESTYLE+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 17, &align_pm, 
      nop, nop, align_help },

#define M_STIPPLE3 (M_ALIGN+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 5, &_50_pm, 
      LGStipple3, LGMStipple3, stipple3_help },

#define M_STIPPLE7 (M_STIPPLE3+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 5, &stipple7_pm, 
      LGStipple7, LGMStipple7, stipple7_help },

#define M_STIPPLE8 (M_STIPPLE7+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 6, &stipple8_pm, 
      LGStipple8, LGMStipple8, stipple8_help },

#define M_STIPPLE4 (M_STIPPLE8+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 0, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 6, &black_pm, 
      LGStipple4, LGMStipple4, stipple4_help },

#define M_SIZE4 (M_STIPPLE4+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 1, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 2, &size4_pm, 
      LGSize4, LGMSize4, size4_help },

#define M_SPECIAL (M_SIZE4+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 3, &special_pm, 
      LGFont4 , LGMFont4, special_help },

#define M_BRUSH4 (M_SPECIAL+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3,
      MENU_TOP  + ICON_BORDER + MENU_Y * 5, &dashed_pm, 
      LGBrush4, LGMBrush4, brush4_help },

#define M_BRUSH5 (M_BRUSH4+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 4, &narrow_pm, 
      LGBrush5, LGMBrush5, brush5_help },

#define M_BRUSH6 (M_BRUSH5+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 5, &medium_pm, 
      LGBrush6, LGMBrush6, brush6_help },

#define M_DRAW_ARC (M_BRUSH6+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 8, &arc_pm, 
      SHDrawArc, nop, arc_help, &INArc },

#define M_DRAW_CURVE (M_DRAW_ARC+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 9, &curve_pm, 
      SHDrawCurve, LGMCurve, curve_help, &INCurve },

#define M_DRAW_VECTOR (M_DRAW_CURVE+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 10, &vector_pm, 
      SHDrawVector, LGMVector, vector_help, &INVect },

#define M_BOX (M_DRAW_VECTOR+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 11, &box_pm, 
      SHBox, nop, box_help, &INBox },

#define M_UNDO (M_BOX+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 13, &undo_pm, 
      LGUndo, nop, undo_help },

#define M_GET2 (M_UNDO+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 15, &get2_pm, 
      LGGet2, nop, get2_help },

#define M_GET4 (M_GET2+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 16, &get4_pm, 
      LGGet4, nop, get4_help },

#define M_GRID (M_GET4+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 3, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 18, &grid_pm, 
      SHGrid, nop, grid_help },

#define M_LITTLE_POINT (M_GRID+1) 
    { MENU_LEFT + ICON_BORDER + MENU_X * 2, 
      MENU_TOP  + ICON_BORDER + MENU_Y * 18, &littlepoint_pm, 
      LGLittlePoint, nop, littlepoint_help },

    { -1, -1, (PMRec *)0, (int(*)())0 }
};

/*
 * The following are pseudo-icon indices for commands which
 * do not have an icon (mostly short commands).
 */
#define M_UPDATE (M_LITTLE_POINT+1)
#define M_AGAIN  (M_UPDATE+1)
#define M_HIDE   (M_AGAIN+1)
#define M_ICONIC (M_HIDE+1)
#define M_ONTOP  (M_ICONIC+1)
#define M_NO_INTER (M_ONTOP+1)
#define M_NO_CURR  (M_NO_INTER+1)

/* 
 * Following is a map of brushes and mode made available to the
 * outside world for selection of these items for highlighting.
 * The numbers indicate the items location in the menu.
 */
    
int HiBuffer[] = { M_PUT1, M_PUT2, M_PUT3, M_PUT4 };	/* user symbols */
int HiFont[] = { M_ROMAN, M_ITALICS, M_BOLD, M_SPECIAL }; 
int HiSize[] = { M_SIZE1, M_SIZE2, M_SIZE3, M_SIZE4 }; 
int HiBrush[] = { M_BRUSH1, M_BRUSH2, M_BRUSH3, 
		   M_BRUSH4, M_BRUSH5, M_BRUSH6 };	/* brushes */
int HiMode[] = { M_HORZ_ADJUST, M_VERT_ADJUST, M_MANHATTAN_ADJUST, 
		  M_GRAVITY };			/* horz, vert, man, grav. */
int HiStipple[] = { M_STIPPLE1, M_STIPPLE2, M_STIPPLE3, M_STIPPLE4, 
		    M_STIPPLE5, M_STIPPLE6, M_STIPPLE7, M_STIPPLE8 }; 
int HiArtMode = M_LITTLE_POINT;
int HiLineStyle = M_LINESTYLE;


/* 
 * The following two arrays define the short commands and the routines
 * that process them.
 */
static char shcmds[] = { '\14', '.', '1', '2', '3', '4', '?', 'a', 'b',
                          'c', 'd', 'e', 'f', 'g', 'h', 'i', 'o', 'q', 'r',
                          's', 't', 'u', 'v', 'w', 'x', 'z', 'n', 'D', '\0'};

static shrtns[] = {
    M_UPDATE,			/* redraw screen */
    M_AGAIN,			/* repeat last command */
    M_PUT1,                     /* save user symbol */
    M_PUT2,                    /* save user symbol */
    M_PUT3,                    /* save user symbol */
    M_PUT4,                    /* save user symbol */
    M_HELP,			/* help screen */
    M_DRAW_ARC,			/* draw arc */
    M_DRAW_CURVE,		/* DRAW CURVE */
    M_COPY,			/* COPY CURRENT SET */
    M_DEFINE_SET,		/* DEFINE CURRENT SET */
    M_ERASE,			/* ERASE ELEMENTS */
    M_SET_AREA,			/* SELECT AREA FOR CURRENT SET */
    M_GRAVITY,			/* GRAVITY */
    M_HIDE,                     /* HIDE THE GREMLIN WINDOW */
    M_ICONIC,                     /* CLOSE THE WINDOW */
    M_ONTOP,                    /* ON TOP OF OTHER WINDOWS */
    M_GRID,			/* TOGGLE GRID DISPLAY */
    M_ROTATE,			/* ROTATE CURRENT SET */
    M_SCALE,			/* SCALE CURRENT SET IN X AND Y DIRECTIONS */
    M_TRANSLATE,		/* TRANSLATE CURRENT SET */
    M_UNDO,			/* UNDO LAST COMMAND */
    M_DRAW_VECTOR,		/* DRAW VECTORS */
    M_ARROW,                    /* ARROWHEAD */
    M_BOX,                      /* RECTANGLE FROM TWO POINTS */
    M_MANHATTAN_ADJUST,		/* MANHATTAN ADJUST */
    M_NO_INTER,			/* go non-interactive */
    M_NO_CURR,			/* no current set */
    0
};

Sx_MenuEntry inter_items[] = {
    { NULL, "Current Set", NULL, -1, NULL, (ClientData) M_SET_AREA},
    { NULL, "Move", NULL, -1, NULL, (ClientData) M_TRANSLATE},
    { NULL, "Copy", NULL, -1, NULL, (ClientData) M_COPY},
    { NULL, "Modify", NULL, -1, NULL, (ClientData) M_MOVE_POINT},
    { NULL, "Erase", NULL, -1, NULL, (ClientData) M_ERASE},
    { NULL, "No Mode", NULL, -1, NULL, (ClientData) M_NO_INTER},
    { NULL, "Help", NULL, -1, NULL, (ClientData) M_HELP}
}; 
Menu *inter_menu_ptr;

/*
 * This routine initializatizes the positioning of menu items.
 */
/*ARGSUSED*/
MNInitMenu()
{
    int justify_x, justify_y;
    int i;
    XGCValues gcValues;

    misc_menu_ptr = Menu_Create(display, root_window, 5, misc_menu, text_pf,
		           	 foreground, background);
    file_menu_ptr = Menu_Create(display, root_window, 7, file_items, text_pf,
		           	 foreground, background);
    scale_menu_ptr = Menu_Create(display, root_window, 3, scale_items, text_pf,
		           	 foreground, background);
    inter_menu_ptr = Menu_Create(display, root_window, 7, inter_items, text_pf,
		           	 foreground, background);

    justify_x = menu[M_JUSTIFY].menu_x;
    justify_y = menu[M_JUSTIFY].menu_y;

    for (i=0; i<9; i++) {
	switch (i % 3) {
	    case 0:		/* left */
		justify_mark[i].x = justify_x - 1;
		break;
	    case 1:		/* center */
		justify_mark[i].x = justify_x + 18;
		break;
	    case 2:		/* right */
		justify_mark[i].x = justify_x + 35;
		break;
	}

	switch (i / 3) {
	    case 0:		/* top */
		justify_mark[i].y = justify_y - 2;
		break;
	    case 1:		/* center */
		justify_mark[i].y = justify_y + 5;
		break;
	    case 2:		/* bottom */
		justify_mark[i].y = justify_y + 12;
		break;
	}
    }
    /* Create an image of the Justification select region */

    save_just_pm.pm = XCreatePixmap(display, root_window, 44, 20, 1);
    save_just_pm.width = 44;
    save_just_pm.height = 20;
    gcValues.foreground = 0;
    gcValues.function = GXcopy;
    XChangeGC(display, pixmap_gc, GCForeground | GCFunction, &gcValues);
    XFillRectangle(display, save_just_pm.pm, pixmap_gc, 0, 0, 44, 20);
    XSetForeground(display, pixmap_gc, foreground);
    XCopyArea(display, justify_pm.pm, save_just_pm.pm, pixmap_gc, 0, 0,
	      40, 16, 1, 2);
}  /* end MNInitMenu */


/*
 * This routine displays the menu defined by initmenu
 */
MNDisplayMenu()
{
    register i;
    register struct _menu *m;
    char buf[4];

    XSetFunction(display, menu_gc, drawOP);
    XDrawString(display, menu_sw, menu_gc, 28, 
		menu[M_SIZE1].menu_y - MENU_Y + TEXT_BASELINE,
		"styles", 6);
    XDrawString(display, menu_sw, menu_gc, 24, 
		menu[M_DRAW_ARC].menu_y - MENU_Y + TEXT_BASELINE,
		"drawing", 7);
    XDrawString(display, menu_sw, menu_gc, 32, 
		menu[M_FILE].menu_y - MENU_Y + TEXT_BASELINE,
		"other", 5);
    XDrawRectangle(display, menu_sw, menu_gc, 2, 2,
		   MENUSW_WIDTH - 3, 
		   (unsigned)menu[M_BRUSH2].menu_y + ICON_SIZE + 3);
    XDrawRectangle(display, menu_sw, menu_gc, 
		   2, menu[M_DRAW_ARC].menu_y - MENU_Y - 2,
		   MENUSW_WIDTH - 3, 
		   (unsigned)menu[M_BOX].menu_y + ICON_SIZE + 3);
    XDrawRectangle(display, menu_sw, menu_gc, 
		   2, menu[M_FILE].menu_y - MENU_Y - 2, 
		   MENUSW_WIDTH - 3, 509);

    m = &menu[M_DRAW_CURVE];
    m->pm_ptr = curve_icons[Curvetype-CURVE];
    
    i = 0;
    do {
	m = &menu[i];
	XSetFunction(display, menu_gc, GXcopy);
	XCopyPlane(display, m->pm_ptr->pm, menu_sw, menu_gc, 0, 0,
		  m->pm_ptr->width, m->pm_ptr->height,
		  m->menu_x, m->menu_y, (unsigned long)1);
	if (is_stipple(m->pm_ptr)) {
	    XSetFunction(display, menu_gc, drawOP);
	    XCopyPlane(display, openbox_pm.pm, menu_sw, menu_gc, 
		      0, 0, openbox_pm.width, openbox_pm.height, 
		      m->menu_x, m->menu_y, (unsigned long)1);
	}
    } while (menu[++i].menu_x != -1);

    if (Adjustment != NOADJ)
	MNHighLt(HiMode[adj[Adjustment]]);

    if (GravityOn)
	MNHighLt(HiMode[3]);

    if (!SymbolicLines)
	MNHighLt(HiLineStyle);

    if (!Artmode)
	MNHighLt(HiArtMode);

    if (inter_index)
	MNHighLt(inter_index);

    MNHighLt(HiFont[CFONT-1]);
    MNHighLt(HiSize[CSIZE-1]);
    MNHighLt(HiBrush[CBRUSH-1]);
    MNHighLt(HiStipple[CSTIPPLE-1]);

    XCopyPlane(display, dot_pm.pm, menu_sw, menu_gc, 0, 0, 5, 5,
	      justify_mark[just_to_indx[CJUST]].x + 1,
	      justify_mark[just_to_indx[CJUST]].y, (unsigned long)1);

    for (i = 0; i<NUSER; ++i) {
	if (!DBNullelt(MEN[i]))
	    MNHighLt(HiBuffer[i]);
    }
    sprintf(buf, "%3d", Alignment);
    XDrawString(display, menu_sw, menu_xor_gc, menu[M_ALIGN].menu_x + 16, 
		menu[M_ALIGN].menu_y + 12, buf, strlen(buf));
}


menu_left(eventPtr)
    XButtonEvent	*eventPtr;
{
    register index;

    TxMsgOK();
    lasttext = FALSE;

    if ((index = icon_last) >= 0) {
	if (index == M_ALIGN) {
	    lastcommand = align_up;
	    align_up();
	    return;
	}

	/* current set on, un-highlight icon */
	GRCurrentSetOn();
	MNReverse(index);

	if (index == M_FILE) {
	    icon_last = -1;
	    menu_filer(eventPtr);
	    return;
	}

	if (index == M_MISC) {
	    menu_misc(eventPtr);
	    icon_last = -1;
	    return;
	}

	if (index == M_SCALE) {
	    menu_scale(eventPtr);
	    icon_last = -1;
	    return;
	}

	if ((menu[index].inter_act) && (SEQ == 0)) {	/* go interactive */
	    select_inter(icon_last);
	    icon_last = -1;
	    return;
	}

	lastcommand = menu[index].menu_function;
	if (index == M_TEXT)
	    lasttext = TRUE;
	(*lastcommand)();
	/* (*(menu[index].menu_function))(); */
	icon_last = -1;
	FLASH_READY = 0;	/* don't flash; it may have taken a while */
	return;
    }

    if (just_last >= 0) {	/* in JUSTIFY icon */
	/* un-highlight */
	MNReverseJustify();

        /* Redraw the Justify Icon */
	XSetFunction(display, menu_gc, GXcopy);
	XCopyPlane(display, save_just_pm.pm, menu_sw, menu_gc, 0, 0, 44, 20,
		  justify_mark[0].x, justify_mark[0].y, (unsigned long)1);
	CJUST = indx_to_just[just_last];
	XCopyPlane(display, dot_pm.pm, menu_sw, menu_gc, 0, 0, 5, 5,
		  justify_mark[just_to_indx[CJUST]].x + 1,
		  justify_mark[just_to_indx[CJUST]].y, (unsigned long)1);
	just_last = -1;
    }
}


menu_middle(eventPtr)
    XButtonEvent	*eventPtr;
{
    register index;
    register struct _menu *m;

    TxMsgOK();
    lasttext = FALSE;

    if ((index = icon_last) >= 0) {
	if (index == M_ALIGN) {
	    lastcommand = align_down;
	    align_down();
	    return;
	}

	/* current set on, un-highlight icon */
	GRCurrentSetOn();
	MNReverse(index);
	icon_last = -1;

	if (index == M_MISC) {
	    menu_chstipple(eventPtr);
	    return;
	}

	if (index == M_ARROW) {		/* change type of arrow */
	    m = &menu[M_ARROW];
	    if ((++Arrowtype) > 2) Arrowtype = 0;
	    m->pm_ptr = arrow_icons[Arrowtype];
	    XSetFunction(display, menu_gc, GXcopy);
	    XCopyPlane(display, m->pm_ptr->pm, menu_sw, menu_gc,
	              0, 0, m->pm_ptr->width, m->pm_ptr->height,
		      m->menu_x, m->menu_y, (unsigned long)1);
	    return;
	}
	if ((index == M_DRAW_CURVE) && (eventPtr->state & ShiftMask)) {
	    m = &menu[M_DRAW_CURVE];
	    if (Curvetype==CURVE) Curvetype = B_SPLINE;
	    else if (Curvetype==B_SPLINE) Curvetype = BEZIER;
	    else Curvetype = CURVE;
	    m->pm_ptr = curve_icons[Curvetype-CURVE];
	    XSetFunction(display, menu_gc, GXcopy);
	    XCopyPlane(display, m->pm_ptr->pm, menu_sw, menu_gc,
	              0, 0, m->pm_ptr->width, 
		      m->pm_ptr->height, m->menu_x, m->menu_y,
		      (unsigned long)1);
	    TxMsgOK();
	    TxPutMsg(curve_names[Curvetype-CURVE]);
	    return;
	}

	lastcommand = menu[index].menu_modify;
	(*lastcommand)();
	if (index == M_TEXT)
	    lasttext = TRUE;
	/* (*(menu[index].menu_modify))(); */
	FLASH_READY = 0;	/* don't flash; it may have taken a while */
	return;
    }

    if ((index = just_last) >= 0) {
	just_last = -1;

	/* current set on, un-highlight icon */
	GRCurrentSetOn();
	MNReverseJustify();

	LGMJustify(indx_to_just[index]);
    }
}

/*ARGSUSED*/
menu_right(eventPtr)
    XButtonEvent	*eventPtr;
{
    register index;

    TxMsgOK();

    if ((index = icon_last) >= 0) {
	icon_last = -1;

	/* un-highlight icon */
	MNReverse(index);

	(*(menu[index].menu_help))();
	return;
    }

    if ((index = just_last) >= 0) {
	just_last = -1;

	/* un-highlight icon */
	MNReverseJustify();

	justify_help();
	return;
    }

    menusw_help();
}

/*ARGSUSED*/
menu_winexit(eventPtr)
    XCrossingEvent	*eventPtr;
{
    if (icon_last >= 0) {
	MNReverse(icon_last);
	icon_last = -1;
    }
    else if (just_last >= 0) {
	MNReverseJustify();
	just_last = -1;
    }

    GRCurrentSetOn();
}

/*
 * Select a new interactive mode
 */
select_inter(index)
int index;
{
	if (inter_index)			/* already interactive */
	    MNUnHighLt(inter_index);	/* unhighlight current mode */

	if (inter_index == index) {	/* just go non-interactive */
	    inter_index = 0;
	    pix_act = &Normal_act;
	} else {				/* begin interactive mode */
	    inter_index = index;
	    pix_act = menu[index].inter_act;	/* pix actions */
	    MNHighLt(inter_index);			/* highlight mode */
	}
}

/*
 * Move align to next higher value.
 */
align_up()
{
    menu_align(TRUE);
}


/*
 * Move align to next lower value.
 */
align_down()
{
    menu_align(FALSE);
}


#define ALIGNMAX 512
menu_align(up)
int up;
{
    char buf[4];


    sprintf(buf, "%3d", Alignment);
    XDrawString(display, menu_sw, menu_xor_gc, menu[M_ALIGN].menu_x + 16, 
		menu[M_ALIGN].menu_y + 12, buf, strlen(buf));

    if (up) {
	if ((Alignment <<= 1) > ALIGNMAX)
	    Alignment = 1;
    }
    else {
	if ((Alignment >>= 1) == 0)
	    Alignment = ALIGNMAX;
    }

    sprintf(buf, "%3d", Alignment);
    XDrawString(display, menu_sw, menu_xor_gc, menu[M_ALIGN].menu_x + 16, 
		menu[M_ALIGN].menu_y + 12, buf, strlen(buf));
}

/*ARGSUSED*/
menu_filer(eventPtr)
    XButtonEvent	*eventPtr;
{
    register	int	item;

    item = (int)Menu_PullMenuDown(file_menu_ptr, 
				  eventPtr->x_root, eventPtr->y_root);
    if (item < 0) {
	return;
    }

    switch (item) {
	case M_EDIT:
	    lastcommand = LGEdit;
	    break;
	case M_PATH:
	    lastcommand = LGPath;
	    break;
	case M_CD:
	    lastcommand = LGChDir;
	    break;
	case M_READ:
	    lastcommand = LGRead;
	    break;
	case M_SAVE_SET:
	    lastcommand = LGSave;
	    break;
	case M_WRITE:
	    lastcommand = LGWrite;
	    break;
	case M_QUIT:
	    lastcommand = LGQuit;
	    break;
	default:
	    return;
    }
    (*lastcommand)();
    lasttext = TRUE;
}

/*ARGSUSED*/
menu_misc(eventPtr)
    XButtonEvent	*eventPtr;
{
    int	item;

    item = (int)Menu_PullMenuDown(misc_menu_ptr, 
				  eventPtr->x_root, eventPtr->y_root);
    if (item < 0) {
	return;
    }

    switch (item) {
	case M_CLEAR_POINTS:
	    lastcommand = LGClearPoints;
	    break;
	case M_SHOW_POINTS:
	    lastcommand = LGShowPoints;
	    break;
	case M_GRIPE:
	    lastcommand = LGGripe;
	    break;
	case M_POINT:
	    lastcommand = LGOPoint;
	    lasttext = TRUE;
	    break;
	case M_FLASH:
	    if (timeoff_usec == 0) {  /* no flash */
		timeoff_usec = 400000;
		timeoff_sec = 0;
		GRCurrentSet();
	    } else {
		timeoff_usec = 0;
		GRCurrentSetOn();
	    }
	    return;
	default:
	    return;
    }
    (*lastcommand)();
}

menu_inter(eventPtr)
    XButtonEvent	*eventPtr;
{
    int	item;

    item = (int)Menu_PullMenuDown(inter_menu_ptr, 
				  eventPtr->x_root, eventPtr->y_root);

    if (item < 0)
	return;

    switch (item) {
	case M_GRAVITY:
	    SHGravity();
	    break;
	case M_ERASE:
	    SHErase();
	    break;
	case M_NO_INTER:
	    select_inter(inter_index);
	    break;
	case M_SET_AREA:
	case M_MOVE_POINT:
	case M_COPY:
	case M_TRANSLATE:
	    CP();
	    if (inter_index != item) /* not already selected */
	      select_inter(item); /* select interactive mode */
	    break;
	case M_HELP:
	    if (inter_index)
		(*(menu[inter_index].menu_help))(); /* specific help */
	    else
		pixsw_help();		/* non-interactive help */
	    break;
    }
}

/*ARGSUSED*/
menu_scale(eventPtr)
    XButtonEvent	*eventPtr;
{
    int item;

    item = (int)Menu_PullMenuDown(scale_menu_ptr, 
				  eventPtr->x_root, eventPtr->y_root);
    if (item < 0) {
	return;
    }

    switch (item) {
        case M_SCALEX:
	    lastcommand = SHScaleX;
	    break;
        case M_SCALEY:
	    lastcommand = SHScaleY;
	    break;
        case M_SCALEXY:
	    lastcommand = SHScaleXY;
	    break;
	default:
	    return;
    }
    (*lastcommand)();
}

/*
 *  dynamically change the stipple mappings
 */
/*ARGSUSED*/
menu_chstipple(eventPtr)
    XButtonEvent	*eventPtr;
{
#ifdef notdef
    register struct menuitem *item;

    item = menu_display(&stipple_menu_stack, ie, menu_fd);

    if (item == (struct menuitem *) NULL)
	return;

    switch (item->mi_data) {
	case M_CHSTIPPLE1:
	    lastcommand = LGChangeStippleMap1;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE2:
	    lastcommand = LGChangeStippleMap2;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE3:
	    lastcommand = LGChangeStippleMap3;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE4:
	    lastcommand = LGChangeStippleMap4;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE5:
	    lastcommand = LGChangeStippleMap5;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE6:
	    lastcommand = LGChangeStippleMap6;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE7:
	    lastcommand = LGChangeStippleMap7;
	    lasttext=TRUE;
	    break;
	case M_CHSTIPPLE8:
	    lastcommand = LGChangeStippleMap8;
	    lasttext=TRUE;
	    break;
	default:
	    return;
    }
    (*lastcommand)();
#endif
}
  

/*
 * MNSHLookup searches a table of characters to find one that matches a
 * the given character.
 * If c is a valid command character, its index is returned;
 * if c is the null command, -2 is returned, else -1 is returned.
 */
MNSHLookup(c, table)
char c;				/* char to be looked up */
register char table[];		/* pointer to the valid commands */
{
    register index;

    if ((c == ' ') || (c == '\0')) 
	return(-2);

    for (index=0; table[index] != '\0'; index++) {
        if (table[index] == c) 
	    return(index);
    }

    return(-1);
}  /* end MNSHLookup */


/*
 * This routine reads in, looks up, and executes a short command.
 */
MNSHCommand(command)
register char *command;
{
    register index,menuindex;

    if ((index = MNSHLookup(*command, shcmds)) == -2) 
	return;

    if (index >= 0) {
	GRCurrentSetOn();
	TxMsgOK();
	menuindex = shrtns[index];
	switch (menuindex) {
	case M_UPDATE:
	    SHUpdate();
	    break;
	case M_AGAIN:
	    SHAgain();
	    break;
	case M_ICONIC:
	    DISIcon();
	    break;
	case M_HIDE:
	    DISHide();
	    break;
	case M_ONTOP:
	    DISOnTop();
	    break;
	case M_SCALE:
	    MNReverse(M_SCALE);
	    SHScaleXY();
	    MNReverse(M_SCALE);
	    break;
	case M_NO_INTER:
	    select_inter(inter_index); /* turn off any interactive mode */
	    break;
	case M_NO_CURR:
	    DISClearSetDisplay();
            DBClearSet();
	    break;
	case M_GRAVITY:		/* don't highlight toggle icons */
	case M_MANHATTAN_ADJUST:
	case M_PUT1:
	case M_PUT2:
	case M_PUT3:
	case M_PUT4:
	    lastcommand = menu[menuindex].menu_function;
	    (*lastcommand)();
	    break;
	default:
	    MNReverse(menuindex); /* highlight icon momentarily */
	    icon_last = menuindex;
	    
	    if ((menu[menuindex].inter_act) && (SEQ == 0)) {  /* go interactive */
		MNReverse(menuindex); /* unhighlight icon */
		select_inter(icon_last);
		icon_last = -1;
		return;
	    }
	    
	    lastcommand = menu[menuindex].menu_function;
	    (*lastcommand)();
	    MNReverse(menuindex); /* unhighlight icon */
	    icon_last = -1;
	}	    
    }
    else
	error("no such command");
    
}  /* end MNSHCommand */

/*
 * This routine implements a special short command for the SHIFT key.
 * If there is no interactive mode and there are points, it acts
 * just like the 'd' short command.  Otherwise, it saves the current
 * interactive mode and enters the interactive define set until the
 * SHIFT key goes up
 */
MNShift(cmdPtr)
    char	*cmdPtr;
{
    if (inter_index == M_SET_AREA) return; /* already defining */
    if (SEQ) {			/* are there points in the picture */
	if (inter_index) return; /* ignore the shift key */
	else *cmdPtr = 'd';	/* treat it as a short command */
    } else {			/* no points plotted yet */
	shifted_index = inter_index;
	select_inter(M_SET_AREA);
	Defining = TRUE;
    }
}

/*
 * This routine restores the saved function when the SHIFT key goes up.
 * If there are points in the picture then a definition is in progress and
 * it simply returns. Otherwise it restores the saved interactive
 * mode, if any.
 */
MNUnShift()
{
    register int index = M_SET_AREA;
    
    if (inter_index != M_SET_AREA) { /* shouldn't happen */
	Defining = FALSE;
	return;
    }
    
    if (SEQ) {			/* definition in progress */
	return;
    } else {
	if (shifted_index) index = shifted_index; /* if something saved */
	select_inter(index);	/* restore saved mode */
	shifted_index = 0;
	Defining = FALSE;
    }
}

/* 
 *  find justification mode determined by (x, y) screen position 
 *  return -1 if not in justify icon
 */
just_index(x, y)
register x;
register y;
{
    x -= menu[M_JUSTIFY].menu_x - 1;
    y -= menu[M_JUSTIFY].menu_y - 2;

    if ((x > 41) || (x < 0) || (y > 19) || (y < 0))
	return(-1);
    else
	return((y / 7) * 3 + (x / 14));
}


mouse_move(eventPtr)
    XMotionEvent	*eventPtr;
{
    register index, in_icon, in_xy;

    /* handle justify icon as a special case */
    if ((index = just_index(eventPtr->x, eventPtr->y)) >= 0) {
	if (icon_last >= 0) {	/* erase last highlighted icon */
	    MNReverse(icon_last);
	    icon_last = -1;
	}

	if (just_last >= 0)	/* erase current justification marker */
	    MNReverseJustify();

	just_x = justify_mark[index].x;
	just_y = justify_mark[index].y;
	just_last = index;
	MNReverseJustify();
	return;
    }


    /* ... not currently in justify icon */

    if (just_last >= 0) {	/* previously in justify icon */
	MNReverseJustify();
	just_last = -1;
    }

    in_xy = ((index = MNFindMenuItem(eventPtr->x, eventPtr->y)) >= 0);
    in_icon = (icon_last >= 0);

    switch ((in_xy << 1) | in_icon) {
	case 0:		/* was outside still outside */
	    break;
	case 1:		/* was inside now outside */
	    MNReverse(icon_last);
	    icon_last = -1;
	    break;
	case 2:		/* was outside now inside */
	    icon_x = menu[index].menu_x;
	    icon_y = menu[index].menu_y;
	    MNReverse(icon_last = index);
	    break;
	case 3:		/* was inside still inside */
	    if ((icon_y != menu[index].menu_y) || 
		(icon_x != menu[index].menu_x)) {
		MNReverse(icon_last);
		icon_x = menu[index].menu_x;
		icon_y = menu[index].menu_y;
		MNReverse(icon_last = index);
	    }
	    break;
    }
}


/*
 *  return index into menu[] for icon at (x, y)
 */
MNFindMenuItem(x, y)
int x, y;
{
    register struct _menu *m;
    register mleft, mright, mtop, mbottom;
    int i;

    i = 0;

    do {
	m = &menu[i];
	mleft = m->menu_x;
	mtop = m->menu_y;
	mright = mleft + m->pm_ptr->width - 1;
	mbottom = mtop + m->pm_ptr->height - 1;

	if ((x <= mright) && (y <= mbottom) && (x >= mleft) && (y >= mtop))
	    return(i);

    } while (menu[++i].menu_x != -1);

    return(-1);		/* not found */

} /* end MNFindMenuItem */


/*
 *  Highlight menu item specified by menu array index.
 */
MNHighLt(index)
int index;
{
    XSetFunction(display, menu_gc, drawOP);
    XCopyPlane(display, bigopenbox_pm.pm, menu_sw, menu_gc, 0, 0, 
	      bigopenbox_pm.width, bigopenbox_pm.height,
	      menu[index].menu_x-2, menu[index].menu_y-2, (unsigned long)1);
}


/*
 *  Un-highlight menu item specified by menu array index.
 */
MNUnHighLt(index)
int index;
{
    XSetFunction(display, menu_gc, menuEraseOP);
    XCopyPlane(display, bigopenbox_pm.pm, menu_sw, menu_gc,
	      0, 0, bigopenbox_pm.width, bigopenbox_pm.height,
	      menu[index].menu_x - 2, menu[index].menu_y - 2, 
	      (unsigned long)1);
}


/*
 *  Reverse video menu item specified by menu array index.
 */
MNReverse(index)
int index;
{
    XCopyPlane(display, bigblack_pm.pm, menu_sw, menu_xor_gc,
	      0, 0, menu[index].pm_ptr->width + 4, 
	      menu[index].pm_ptr->height + 4, 
	      menu[index].menu_x - 2, menu[index].menu_y - 2, (unsigned long)1);
}

MNReverseJustify()
{
    XCopyPlane(display, dot_pm.pm, menu_sw, menu_xor_gc, 0, 0, 5, 5,
	      just_x + 1, just_y, (unsigned long)1);
}
