/* 
 * menu.h --
 *
 *	Menu header file.
 *
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <strings.h>
#include "sx.h"

/*
 * The structure below is used internally to this file to describe
 * a menu entry.
 */

typedef struct {
    char *leftText;		/* Text to display left-justified in
				 * menu entry.  NULL means no text. */
    int leftSize;		/* # of characters in leftText. */
    int leftPos;		/* X position at which to draw text in menu. */
    char *centerText;		/* Text to display in center of menu
				 * entry. NULL means no centered text. */
    int centerSize;		/* Number of characters in centerText. */
    int centerPos;		/* X position at which to draw text in menu. */
    char *rightText;		/* Text to display right-justified in
				 * menu entry. NULL means no text. */
    int rightSize;		/* Number of characters in rightText. */
    int rightPos;		/* X position at which to draw text in menu. */
    unsigned long background;	/* Background color for menu entry. */
    void (*proc)();		/* Procedure to call when menu entry
				 * is selected. */
    ClientData clientData;	/* Client-supplied information to pass
				 * to proc. */
} MenuEntry;

/*
 * The structure below defines a pull-down menu.  It consists of a
 * number of menu entries plus a mask indicating which entries are
 * enabled:  disabled entries are displayed but do not respond to
 * button presses.  Two windows are used to display the menu:  one
 * for the menu title, and one for the menu itself (which is a child
 * of the root window and is only displayed when the menu is active).
 */

typedef struct Menu {
    Display *display;			/* Connection to X server. */
    Window menuWindow;			/* Window used to display menu
					 * entries. */
    XFontStruct *fontPtr;		/* Font to be used for this menu. */
    unsigned long foreground;		/* Foreground color for title and
					 * menu. */
    unsigned long background;		/* Background color for title. */
    GC gc;				/* Graphics context used for drawing
					 * in menu or menu title.  Only the
					 * font part never changes. */
    int numEntries;			/* Number of entries in menu. */
    MenuEntry *entryPtrs[SX_MAX_MENU_ENTRIES];
					/* Array of entries for this menu.
					 * Entry 0 is displayed at the top.
					 * NULL means entry doesn't exist.
					 * Only first numEntries are valid. */
    int menuWidth;			/* Width of menuWindow in pixels,
					 * including border and shadow. */
    int menuHeight;			/* Height of menuWindow in pixels,
					 * including border and shadow. */
} Menu;

extern	Menu		*Menu_Create();
extern  ClientData	Menu_PullMenuDown();
