/*
 * @(#)short.c	1.2	1/3/85
 *
 * Routines for the "short" commands of the SUN Gremlin picture editor.
 *
 * Mark Opperman (opcode@monet.BERKELEY)
 *
 * Modifications by Jim Thompson (james@ernie.berkeley.edu)
 */

#include "icondata.h"
#include "gremlin.h"

/* imports from graphics files */

extern GRClear();
extern GRCurrentSet();
extern GRCurrentSetOn();
extern GRCurrentSetOff();
extern GRDisplayPoint();
extern GRSetCurve();
extern GRSetTextPos();

/* imports from undodb.c */

extern UNForget();

/* imports from display.c */

extern DISClearSetDisplay();
extern DISScreenAdd();
extern DISScreenErase();
extern fasterase();

/* imports from database files */

extern ELT *DBCopy();
extern ELT *DBCreateElt(); 

extern POINT *PTMakePoint();
extern POINT *PTMakeTextPoints();

extern DBDelete();
extern DBClearElt();
extern DBXform();
extern DBBounded();
extern DBAddSet();
extern DBClearSet();

/* imports from long.c */

extern LGIncludeSet();
extern CP();			/* clear points */
extern CSP();			/* clear show points */

/* imports from menu.c  */

extern MNHighLt();
extern MNUnHighLt();

extern HiBrush[];
extern HiBuffer[];
extern HiFont[];
extern HiMode[];
extern HiSize[];

int adj[4] = { 0, 0, 1, 2 };

/* imports from help.c */

extern help();

/* imports from text.c */

extern TxMsgOK();
extern TxPutMsg();
extern text_putvalue();
extern text_restorebuf();

/* imports from sun.c */

extern prompt_ok();

/* imports from C */

extern char *malloc();

/* imports from main.c */

extern ELT *PICTURE;                /* current PICTURE database      */
extern ELT *cset;                   /* current set database          */
extern ELT *MEN[];                  /* pointers for user symbols     */
extern ELT arhead;                  /* arrow head template           */

extern POINT MENPOINT[];            /* pointers used fo user symbols */
extern POINT *POINTLIST;            /* accumulated point list        */

extern CBRUSH;        		    /* current brush                 */
extern Gridon;                      /* grid mode flag                */
extern SEQ;                         /* point sequence number         */
extern Adjustment;                  /* point adjustment mode         */
extern Arrowtype;		    /* current type of arrow	     */
extern Curvetype;		    /* current style of curve        */
extern GravityOn;                   /* gravity mode flag             */
extern CHANGED;                     /* PICTURE changed flag          */
extern CsetOn;			    /* current set displayed on	     */

extern (*lastcommand)();	    /* previous command */
extern lasttext;		    /* TRUE if previous command wants text */

extern SHUpdate(), SHAgain(), SHDrawArc(), SHDrawCurve(), SHCopy(),
       SHDefineSet(), SHErase(), SHSetArea(), SHGravity(), SHHide(),
       SHGrid(), SHIcon(), SHOnTop(), SHRotate(), SHScale(),
       SHScaleX(), SHScaleY(), SHScaleXY(), SHTranslate(),
       SHDrawVector(), SHMAdjust(), SHBox(), SHArrow(),
       SHSave1(), SHSave2(), SHSave3(), SHSave4();

extern LGUndo();

#define Pi 3.14159
#define twoPi 6.2832
#define SCALEX 1
#define SCALEY 2
#define SCALEXY 3
#define ARROWHEAD	1	/* arrowhead without shaft */
#define ARROW		2	/* arrow with attached line */
#define DBLARROW	3	/* line with arrowheads at both ends */

static char nopnt[18] = "not enough points";
static char noset[15] = "no current set";
static char select_all_msg[] = "SELECT TOTAL PICTURE?             \
Press left button to confirm, middle or right button to cancel.";



/*
 * Repeat previous command.
 */
SHAgain()
{
    if (lasttext)
	text_restorebuf();
    (*lastcommand)();
}


/*
 * This routine creates and displays a VECTOR element from the
 * points previously specified.
 */
SHDrawVector()
{
    register POINT *p1, *p2;
    register ELT *e1;
    POINT *plist;
    char *txt;

    if (SEQ < 2) {      /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    DISClearSetDisplay();
    DBClearSet();

    plist = PTInit();
    p1 = POINTLIST;
    (void) PTMakePoint(p1->x, p1->y, &plist);
    p2 = PTNextPoint(p1);

    while (!Nullpoint(p2)) {
	(void) PTMakePoint(p2->x, p2->y, &plist);
	p1 = p2;
	p2 = PTNextPoint(p1);
    }

    txt = malloc(1);
    *txt = '\0';
    e1 = DBCreateElt(VECTOR, plist, CBRUSH, 0, txt, &PICTURE);
    DISScreenAdd(e1, pixmask | csetmask);
    DBAddSet(e1);

    CP();
    CHANGED = TRUE;
}  /* end SHDrawVector */


/*
 * This routine creates and displays an ARC element based on the 
 * points previously defined.  If 3 or more points are defined, the
 * extent of the arc is calculated as the angle formed by the 
 * respective lines through the second and third points and the first
 * point.  If only 2 points are specified, a full circle is drawn.
 */
SHDrawArc()
{
    register POINT *p1, *p2, *p3;
    register ELT *e1;
    POINT *plist;
    char *txt;
    float angle;

    if (SEQ < 2) {       /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    CSP();

    plist = PTInit();
    p1 = POINTLIST;
    p2 = PTNextPoint(p1);
    p3 = PTNextPoint(p2);

    if (MakeArcPoints(p1,p2,p3,&plist,&angle)) /* if error in parms */
	return;

    DISClearSetDisplay();
    DBClearSet();

    txt = malloc(1);
    *txt = '\0';
    e1 = DBCreateElt(ARC, plist, CBRUSH, (int) (angle + 0.5), txt, &PICTURE);

    DISScreenAdd(e1, pixmask | csetmask);
    DBAddSet(e1);

    CP();
    CHANGED = TRUE;
}  /* end SHDrawArc */

/*
 * Make a list of points for a circle or arc, and the angle.
 * (Used when an arc is created, or when a point is moved by LGMPoint)
 * Returns 0 if the parameters are valid, otherwise returns 1.
 */

int
MakeArcPoints(p1,p2,p3,plist,angle)
register POINT *p1, *p2, *p3;
POINT **plist;
float *angle;
{
    float a1, a2, radius;

    radius = sqrt((double) ((p2->x - p1->x) * (p2->x - p1->x) +
			    (p2->y - p1->y) * (p2->y - p1->y)));
    if (radius == 0.0) {
	error("zero radius");
	return 1;
    }

    if (Nullpoint(p3)) {       /* draw full circle */
	*angle = 0;
        /* Add extra positioning points  */
	(void) PTMakePoint(p1->x, p1->y, plist);
	(void) PTMakePoint(p2->x, p2->y, plist);
	(void) PTMakePoint(p1->x, p1->y + radius, plist);
	(void) PTMakePoint(p1->x, p1->y - radius, plist);
	(void) PTMakePoint(p1->x + radius, p1->y, plist);
	(void) PTMakePoint(p1->x - radius, p1->y, plist);
    }
    else {
	(void) PTMakePoint(p1->x, p1->y, plist);
	(void) PTMakePoint(p2->x, p2->y, plist);
	a1 =  atan2((p2->x - p1->x), (p2->y - p1->y));
	a2 =  atan2((p3->x - p1->x), (p3->y - p1->y));
	*angle = a1 - a2;
	if (*angle < 0.0) 
	    *angle += twoPi;

	/* Set second point to lie on arc */
	(void) PTMakePoint((radius * sin(a2) + p1->x),
			   (radius * cos(a2) + p1->y), plist);
	*angle *= 360.0/twoPi;      /* convert to degrees */
    }
    return 0;
} /* end MakeArcPoints */


/*
 * Draw curve object.
 */
SHDrawCurve()
{
    register POINT *p1;
    register ELT *e1;
    POINT *plist;
    char *txt;

    if (SEQ < 2) {
	error("need at least 2 points");
	return;
    }

    plist = PTInit();
    p1 = POINTLIST;

    do {
	(void) PTMakePoint(p1->x, p1->y, &plist);
	p1 = PTNextPoint(p1);
    } while (!Nullpoint(p1));
    
    switch (Curvetype) {
    case CURVE:
	if (GRSetCurve(plist) != 0) {
	    error("too many consecutive knots at same place");
	    return;
	}
	break;
    case B_SPLINE:
	if (GRSetBSpline(plist) != 0) {
	    error("too many consecutive knots at same place");
	    return; 
	}
	break;
    case BEZIER:
	if (GRSetBezier(plist) != 0) {
	    error("too many consecutive knots at same place");
	    return; 
	}
	break;
    default:
	error("unknown curve type");
	return;
    }
    UNForget();

    txt = malloc(1);
    *txt = '\0';
    e1 = DBCreateElt(Curvetype, plist, CBRUSH, 0, txt, &PICTURE);

    DISClearSetDisplay();
    DBClearSet();
    DISScreenAdd(e1, pixmask | csetmask);
    DBAddSet(e1);

    CP();
    CHANGED = TRUE;
}  /* end SHDrawCurve */


/*
 * This routine erases selected elements from the screen and deletes
 * them from the picture database.  
 */
SHErase()
{
    register ELT *e1;

    if (DBNullelt(cset)) {
	error(noset);
	return;
    }

    UNForget();

    fasterase();

    while (!DBNullelt(cset)) {   /* delete elements in current set */
/*
	DISScreenErase(cset, pixmask | csetmask);
*/
	e1 = DBNextofSet(cset);
	DBDelete(cset, &PICTURE);
	cset = e1;
    }

    CHANGED = TRUE;
}  /* end SHErase */


/*
 * This routine toggles the gravity mode.
 */
SHGravity()
{
    if (GravityOn = !GravityOn)
	MNHighLt(HiMode[3]);
    else
	MNUnHighLt(HiMode[3]);
}  /* End GravityOn */


/*
 * This routine toggles the display of the grid.
 */
SHGrid()
{
    if (Gridon = !Gridon)
	GRDisplayGrid();
    else
	GRBlankGrid();
}  /* end SHGrid */


/*
 * Manhattan Adjust -
 * This routine toggles the adjustment mode.
 */
SHMAdjust()
{
    if (Adjustment == MAN) {
	MNUnHighLt(HiMode[adj[MAN]]);
	Adjustment = NOADJ;
    }
    else {
	if (Adjustment != NOADJ)
	    MNUnHighLt(HiMode[adj[Adjustment]]);
	MNHighLt(HiMode[adj[MAN]]);
	Adjustment = MAN;
    }
}  /* end SHMAdjust */


/*
 * This routine defines the current set based upon previously
 * defined points to select elements.  If no points are specified
 * the entire picture becomes the current set.  In this case, the
 * old current set is not erased (optimization) and any elements
 * not in it are added.  Otherwise, the new current set can not be
 * guaranteed to contain the old current set, and so it is erased
 * and set to empty before adding the new elements.
 */
SHDefineSet()
{
    if (SEQ == 0) {		/* select everything */
	if (!prompt_ok(select_all_msg)) /* ask for confirmation */
		return;
    }
    if (SEQ > 0) {		/* redefine current set */
	DISClearSetDisplay();
	DBClearSet();
    }

    CSP();
    LGIncludeSet();
}  /* end SHDefineSet */


/*
 * This routine defines the current set by selecting all elements
 * bounded by a rectangle whose diagonal is defined by specifed points.
 */
SHSetArea()
{
    if (SEQ < 2) {
	error(nopnt);
	return;
    }

    if (DBNullelt(PICTURE))
	return;

    DISClearSetDisplay();
    DBClearSet();

    SHMSetArea();
}  /* end SHSetArea */


/*
 * This routine ADDS to the current set all elements bounded by a 
 * rectangle whose diagonal is defined by two specifed points.
 */
SHMSetArea()
{
    register ELT *e1;
    float x1, y1, x2, y2;

    if (SEQ < 2) {
	error(nopnt);
	return;
    }

    if (DBNullelt(PICTURE))
	return;

    x1 = POINTLIST->x;
    y1 = POINTLIST->y;
    x2 = PTNextPoint(POINTLIST)->x;
    y2 = PTNextPoint(POINTLIST)->y;
    e1 = PICTURE;

    while (!DBNullelt(e1)) {
	if (DBBounded(e1, x1, y1, x2, y2) && !DBInCset(e1)) {
	    DISScreenAdd(e1, csetmask);
	    DBAddSet(e1);
	}
	e1 = DBNextElt(e1);
    }

    CP();
}  /* end SHMSetArea */


/*
 * This routine translates the elements in the current set as defined
 * by points.  The translation is accomplished by defining a transformation
 * matrix and calling DBXform.
 */
SHTranslate()
{
    register ELT *e1;
    register POINT *p1;
    float xmat[3][2];

    if (DBNullelt(cset)) {
	error(noset);
	return;
    }

    if (SEQ < 2) {      /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    p1 = PTNextPoint(POINTLIST);
    xmat[0][0] = xmat[1][1] = 1;    /* set up translation matrix */
    xmat[1][0] = xmat[0][1] = 0;
    xmat[2][0] = p1->x - POINTLIST->x;
    xmat[2][1] = p1->y - POINTLIST->y;
    e1 = cset;

    fasterase();

    while (!DBNullelt(e1)) {
	/*
	DISScreenErase(e1, pixmask | csetmask);
	*/
	TxMsgOK();
	DBXform(e1, xmat, &PICTURE);
	DISScreenAdd(e1, pixmask | csetmask);
	e1 = DBNextofSet(e1);
    }

    CP();
    CHANGED = TRUE;
}  /* end SHTranslate */


/*
 * This routine copies the elements in the current set as defined
 * by points.  To copy, the current set pointer is cleared so that new
 * elements as added by DBCopy can be used to comprise the new current
 * set.  A pointer is maintained to the old current set which is traversed
 * to determine the elements to be copied.  This process continues for all
 * points specified.
 *
 * NOTE:  This assumes that the DBClearSet routine does not alter the
 *        pointers between elements in the set (which is currently true),
 *        and must be changed it this does not hold.
 */
SHCopy()
{
    register ELT *e1, *e2;
    register POINT *p1, *p2;
    float xmat[3][2];

    if (DBNullelt(cset)) {
	error(noset);
	return;
    }

    if (SEQ < 2) {      /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    p1 = POINTLIST;
    p2 = PTNextPoint(POINTLIST);

    while (!Nullpoint(p2)) {
	xmat[0][0] = xmat[1][1] = 1;    /* set up translation matrix */
	xmat[1][0] = xmat[0][1] = 0;
	xmat[2][0] = p2->x - p1->x;
	xmat[2][1] = p2->y - p1->y;
	e1 = cset;

	DISClearSetDisplay();
	DBClearSet();    /* Dependent on Clearset preserving pointers */

	while (!DBNullelt(e1)) {
	    e2 = DBCopy(e1, xmat, &PICTURE);
	    DISScreenAdd(e2, pixmask | csetmask);
	    DBAddSet(e2);
	    e1 = DBNextofSet(e1);
	}

	p1 = p2;
	p2 = PTNextPoint(p2);
    }

    CP();
    CHANGED = TRUE;
}  /* end SHCopy */


/*
 * This routine rotates the elements in the current set as defined
 * by points.  The rotation is accomplished by defining a transformation
 * matrix and calling DBXform.
 */
SHRotate()
{
    register ELT *elt;
    register POINT *p1, *p2;
    POINT pos;
    float xmat[3][2], angle, s, c;

    if (DBNullelt(cset)) {
	error(noset);
	return;
    }

    if (SEQ < 3) {      /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    p1 = PTNextPoint(POINTLIST);  /* calculate rotation angle */
    p2 = PTNextPoint(p1);
    angle = (float) atan2((p2->x - POINTLIST->x), (p2->y - POINTLIST->y)) -
	    (float) atan2((p1->x - POINTLIST->x), (p1->y - POINTLIST->y));
    s = (float) sin(angle);      
    c = (float) cos(angle);      

    /* Define transformation matrix to translate set to origin, rotate,
      and translate back. */

    xmat[0][0] = c;
    xmat[0][1] = -s;
    xmat[1][0] = s;
    xmat[1][1] = c;
    xmat[2][0] = (-c) * POINTLIST->x - s * POINTLIST->y + POINTLIST->x;
    xmat[2][1] = (-c) * POINTLIST->y + s * POINTLIST->x + POINTLIST->y;

    elt = cset;
    /* DISClearSetDisplay(); */

    fasterase();

    while (!DBNullelt(elt)) {
	/* DISScreenErase(elt, pixmask); */
	TxMsgOK();
	DBXform(elt, xmat, &PICTURE);

	if (TEXT(elt->type)) {
	    GRSetTextPos(elt->textpt, elt->type, elt->brushf, elt->size, 
						elt->ptlist, &pos);
	    elt->ptlist = PTMakeTextPoints(elt->textpt, elt->brushf, elt->size, 
						elt->ptlist, &pos);
	}

	DISScreenAdd(elt, pixmask | csetmask);
	elt = DBNextofSet(elt);
    }

    CP();
    CHANGED = TRUE;
}  /* end SHRotate */


SHScaleX()
{
    SHScale(SCALEX);
}


SHScaleY()
{
    SHScale(SCALEY);
}


SHScaleXY()
{
    SHScale(SCALEXY);
}

/*
 * This routine scales the elements in the current set as defined
 * by points.  The scaling is accomplished by defining a transformation
 * matrix and calling DBXform.
 */
SHScale(direction)
int direction;
{
    register ELT *elt;
    register POINT *p1, *p2;
    POINT pos;
    float xmat[3][2], d1, d2, scalex, scaley;

    if (DBNullelt(cset)) {
	error(noset);
	return;
    }

    if (SEQ < 3) {      /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    p1 = PTNextPoint(POINTLIST);
    p2 = PTNextPoint(p1);
    d1 = sqrt(pow((p1->x - POINTLIST->x), 2.0) + 
	      pow((p1->y - POINTLIST->y), 2.0));
    d2 = sqrt( pow((p2->x - POINTLIST->x), 2.0) + 
	      pow((p2->y - POINTLIST->y), 2.0));

    if (d1 == 0) {
	error("infinite scale");
	return;
    }
 
    scalex = scaley = 1;
    if (direction==SCALEX | direction==SCALEXY) scalex = d2 / d1;
    if (direction==SCALEY | direction==SCALEXY) scaley = d2 / d1;

    /* create transformation matrix to translate set to origin, 
       performaing the scaling and translating back */

    xmat[0][0] = scalex;
    xmat[1][1] = scaley;
    xmat[1][0] = xmat[0][1] = 0;
    xmat[2][0] = - POINTLIST->x * (scalex - 1.0);
    xmat[2][1] = - POINTLIST->y * (scaley - 1.0);

    elt = cset;
    fasterase();
    /* DISClearSetDisplay(); */

    while (!DBNullelt(elt)) {
	/* DISScreenErase(elt, pixmask); */
	TxMsgOK();
	DBXform(elt, xmat, &PICTURE);

	if (TEXT(elt->type)) {
	    GRSetTextPos(elt->textpt, elt->type, elt->brushf, 
					    elt->size, elt->ptlist, &pos);
	    elt->ptlist = PTMakeTextPoints(elt->textpt, elt->brushf, 
					    elt->size, elt->ptlist, &pos);
	}

	DISScreenAdd(elt, pixmask | csetmask);
	elt = DBNextofSet(elt);
    }

    CP();
    CHANGED = TRUE;
}  /* end SHScale */


/*
 * This routine redraws the graphics screen by clearing the screen ,
 * redisplaying the menu and adding each element back to the display.
 */
SHUpdate()
{
    register ELT *e1;
    register POINT *plist;
    register i;

    GRClear(pixmask | csetmask);

    if (Gridon)
	GRDisplayGrid();

    e1 = PICTURE;
    while (!DBNullelt(e1)) {
	if (DBInCset(e1))
	    DISScreenAdd(e1, pixmask | csetmask);
	else
	    DISScreenAdd(e1, pixmask);
	e1 = DBNextElt(e1);
    }

    CsetOn = 1;

    i = 0;
    plist = POINTLIST;
    while (!Nullpoint(plist)) {
	GRDisplayPoint(plist->x, plist->y, i++);
	plist = PTNextPoint(plist);
    }
}  /* end SHUpdate */


/*
 * This local routine stores the current set in the specified
 * user symbol.
 */
static 
savemen(sym)
register sym;
{
    register ELT *elt;
    float xmat[3][2];

    xmat[0][0] = xmat[1][1] = 1;	/* set up copy transformation */
    xmat[0][1] = xmat[1][0] = 0;	/* matrix for no transformation */
    xmat[2][0] = xmat[2][1] = 0;

    while (!DBNullelt(MEN[sym])) {	/* clear out existing symbols */
	elt = DBNextElt(MEN[sym]);
	DBClearElt(MEN[sym]);     
	MEN[sym] = elt;
    }

    elt = cset;				/* copy current set to symbol */

    while (!DBNullelt(elt)) {
	(void) DBCopy(elt, xmat, &(MEN[sym]));
	elt = DBNextofSet(elt);
    }

    if (SEQ == 0) {			/* no positioning points */
	MENPOINT[sym].x = 0;
	MENPOINT[sym].y = 0;
    }
    else {
	MENPOINT[sym].x = POINTLIST->x;
	MENPOINT[sym].y = POINTLIST->y;
    }

    if (!DBNullelt(MEN[sym]))
	MNHighLt(HiBuffer[sym]);
    else
	MNUnHighLt(HiBuffer[sym]);

    CP();
    CHANGED = TRUE;
}  /* end savemen */


/*
 * This routine saves the current set in user symbol 1 by
 * calling savemen.
 */
SHSave1()
{
    savemen(0);
}


/*
 * This routine saves the current set in user symbol 2 by
 * calling savemen.
 */
SHSave2()
{
    savemen(1);
}


/*
 * This routine saves the current set in user symbol 3 by
 * calling savemen.
 */
SHSave3()
{
    savemen(2);
}


/*
 * This routine saves the current set in user symbol 4 by
 * calling savemen.
 */
SHSave4()
{
    savemen(3);
}


/*
 * This routine creates and displays a rectangle whose diagonal is
 * defined by two points.  The routine uses the coordinates of these
 * points to define a VECTOR element with the appropriate vertices.
 */
SHBox()
{
    register POINT *p1, *p2;
    register ELT *e1;
    POINT *plist;
    char *txt;
    
    if (SEQ < 2) {
	error("not enough points");
	return;
    }

    UNForget();

    p1 = POINTLIST;
    p2 = PTNextPoint(p1);
    plist = PTInit();   /* create points for vector elements which define 
			   the rectangle */
    (void) PTMakePoint(p1->x, p1->y, &plist);
    (void) PTMakePoint(p1->x, p2->y, &plist);
    (void) PTMakePoint(p2->x, p2->y, &plist);
    (void) PTMakePoint(p2->x, p1->y, &plist);
    (void) PTMakePoint(p1->x, p1->y, &plist);   /* close rectangle */
    txt = malloc(1);
    *txt = '\0';

    DISClearSetDisplay();
    DBClearSet();
    e1 = DBCreateElt(VECTOR, plist, CBRUSH, 0, txt, &PICTURE);
    DISScreenAdd(e1, pixmask | csetmask);
    DBAddSet(e1);

    CP();
    CHANGED = TRUE;
}  /* end SHBox */


/*
 * This routine draws arrow heads by 'copying' the arrow head template
 * into the picture appropriately transformed.
 *
 * The head of the arrow goes at the SECOND point drawn. (JGT)
 */
SHArrow()
{
    register ELT *e1;
    register ELT *e2;
    POINT p1,p2;
    register POINT *pt;
    float xmat[3][2], angle, s, c;

    if (SEQ < 2) {      /* not enough points */
	error(nopnt);
	return;
    }

    UNForget();

    p1.x = POINTLIST->x - 1;
    p1.y = POINTLIST->y;
    p2.x = (PTNextPoint(POINTLIST))->x;
    p2.y = (PTNextPoint(POINTLIST))->y;
    angle = Pi +
	    (float) atan2((POINTLIST->x - p2.x),(POINTLIST->y - p2.y)) -
	    (float) atan2((POINTLIST->x - p1.x),(POINTLIST->y - p1.y));
    s = (float) sin(angle);
    c = (float) cos(angle);

   /* Define transformation matrix to translate element from origin
      and rotate. */

    xmat[0][0] = c;
    xmat[0][1] = -s;
    xmat[1][0] = s;
    xmat[1][1] = c;
    xmat[2][0] = p2.x;
    xmat[2][1] = p2.y;

    DISClearSetDisplay();	/* the new current set */
    DBClearSet();		/* clear old set in preparation to make */
    arhead.brushf = CBRUSH;
    e1 = DBCopy(&arhead, xmat, &PICTURE);

    if (Arrowtype) {	/* need to draw the shaft */
	(void) PTMakePoint(POINTLIST->x, POINTLIST->y, &(e1->ptlist));
    }
    if (Arrowtype == 2) {	/* double arrow */
	s = (float) sin(Pi+angle);	  
	c = (float) cos(Pi+angle);      

   /* Define transformation matrix to translate element from origin
      and rotate. */

	xmat[0][0] = c;
	xmat[0][1] = -s;
	xmat[1][0] = s;
	xmat[1][1] = c;
	xmat[2][0] = POINTLIST->x;
	xmat[2][1] = POINTLIST->y;

	DISClearSetDisplay();	/* the new current set */
	DBClearSet();		/* clear old set in preparation to make */
	arhead.brushf = CBRUSH;
	e2 = DBCopy(&arhead, xmat, &PICTURE);
	pt = e2->ptlist;
	while (!Nullpoint(pt)) {
		(void) PTMakePoint(pt->x, pt->y, &(e1->ptlist));
		pt = pt->nextpt;
	}
	DBDelete(e2,&PICTURE);
    }

    DISScreenAdd(e1, pixmask | csetmask);
    DBAddSet(e1);

    CP();
    CHANGED = TRUE;
}  /* end SHArrow */

