/*
 * @(#)text.c	1.2	1/3/85
 *
 * Text subwindow routines for the SUN Gremlin picture editor.
 *
 * Mark Opperman 
 *
 */

#include <sgtty.h>
#include "gremlin.h"
#include "icondata.h"

/* imports from main.c */

extern Window	text_sw;
extern struct rect text_size;
extern (*lastcommand)();
extern lasttext;
extern FLASH_READY; /*::*/
extern ELT *cset;

/* imports from long.c */

extern LGTextSW();

/* imports from graphics.c */

extern GRCurrentSetOn();

/* imports from help.c */

extern textsw_help();
extern get_any_button();

/* imports from strings.c */
extern char text_font[];

/* imports from C */

extern char *strcpy();

/* locals */

struct XFontStruct *text_pf;

#define TEXT_LEFT 96		/* start of text input */
#define TEXT_FIRST  ' '
#define TEXT_LAST  '~'
#define CTRL_H  '\010'		/* backspace */
#define CTRL_U  '\025'		/* kill line */
#define CTRL_W  '\027'		/* delete word */
#define CTRL_B	'\002'		/* non-destructive backspace */
#define CTRL_D	'\004'		/* delete character under cursor */
#define CTRL_F	'\006'		/* non-destructive forward space */
#define ESC	'\033'
#define RETURN  '\r'
#define MSGLINE 17

unsigned	text_width;
unsigned	text_height;

static char text_buf[TEXT_BUFMAX] = "";
static char mesg_buf[TEXT_BUFMAX] = "";
static char save_buf[TEXT_BUFMAX] = "";
static char text_erase;
static char text_kill;
static char text_erase_word;
static text_length;
static text_next;		/* location of text cursor */
static mesg_length;	/* length of last message displayed */
static putmsg = FALSE;	/* TRUE if TxPutMsg called since last button input. */
static escseen = 0;		/* last character was esc */


void
text_cursor(op)
int op;
{
    if (op != GXxor) {
	XSetFunction(display, text_gc, op);
	XCopyPlane(display, bigblack_pm.pm, text_sw, text_gc,
	      0, 0, text_width, text_height, 
	      (int)(TEXT_LEFT + text_next * text_width), 0, (unsigned long)1);
    } else {
	XCopyPlane(display, bigblack_pm.pm, text_sw, text_xor_gc,
	      0, 0, text_width, text_height, 
	      (int)(TEXT_LEFT + text_next * text_width), 0, (unsigned long)1);
    }
}


void
text_output(c)
register c;
{
    register i, count;

    TxMsgOK();
    
    if (escseen) {		/* there was an ESC */
	if (c == '[') return;	/* still could be ESC-[-D/C */
	if (c == 'D') c = CTRL_B; /* pretend it was CTRL-B */
	else if (c == 'C') c = CTRL_F; /* pretend it was CTRL-F */
	escseen = 0;		/* in all cases, no longer under ESC */
    }

    if ((c>=TEXT_FIRST) && (c<=TEXT_LAST) && (text_length<TEXT_BUFMAX-1)) {
	if (text_next != text_length) {	/* need to shift remaining chars */
	    for (i=text_length; i > text_next; i--)
	      text_buf[i] = text_buf[i-1];
	}
	text_buf[text_next] = c;
	text_buf[text_length+1] = '\0';
	text_clear(text_next,text_length-text_next+1);
	XSetFunction(display, text_gc, GXcopy);
	XDrawString(display, text_sw, text_gc, 
		    (int)(TEXT_LEFT + text_next * text_width), TEXT_BASELINE,
		    &text_buf[text_next], strlen(&text_buf[text_next]));
	text_length++;
	text_next++;
	text_cursor(GXxor);
    }

    else if (((c == text_erase) || (c == CTRL_H)) && (text_next > 0)) {
	text_cursor(GXxor);
	text_length--;
	text_next--;
	text_clear(text_next,text_length-text_next+1);
	if (text_next != text_length) {	/* need to shift remaining chars */
	    for (i=text_next; i <= text_length; i++)
	      text_buf[i] = text_buf[i+1];
	    XSetFunction(display, text_gc, GXcopy);
	    XDrawString(display, text_sw, text_gc, 
			(int)(TEXT_LEFT + text_next * text_width),
			TEXT_BASELINE,
			&text_buf[text_next], strlen(&text_buf[text_next]));
	}
	text_buf[text_length] = '\0';
	text_cursor(GXxor);
    }

    else if ((c == text_kill) || (c == CTRL_U)) {
	TxKillLine();
    }
    
    else if (c == ESC) {
	escseen = 1;
    }
	
    else if ((c == CTRL_B) && (text_next > 0)) {
	text_cursor(GXxor);
	text_next--;
	text_cursor(GXxor);
    }

    else if ((c == CTRL_F) && (text_next < text_length)) {
	text_cursor(GXxor);
	text_next++;
	text_cursor(GXxor);
    }

    else if ((c == text_erase_word) && (text_next > 0)) {
	text_cursor(GXcopyInverted);
	count = 0;
	while ((text_next > 0) && (text_buf[text_next-1] == ' ')) {
	    text_length--;
	    text_next--;
	    count++;
	}
	while ((text_next > 0) && (text_buf[text_next-1] != ' ')) {
	    text_length--;
	    text_next--;
	    count++;
	}
	text_clear(text_next, text_length-text_next+count);
	if (text_next != text_length) {
	    for (i=text_next; i <= text_length; i++)
	      text_buf[i] = text_buf[i+count];
	    XSetFunction(display, text_gc, GXcopy);
	    XDrawString(display, text_sw, text_gc, 
			(int)(TEXT_LEFT + text_next * text_width),
			TEXT_BASELINE,
			&text_buf[text_next], strlen(&text_buf[text_next]));
	}
	text_buf[text_length] = '\0';

	text_cursor(GXxor);
    }

    else if (c == RETURN) {	/* new text entry method */
	lastcommand = LGTextSW;
	lasttext = TRUE;
	LGTextSW();
    }
}

text_clear(start,count)
    int start, count;
{
    XGCValues gcValues;

    gcValues.foreground = background;
    gcValues.function = GXcopy;
    XChangeGC(display, text_gc, GCForeground | GCFunction, &gcValues);
    XFillRectangle(display, text_sw, text_gc, 
		   (int)(TEXT_LEFT + start * text_width), 0,
		   count * text_width, text_height);
    XSetForeground(display, text_gc, foreground);
}

text_getvalue(s)
register char *s;
{
    register i = 0;

    while (text_buf[i] == ' ')
	i++;

    (void) strcpy(save_buf, &text_buf[i]);
    (void) strcpy(s, &text_buf[i]);
}


text_putvalue()
{
    XSetFunction(display, text_gc, GXcopy);
    XDrawString(display, text_sw, text_gc, 2, TEXT_BASELINE,
		"text input:", strlen("text input:"));
    XDrawString(display, text_sw, text_gc, TEXT_LEFT, TEXT_BASELINE,
		text_buf, strlen(text_buf));
    text_cursor(GXcopy);
    XDrawString(display, text_sw, text_gc, 0, MSGLINE + TEXT_BASELINE,
		mesg_buf, strlen(mesg_buf));
}


text_restorebuf()
{
    register char *s;

    TxKillLine();
    s = save_buf;
    while (*s)
	text_output(*s++);
}

text_restorecset()
{
    register ELT *elt;
    register char *s;

    elt = cset;
    while (!DBNullelt(elt)) {
	if (TEXT(elt->type)) {
	    s = elt->textpt;
	    while (*s) text_output(*s++);
	    break;
	}
	elt = DBNextofSet(elt);
    }
}

text_left(eventPtr)
    XButtonEvent	*eventPtr;
{
    register count;
    
    if (putmsg)
	TxMsgOK();
    else if (!text_length) 	/* if message clear restore text from cset */
	text_restorecset();
    else {			/* see if pointing to something */
	count = (eventPtr->x - TEXT_LEFT) / text_width;
	if (count < 0) count = 0;
	if (count > text_length) count = text_length;
	text_cursor(GXxor);
	text_next = count;
	text_cursor(GXxor);
    }
}

/*ARGSUSED*/
text_middle(eventPtr)
    XButtonEvent	*eventPtr;
{
    if (!text_length) {		/* ignore if text already present */
	TxMsgOK();
	text_restorebuf();
    }
}

/*ARGSUSED*/
text_right(eventPtr)
    XButtonEvent	*eventPtr;
{
    TxMsgOK();
    textsw_help();
}

/*ARGSUSED*/
text_winexit(eventPtr)
    XCrossingEvent	*eventPtr;
{
    GRCurrentSetOn();
}


TxInit()
{
    struct sgttyb sgtty_buf;
    
    ioctl(fileno(stdin), TIOCGETP, (char *)&sgtty_buf);
    text_erase = sgtty_buf.sg_erase;
    text_kill = sgtty_buf.sg_kill;
    text_erase_word = CTRL_W;

    text_length = text_next = mesg_length = 0;
    text_buf[0] = mesg_buf[0] = '\0';
}  /* end TxInit */


/*
 * This routine marks it OK to output messages again
 * just as if a button had been pressed.
 */
TxMsgOK()
{
    if (putmsg) {        /* message on the screen to be blanked */
	TxClearMsg();
    }

    putmsg = FALSE;
}  /* end TxMsgOK */


/*
 * Output a one line message into the text subwindow.
 * If the message follows another without intervening user input,
 * the prompt "More" is output and we wait for a button input event
 * before displaying the new message.
 */
TxPutMsg(msg)
register char *msg;
{
    XSetFunction(display, text_gc, GXcopy);
    if (putmsg) {
	XDrawString(display, text_sw, text_gc, 
		    (int)(mesg_length * text_width), MSGLINE+TEXT_BASELINE, 
		    " --More--", strlen(" --More--"));
	get_any_button();
	TxClearMsg();
    }

    XDrawString(display, text_sw, text_gc, 0, MSGLINE+TEXT_BASELINE,
		msg, strlen(msg));

    putmsg = TRUE;
    mesg_length = strlen(msg);
    (void) strcpy(mesg_buf, msg);
}  /* end TxPutMsg */


/*
 * Clear current message.
 */
TxClearMsg()
{
    XGCValues gcValues;

    gcValues.foreground = background;
    gcValues.function = GXcopy;
    XChangeGC(display, text_gc, GCForeground | GCFunction, &gcValues);
    XFillRectangle(display, text_sw, text_gc, 
		   0, MSGLINE, text_size.r_width, text_height);
    XSetForeground(display, text_gc, foreground);

    mesg_length = 0;
    mesg_buf[0] = '\0';
}


/*
 * Consume user text parameter - don't touch message area.
 */
TxKillLine()
{
    if (text_length > 0) {
	XGCValues gcValues;
    
	gcValues.foreground = background;
	gcValues.function = GXcopy;
	XChangeGC(display, text_gc, GCForeground | GCFunction, &gcValues);
	XFillRectangle(display, text_sw, text_gc, 
		       TEXT_LEFT, 0, text_size.r_width, text_height);
	XSetForeground(display, text_gc, foreground);
	text_length = 0;
	text_next = 0;
	text_buf[0] = '\0';
	text_cursor(GXcopy);
    }
}
