/* 
 * pmmenu.c --
 *
 *	This file implements pull-down menus using the facilities of the
 *	X window package.
 *
 * Copyright (C) 1986, 1988 Regents of the University of California.
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */

#ifndef lint
static char rcsid[] = "$Header: /a/X/src/cmds/gremlin/RCS/xmenu.c,v 1.2 88/11/12 12:00:40 nelson Exp $ SPRITE (Berkeley)";
#endif not lint

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <strings.h>
#include "sx.h"
#include "sxInt.h"
#include "menu.h"

/*
 * Imports from library:
 */

char *malloc();

/*
 * Constants for displaying menus:
 *
 * MARGIN		Vertical spacing to leave around entries in menus.
 */

#define MARGIN 1

/*
 * The include's below are used to define the cursor used when a menu
 * is active.
 */

#include "icons/star"
#include "icons/starMask"

static Cursor cursor;

/*
 * If a menu is actually pulled down, the information below is used
 * to keep track of information about it.
 */

static int menuDown = 0;		/* 1 means a menu is actually
					 * pulled down. */
static int selectedEntry = -1;		/* Index of selected entry, or -1 if
					 * no entry is selected. */

static int init = 0;			/* 1 means initialized OK. */

/*
 * Forward references:
 */

extern void	ComputeMenuLayout();
extern void	MenuEntryDisplay();
extern int	MenuEventProc();
extern void	MenuInit();


/*
 *----------------------------------------------------------------------
 *
 * Menu_Create --
 *
 *	Create a new pulldown menu.
 *
 * Results:
 *	The pixmap window.
 *
 * Side effects:
 *	A new pulldown menu is created and displayed.  It will have
 *	numEntries entries, as described by the entries in the entries[]
 *	array.
 *
 *	Later, when buttons are pressed over the entries in the pulldown
 *	menu, the client procedures indicated in entries[] will be called
 *	as follows:
 *
 *	void
 *	proc(clientData, entry, menuWindow)
 *	    ClientData clientData;
 *	    int entry;
 *	    Window menuWindow;
 *	{
 *	}
 *	
 *	The clientData parameter is the one passed in the entries[]
 *	element that was buttoned, entry is the index of the entry
 *	that was invoked, and menuWindow indicates the menu (it's
 *	the same as the value returned by this procedure).
 *
 *----------------------------------------------------------------------
 */
#define MotionMask ( \
	PointerMotionMask | Button1MotionMask | \
	Button2MotionMask | Button3MotionMask | Button4MotionMask | \
	Button5MotionMask | ButtonMotionMask )
#define PropagateMask ( \
	KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask | \
	MotionMask )
Menu *
Menu_Create(display, parent, numEntries, entries, fontPtr,
	     foreground, background)
    Display	*display;		/* Connection to X server. */
    Window	parent;			/* Parent window for the menu. */
    int		numEntries;		/* Number of entries in menu.  Must be
					 * less than or equal to
					 * SX_MAX_MENU_ENTRIES. */
    Sx_MenuEntry entries[];		/* Entries:  must have numEntries
					 * values. */
    XFontStruct	*fontPtr;		/* Font to use for displaying menu.
					 * NULL means use default font. */
    unsigned long foreground;		/* Foreground color to use to display
					 * menu title and menu entries. */
    unsigned long background;		/* Background color to use for title.
					 * Each menu entry indicates its own
					 * background color. */
{
    register Menu		*menuPtr;
    register MenuEntry		*entryPtr;
    register Sx_MenuEntry 	*paramPtr;
    int 			i;
    XGCValues			gcValues;
    XSetWindowAttributes	atts;

    if (!init) {
	MenuInit(display);
    }

    if (fontPtr == NULL) {
	fontPtr = Sx_GetDefaultFont(display);
    }

    menuPtr = (Menu *) malloc(sizeof(Menu));
    menuPtr->display = display;

    atts.background_pixmap = None;
    atts.do_not_propagate_mask = PropagateMask ;
    atts.save_under = True;
    atts.override_redirect = True;

    menuPtr->menuWindow = XCreateWindow(display, parent, 0, 0, 1, 1, 0,
	    (int)CopyFromParent, InputOutput, (Visual *)CopyFromParent,
	    CWBackPixmap|CWDontPropagate|CWSaveUnder|CWOverrideRedirect,
	    &atts);
    (void) Sx_HandlerCreate(display, menuPtr->menuWindow, ExposureMask,
	    MenuEventProc, (ClientData) menuPtr);
    menuPtr->fontPtr = fontPtr;
    menuPtr->foreground = foreground;
    menuPtr->background = background;
    gcValues.font = fontPtr->fid;
    menuPtr->gc = XCreateGC(display, menuPtr->menuWindow, GCFont, &gcValues);
    if (numEntries > SX_MAX_MENU_ENTRIES) {
	numEntries = SX_MAX_MENU_ENTRIES;
    }
    menuPtr->numEntries = numEntries;
    for (i = 0, paramPtr = entries; i < numEntries; i++, paramPtr++) {
	entryPtr = (MenuEntry *) malloc(sizeof(MenuEntry));
	menuPtr->entryPtrs[i] = entryPtr;
	if (paramPtr->leftText == NULL) {
	    entryPtr->leftText = NULL;
	    entryPtr->leftSize = 0;
	} else {
	    entryPtr->leftSize = strlen(paramPtr->leftText);
	    entryPtr->leftText = (char *)
		    malloc((unsigned) (entryPtr->leftSize + 1));
	    (void) strcpy(entryPtr->leftText, paramPtr->leftText);
	}
	if (paramPtr->centerText == NULL) {
	    entryPtr->centerText = NULL;
	    entryPtr->centerSize = 0;
	} else {
	    entryPtr->centerSize = strlen(paramPtr->centerText);
	    entryPtr->centerText = (char *)
		    malloc((unsigned) (entryPtr->centerSize + 1));
	    (void) strcpy(entryPtr->centerText, paramPtr->centerText);
	}
	if (paramPtr->rightText == NULL) {
	    entryPtr->rightText = NULL;
	    entryPtr->rightSize = 0;
	} else {
	    entryPtr->rightSize = strlen(paramPtr->rightText);
	    entryPtr->rightText = (char *)
		    malloc((unsigned) (entryPtr->rightSize + 1));
	    (void) strcpy(entryPtr->rightText, paramPtr->rightText);
	}
	if (paramPtr->background != -1) {
	    entryPtr->background = paramPtr->background;
	} else {
	    entryPtr->background = menuPtr->background;
	}
	entryPtr->proc = paramPtr->proc;
	entryPtr->clientData = paramPtr->clientData;
    }
    ComputeMenuLayout(menuPtr);
    return(menuPtr);
}


/*
 *----------------------------------------------------------------------
 *
 * MenuInit --
 *
 *	Performs various once-only initialization functions.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Random stuff gets initialized.  See code for details.
 *
 *----------------------------------------------------------------------
 */

static void
MenuInit(display)
    Display *display;		/* Connection to X server. */
{
    static XColor black = {0, 0, 0, 0, 0};
    static XColor white = {0, ~0, ~0, ~0, 0};
    Pixmap source, mask;
    Window root;

    root = RootWindow(display, DefaultScreen(display));
    source = XCreateBitmapFromData(display, root, star_bits,
	    star_width, star_height);
    mask = XCreateBitmapFromData(display, root, starMask_bits,
	    starMask_width, starMask_height);
    cursor = XCreatePixmapCursor(display, source, mask, &black, &white,
	    star_x_hot, star_y_hot);
    XFreePixmap(display, source);
    XFreePixmap(display, mask);

    init = 1;
}


/*
 *----------------------------------------------------------------------
 *
 * ComputeMenuLayout --
 *
 *	This procedure scans all the entries in a menu and computes
 *	the overall size of the menu as well as where to position
 *	each text field of each entry.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Entries in the menu get modified to reflect the new layout.
 *	Menu->menuWindow also gets modified in size to reflect
 *	the new layout.
 *
 *----------------------------------------------------------------------
 */

static void
ComputeMenuLayout(menuPtr)
    register Menu *menuPtr;		/* Menu to resize. */
{
    register MenuEntry *entryPtr;
    int leftMax, centerMax, rightMax;
    int i, length, space;

    /*
     * Compute the height of the menu.
     */

    menuPtr->menuHeight = menuPtr->numEntries*(menuPtr->fontPtr->ascent
	    + menuPtr->fontPtr->descent + 2*MARGIN) + SHADOW_TOP
	    + SHADOW_BOTTOM;

    /*
     * Compute the maximum width of any entry for each column, allowing
     * an extra one-space pad on each side of each entry.
     */
    
    leftMax = centerMax = rightMax = 0;
    space = XTextWidth(menuPtr->fontPtr, "m", 1);
    for (i = 0; i < menuPtr->numEntries; i++) {
	entryPtr = menuPtr->entryPtrs[i];
	if (entryPtr->leftText != NULL) {
	    length = XTextWidth(menuPtr->fontPtr, entryPtr->leftText,
		    strlen(entryPtr->leftText)) + 2*space;
	    if (length > leftMax) {
		leftMax = length;
	    }
	}
	if (entryPtr->centerText != NULL) {
	    length = XTextWidth(menuPtr->fontPtr, entryPtr->centerText,
		    strlen(entryPtr->centerText)) + 2*space;
	    if (length > centerMax) {
		centerMax = length;
	    }
	    entryPtr->centerPos = length;   /* Save to avoid recomputation. */
	}
	if (entryPtr->rightText != NULL) {
	    length = XTextWidth(menuPtr->fontPtr, entryPtr->rightText,
		    strlen(entryPtr->rightText)) + 2*space;
	    if (length > rightMax) {
		rightMax = length;
	    }
	    entryPtr->rightPos = length;   /* Save to avoid recomputation. */
	}
    }

    /*
     * Size the window so that none of the columns overlap (be sure
     * to leave space for the fancy border).  Then go back and fill
     * in the exact drawing position for each string.
     */
    
    menuPtr->menuWidth = leftMax + centerMax + rightMax +
	    SHADOW_LEFT + SHADOW_RIGHT;
    XResizeWindow(menuPtr->display, menuPtr->menuWindow, 
		 (unsigned)menuPtr->menuWidth,
		 (unsigned)menuPtr->menuHeight);
    
    for (i = 0; i < menuPtr->numEntries; i++) {
	entryPtr = menuPtr->entryPtrs[i];
	entryPtr->leftPos = SHADOW_LEFT + space;
	entryPtr->centerPos = SHADOW_LEFT + leftMax + space
		+ (centerMax - entryPtr->centerPos)/2;
	entryPtr->rightPos = menuPtr->menuWidth + space - entryPtr->rightPos
		- SHADOW_RIGHT;
    }
}

/*
 *----------------------------------------------------------------------
 *
 * MenuEntryDisplay --
 *
 *	This procedure is called to display one entry of a menu.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The entry is displayed in its menu.
 *
 *----------------------------------------------------------------------
 */

static void
MenuEntryDisplay(menuPtr, index)
    register Menu *menuPtr;	/* Menu containing entry to display. */
    int index;			/* Index of entry within menu. */
{
    register MenuEntry *entryPtr;
    unsigned long background, foreground;
    int y;
    unsigned height;

    entryPtr = menuPtr->entryPtrs[index];
    if (index == selectedEntry) {
	background = menuPtr->foreground;
	foreground = entryPtr->background;
    } else {
	background = entryPtr->background;
	foreground = menuPtr->foreground;
    }
    height = menuPtr->fontPtr->ascent + menuPtr->fontPtr->descent + 2*MARGIN;
    y = index*height + SHADOW_LEFT;
    XSetForeground(menuPtr->display, menuPtr->gc, background);
    XFillRectangle(menuPtr->display, menuPtr->menuWindow, menuPtr->gc,
	    SHADOW_LEFT, y, 
	    (unsigned)menuPtr->menuWidth - (SHADOW_LEFT + SHADOW_RIGHT),height);
    XSetForeground(menuPtr->display, menuPtr->gc, foreground);
    y += MARGIN + menuPtr->fontPtr->ascent - 1;
    if (entryPtr->leftText != NULL) {
	XDrawString(menuPtr->display, menuPtr->menuWindow, menuPtr->gc,
		entryPtr->leftPos, y, entryPtr->leftText, entryPtr->leftSize);
    }
    if (entryPtr->centerText != NULL) {
	XDrawString(menuPtr->display, menuPtr->menuWindow, menuPtr->gc,
		entryPtr->centerPos, y, entryPtr->centerText,
		entryPtr->centerSize);
    }
    if (entryPtr->rightText != NULL) {
	XDrawString(menuPtr->display, menuPtr->menuWindow, menuPtr->gc,
		entryPtr->rightPos, y, entryPtr->rightText,
		entryPtr->rightSize);
    }
}

/*
 *----------------------------------------------------------------------
 *
 * MenuEventProc --
 *
 *	This procedure is called by a dispatcher when events occur
 *	for a menu window.  This can only happen when the menu is
 *	being displayed, which means that a mouse button was pressed
 *	over a menu title and hasn't been released yet.  When this
 *	procedure is called, the mouse has been grabbed for the menu
 *	window.
 *
 * Results:
 *	1 if selected an entry in the menu, 0 otherwise.
 *
 * Side effects:
 *	This procedure tracks the motion of the mouse and button
 *	releases.  It highlights the menu entry underneath the mouse
 *	(if the entry is enabled) and calls a client-supplied
 *	procedure if the mouse button is released over an entry.
 *	When the buttons are released, the menu is unmapped and
 *	mouse-grabbing stops.
 *
 *----------------------------------------------------------------------
 */

static int
MenuEventProc(menuPtr, eventPtr, clientDataPtr)
    register Menu *menuPtr;	/* Menu that is currently active. */
    XEvent *eventPtr;		/* Describes what just happened. */
    ClientData *clientDataPtr;	/* Ptr to menu entry that was selected. */
{
    if (eventPtr->type == Expose) {
	int i;
	unsigned width, height;

	/*
	 * In a string of expose events each giving a small region,
	 * ignore all but the last.
	 */

	if (eventPtr->xexpose.count > 0) {
	    return(0);
	}

	width = menuPtr->menuWidth - SHADOW_LEFT - SHADOW_RIGHT;
	height = menuPtr->menuHeight - SHADOW_TOP - SHADOW_BOTTOM;
    
	XSetForeground(menuPtr->display, menuPtr->gc, menuPtr->background);
	XFillRectangle(menuPtr->display, menuPtr->menuWindow, menuPtr->gc,
		0, 0, width, height);
	XSetForeground(menuPtr->display, menuPtr->gc, menuPtr->foreground);
	SxDrawShadow(menuPtr->display, menuPtr->menuWindow, menuPtr->gc,
		0, 0, width, height);
    
	for (i = 0; i < menuPtr->numEntries; i++) {
	    MenuEntryDisplay(menuPtr, i);
	}
	return(0);
    } else if (eventPtr->type == MotionNotify) {
	int index;

	/*
	 * Find the entry under the mouse (if any).  If it's enabled,
	 * then select it and unselect the previous one, if any.
	 */
	
	index = eventPtr->xmotion.y/(menuPtr->fontPtr->ascent
		+ menuPtr->fontPtr->descent + 2*MARGIN);
	if ((eventPtr->xmotion.y < 0) || (index >= menuPtr->numEntries)
		|| (eventPtr->xmotion.x < SHADOW_LEFT)
	    || (eventPtr->xmotion.x > (menuPtr->menuWidth - SHADOW_RIGHT))) {
	    index = -1;
	}
	if (selectedEntry != index) {
	    int prev = selectedEntry;

	    selectedEntry = index;
	    if (index >= 0) {
		MenuEntryDisplay(menuPtr, index);
	    }
	    if (prev >= 0) {
		MenuEntryDisplay(menuPtr, prev);
	    }
	}
	return(0);
    } else if (eventPtr->type == ButtonRelease) {
	static int masks[] = {
	    0,
	    Button2Mask|Button3Mask|Button4Mask|Button5Mask,
	    Button1Mask|Button3Mask|Button4Mask|Button5Mask,
	    Button1Mask|Button2Mask|Button4Mask|Button5Mask,
	    Button1Mask|Button2Mask|Button3Mask|Button5Mask,
	    Button1Mask|Button2Mask|Button3Mask|Button4Mask
	};

	/*
	 * When all of the buttons have been released, then unmap
	 * the menu.  If an entry was selected at the time, flash it
	 * before erasing the menu, then call its action procedure
	 * after erasing the menu.  The computation of whether all
	 * buttons have been released is a little tricky because the
	 * "detail" value doesn't take into account the current event.
	 */

	if ((eventPtr->xbutton.state & masks[eventPtr->xbutton.button]) == 0) {
	    if (selectedEntry >= 0) {
		int i, index;

		index = selectedEntry;
		for (i = 0; i < 2; i++) {
		    selectedEntry = -1;
		    MenuEntryDisplay(menuPtr, index);
		    SxFlashWait(menuPtr->display);
		    selectedEntry = index;
		    MenuEntryDisplay(menuPtr, index);
		    SxFlashWait(menuPtr->display);
		}
	    }
	    menuDown = 0;
	    XUngrabPointer(menuPtr->display, CurrentTime);
	    XUnmapWindow(menuPtr->display, menuPtr->menuWindow);
	    XFlush(menuPtr->display);

	    if (selectedEntry >= 0) {
		*clientDataPtr = menuPtr->entryPtrs[selectedEntry]->clientData;
	    } else {
		*clientDataPtr = (ClientData)-1;
	    }
	    return(1);
	}
    }
    return(0);
}


/*
 *----------------------------------------------------------------------
 *
 * Menu_PullMenuDown --
 *
 *	This procedure does all the work of "pulling a menu down",
 *	i.e. making the menu visible on the screen.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The menu gets mapped onto the screen.
 *
 *----------------------------------------------------------------------
 */
ClientData
Menu_PullMenuDown(menuPtr, x, y)
    register Menu *menuPtr;	/* Menu to pull down.  There must not
				 * currently be a menu pulled down. */
    int		x, y;		/* X, Y coordinates of menu. */
{
    ClientData		clientData;
    XEvent		event;

    XMoveWindow(menuPtr->display, menuPtr->menuWindow, x, y);
    XMapRaised(menuPtr->display, menuPtr->menuWindow);
    if (XGrabPointer(menuPtr->display, menuPtr->menuWindow, False,
	    (unsigned)(ButtonReleaseMask|PointerMotionMask), GrabModeAsync,
	    GrabModeAsync, None, cursor, CurrentTime) != 0) {
	Sx_Panic(menuPtr->display, "Pull-down menu couldn't grab pointer.");
    }
    selectedEntry = -1;
    menuDown = 1;

    while (1) {
	XNextEvent(menuPtr->display, &event);
	if (event.xany.window == menuPtr->menuWindow) {
	    if (MenuEventProc(menuPtr, &event, &clientData)) {
		return(clientData);
	    }
	}
/*
	if ((event.type == ButtonPress) || (event.type == ButtonRelease) ||
	    (event.type == KeyPress) || (event.type == KeyRelease)) {
	    XAllowEvents(menuPtr->display, AsyncPointer, CurrentTime);
	    XAllowEvents(menuPtr->display, AsyncKeyboard, CurrentTime);
	}
*/
    }
}
